/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||               Mesh3D creates media-infomation in each Yee's cell.   Jan/30/2012 ||||||||||||
||||||||||||                Main and sub programs were coded by Tatsuya Usuki.               ||||||||||||
||||||||||||                            Program language is C99.                             ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                     Please let me know any bug that you find.                   ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                                         http://homepage2.nifty.com/Pass_path/   ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                                                  Last updated on Oct 28, 2014   ||||||||||||
||||||||||||---------------------------------------------------------------------------------||||||||||||
||||||||||||     References:                                                                 ||||||||||||
||||||||||||     1) "WaveguideModeNum_ja2.pdf"                                               ||||||||||||
||||||||||||     2) "WageguideModeNum_v3.pdf"                                                ||||||||||||
||||||||||||     3) "Wageguide_Simulation_v1.pdf"                                            ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
#include <tgmath.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
//#include "constant.h"
#define BUFSIZE 1024 /* buffer for fgets */
#define Pi 3.141592653589793238462643 /* circular constant */
//#define Pi2 6.283185307179586476925286
//
//  main ---- input_file1 
//        |                 
//        |-- input_fileM
//        |                 
//         -- input_file3
//        |                 
//         -- input_file2
//        |                 
//         -- output_file
//
void input_file1(FILE *fp_i1, int L[3]);
void input_fileM(FILE *fp_i1, int L, double x[L*2], double dxdu[L*2]);
void input_file3(FILE *fp_i3, char Yee[], int *division_depth);
int input_N_str(FILE *fp_i2);
void input_file2(FILE *fp_i2, const int N_str, int media_no[N_str], int shape_str[N_str], 
		double shape_length[N_str*6], double origin_str[N_str*3], double array_step[N_str*9], int n_step[N_str*6]);
void output_file(char **argv, const char Yee[], const int division_depth, const int L[3], 
		const double x[L[0]*2], const double y[L[1]*2], const double z[L[2]*2], 
		const double dxdu[L[0]*2], const double dydv[L[1]*2], const double dzdw[L[2]*2], 
		const int N_str, const int media_no[N_str], 
		const int shape_str[N_str], const double shape_length[N_str*6], 
		const double origin_str[N_str*3], const double array_step[N_str*9], const int n_step[N_str*6]);
//
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                             Main routine of YeeCells.                           ||||||||||||
||||||||||||                            Last updated on Oct 22, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
int main(int argc, char **argv)
{
	{
		time_t timer_ini = time(0);
		fprintf(stderr,"# Start time of Yee's lattice creation = %s\n", ctime(&timer_ini));
	}

	if(argc != 4) {
		fprintf(stderr,"error: number of files \n");
		exit(1);
	}
//---------- begin reading Yee lattice information ------------
	FILE *fp_i1;
	fp_i1 = fopen(argv[1],"r");
	if (fp_i1 == NULL){
		fprintf(stderr,"open error!: open input-file1!\n");
		exit(1);
	}
	fprintf(stderr,"Mesh data file (the 1st input file): %s\n",argv[1]);

	int L[3]; //division number: Lu = L, Lv = M, Lw = N
	
	input_file1(fp_i1, L); // inputting parameters L, M and N from an inputfile 1
	
	double x[L[0]*2+1], dxdu[L[0]*2];
	input_fileM(fp_i1, L[0], x, dxdu);
	
	double y[L[1]*2+1], dydv[L[1]*2];
	input_fileM(fp_i1, L[1], y, dydv);
	
	double z[L[2]*2+1], dzdw[L[2]*2];
	input_fileM(fp_i1, L[2], z, dzdw);
	
	if(fclose(fp_i1) != 0) {
		fprintf(stderr,"fclose error after input_file1\n");
		exit(1);
	}
	
	for(int j0 = 0 ; j0 < 3 ; j0++) { 
		if (j0 == 0) fprintf(stderr,"\nL, x0, x1, dxdu0, dxdu1\n");
		if (j0 == 1) fprintf(stderr,"\nM, y0, y1, dydv0, dydv1\n");
		if (j0 == 2) fprintf(stderr,"\nN, z0, z1, dzdw0, dzdw1\n");
		for(int k0 = 0 ; k0 < L[j0] ; k0++) { 
			if (j0 == 0) fprintf(stderr,"%d, %.5E, %.5E, %.5E, %.5E\n", k0, x[k0*2+0], x[k0*2+1], dxdu[k0*2+0], dxdu[k0*2+1]);
			if (j0 == 1) fprintf(stderr,"%d, %.5E, %.5E, %.5E, %.5E\n", k0, y[k0*2+0], y[k0*2+1], dydv[k0*2+0], dydv[k0*2+1]);
			if (j0 == 2) fprintf(stderr,"%d, %.5E, %.5E, %.5E, %.5E\n", k0, z[k0*2+0], z[k0*2+1], dzdw[k0*2+0], dzdw[k0*2+1]);
		}
		if (j0 == 0) fprintf(stderr,"%d, %.5E\n", L[j0], x[L[j0]*2]);
		if (j0 == 1) fprintf(stderr,"%d, %.5E\n", L[j0], y[L[j0]*2]);
		if (j0 == 2) fprintf(stderr,"%d, %.5E\n", L[j0], z[L[j0]*2]);
	}
//----------  end reading Yee lattive information  ------------

//---------- begin reading sub-mesh information and output file name ------------
	char Yee[BUFSIZE];
	FILE *fp_i3;
	fp_i3 = fopen(argv[3],"r");
	if (fp_i3 == NULL){
		fprintf(stderr,"open error!: open input-file3!\n");
		exit(1);
	}
	fprintf(stderr,"\nSub-mesh information (the 3rd input file): %s\n",argv[3]);
	int division_depth;
	input_file3(fp_i3, Yee, &division_depth);
	fprintf(stderr,"Yee's lattice output file `%s*.dat'\n", Yee);
	fprintf(stderr,"division_depth = %d\n", division_depth);
	if(fclose(fp_i3) != 0) {
		fprintf(stderr,"fclose error after input_file3\n");
		exit(1);
	}
//----------  end reading sub-mesh information and output file name  ------------

//---------- begin reading 3D-structure information ------------
	FILE *fp_i2;
	fp_i2 = fopen(argv[2],"r");
	if (fp_i2 == NULL){
		fprintf(stderr,"open error!: open input-file2!\n");
		exit(1);
	}
	fprintf(stderr,"\n3D-structure data file (the 2nd input file): %s\n",argv[2]);


	int N_str = input_N_str(fp_i2); //maximum of structure number
	rewind(fp_i2);
	
	fprintf(stderr,"N_str = %d", N_str);
	int shape_str[N_str], media_no[N_str];
	double shape_length[N_str*6], origin_str[N_str*3];
	double array_step[N_str*9];
	int n_step[N_str*6];

	input_file2(fp_i2, N_str, media_no, shape_str, shape_length, origin_str, array_step, n_step);
	if(fclose(fp_i2) != 0) {
		fprintf(stderr,"\nfclose error after input_file2\n");
		exit(1);
	}
	int Max_media_no = media_no[0];
	for(int j0 = 1 ; j0 < N_str ; j0++) {
		if(Max_media_no < media_no[j0]) Max_media_no = media_no[j0];
	}
	fprintf(stderr,", media_no < %d\n", Max_media_no + 1);
	for(int j0 = 0 ; j0 < N_str ; j0++) { 
		fprintf(stderr,"\nno_str = %d\n", j0);
		fprintf(stderr,"media_no = %d\n", media_no[j0]);
		if (shape_str[j0] < 0 || shape_str[j0] > 4) fprintf(stderr,"shape_str error!\n");
		if (shape_str[j0] == 0) fprintf(stderr,"shape_str = block\n");
		if (shape_str[j0] == 1) fprintf(stderr,"shape_str = cylinder\n");
		if (shape_str[j0] == 2) fprintf(stderr,"shape_str = prism\n");
		if (shape_str[j0] == 3) fprintf(stderr,"shape_str = pyramid\n");
		if (shape_str[j0] == 4) fprintf(stderr,"shape_str = spheroid\n");
		fprintf(stderr,"shape_length = (%.5E, %.5E, %.5E)\n", shape_length[j0*6+0], shape_length[j0*6+1], shape_length[j0*6+2]);
		fprintf(stderr,"shape_angle = (%.5E, %.5E, %.5E)\n", shape_length[j0*6+3], shape_length[j0*6+4], shape_length[j0*6+5]);
		fprintf(stderr,"origin_str = (%.5E, %.5E, %.5E)\n", origin_str[j0*3+0], origin_str[j0*3+1], origin_str[j0*3+2]);
		for(int i0 = 0 ; i0 < 3 ; i0++){
			fprintf(stderr,"Step %d:\n", i0+1);
			fprintf(stderr,"array_step = (%.5E, %.5E, %.5E)\n", 
				array_step[j0*9+0+i0*3], array_step[j0*9+1+i0*3], array_step[j0*9+2+i0*3]);
			fprintf(stderr,"n_step start = %d, finish = %d\n", n_step[j0*6+0+i0*2], n_step[j0*6+1+i0*2]);
		}
	}
//----------  end reading 3D-structure information  ------------
	
	output_file(argv, Yee, division_depth, L, 
		x, y, z, 
		dxdu, dydv, dzdw, 
		N_str, media_no, 
		shape_str, shape_length, 
		origin_str, array_step, n_step);

	{
		time_t timer_f = time(0);
		fprintf(stderr,"# Finish time of Yee's lattice creation = %s", ctime(&timer_f));
	}
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                              input_file1, input_fileM                           ||||||||||||
||||||||||||                            Last updated on Oct 27, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void input_file1(FILE *fp_i, int L[3])
{
	char buf[BUFSIZE];	// buffer for fgets
	while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %d %*[^=] %*[=] %d %*[^=] %*[=] %d", &L[0], &L[1], &L[2]) == 3) goto End_of_LMN;
	}
	fprintf(stderr,"No L,M,N are read @ input_file1!\n");
	exit(1);
	End_of_LMN:;

	if(L[0] < 1 || L[1] < 1 || L[2] < 1) {
		fprintf(stderr,"L,M,N input error!: L = %d, M = %d, N = %d \n", L[0], L[1], L[2]);
		exit(1);
	}
}
void input_fileM(FILE *fp_i, const int L, double x[L*2+1], double dxdu[L*2])
{
	char buf[BUFSIZE];	// buffer for fgets
	int i0 = 0;
	int i1;
	while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
		if(sscanf(buf,"%d %*[,] %lf  %*[,] %lf %*[,] %lf %*[,] %lf", 
			&i1, &x[i0*2+0], &x[i0*2+1], &dxdu[i0*2+0], &dxdu[i0*2+1]) == 5){
			if(i0 == i1) i0++;
			if(i0 == L) goto End_of_x_data;
		}
	}
	fprintf(stderr,"Full %d-data were not read @ input_fileM!\n",L);
	exit(1);
	End_of_x_data:;

	while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
		if(sscanf(buf,"%d %*[,] %lf", &i1, &x[2*L]) == 2){
			if(i1 == L) goto End_of_x_max;
		}
	}
	fprintf(stderr,"The %d-th data were not read @ input_fileM!\n",L+1);
	exit(1);
	End_of_x_max:;
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                    input_file3                                  ||||||||||||
||||||||||||                            Last updated on Oct 28, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void input_file3(FILE *fp_i3, char Yee[], int *division_depth)
{
	char buf[BUFSIZE];	// buffer for fgets

	while(fgets(buf, sizeof( buf ), fp_i3) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %s", Yee) == 1) goto End_of_Yee;
	}
	fprintf(stderr,"No Yee-name is read @ input_file3!\n");
	exit(1);
	End_of_Yee:;

	while(fgets(buf, sizeof( buf ), fp_i3) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %d", division_depth) == 1) goto End_of_division_depth;
	}
	fprintf(stderr,"No division_depth is read @ input_file3!\n");
	exit(1);
	End_of_division_depth:;

	if(*division_depth < 0 || *division_depth > 10) {
		fprintf(stderr,"Mesh input error!: division depth = %d ", *division_depth);
		fprintf(stderr,"division depth is positive number.\n");//20141028 revised
		exit(1);
	}
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                    input_N_str                                  ||||||||||||
||||||||||||                            Last updated on Sep 08, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
int input_N_str(FILE *fp_i2)
{
	char buf[16], command[8];	// buffer for fgets, command for seeking "begin" and "end".
	int n_str0 = 0;
	while(fgets(buf, sizeof( buf ), fp_i2) != NULL) { 
		if(sscanf(buf,"%s", command) != EOF){
			if (strcmp(command, "begin") == 0) {
				while(fgets(buf, sizeof( buf ), fp_i2) != NULL) { 
					if(sscanf(buf,"%s", command) != EOF){
						if ( strcmp(command, "end") == 0) {
							n_str0 += 1;
							goto NEXT_step;
						}
					}
				}
			}
		}
		NEXT_step:;
	}
	return(n_str0);
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                    input_file2                                  ||||||||||||
||||||||||||                            Last updated on Sep 08, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
// void input_file2 ----- int input_media            
//                    |                                                
//                     -- int input_shape ------                         
//                    |                         |-- double ScaleUnit 
//                     -- int input_origin -----|                    
//                    |                         |                    
//                     -- int input_array ------                     
//
int input_media(const char buf[], const int n_str0, int media_no[]);
int input_shape(const char buf[], const int n_str0, int shape_str[], double shape_length[]);
int input_origin(const char buf[], const int n_str0, double origin_str[]);
int input_array(const char buf[], const int n_str0, const int k_array, double array_step[], int n_step[]);
void input_file2(FILE *fp_i2, const int N_str, int media_no[N_str], int shape_str[N_str], 
		double shape_length[N_str*6], double origin_str[N_str*3], double array_step[N_str*9], int n_step[N_str*6])
{
// N_str                  : structure quantity (int).
// media_no[N_str]     : media number (int).
// shape_str[N_str]       : 'block' == 0, 'cylinder' == 1, 'prism' == 2, 'pyramid' == 3, 'spheroid' == 4, (int).
// shaple_length[N_str*6]:  shape_length[][0] = A, shape_length[][1] = B, shape_length[][2] = C (meter, double),
//                           structure rotation: shape_length[][5] = theta_x, shape_length[][6] = theta_y, shape_length[][6] = theta_z (radian, double).
// origin_str[N_str*3]   :  an origin of (x, y, z) coordinates for a structure.(meter, double).
// array_step[N_str*9]   :  step lengths for (1st stage, 2nd stage, 3rd stage) and (meter, double),
//                                        x-direction (array_step[][0], array_step[][3], array_step[][6]), 
//                                        y-direction (array_step[][1], array_step[][4], array_step[][7]), 
//                                        z-direction (array_step[][2], array_step[][5], array_step[][8]). 
// n_step[N_str*6]       :  step number for (1st stage, 2nd stage, 3rd stage), (int),
//                                        initial number (n_step[][0], n_step[][2], n_step[][4]), 
//                                         final  number (n_step[][1], n_step[][3], n_step[][5]). 
	char buf[BUFSIZE], command[16];	// buffer for fgets
	int n_str0 = 0;
	for(int J0 = 0 ; J0 < N_str ; J0++) { 
		while(fgets(buf, sizeof( buf ), fp_i2) != NULL) { 
			if(sscanf(buf,"%s", command) != EOF){
				if (strcmp(command, "begin") == 0) {
					int media_count = 0;// Initialization
					int shape_count = 0;
					int origin_count = 0;
					int array_count = 0;
					for(int k_array = 0 ; k_array < 3 ; k_array++) { 
						array_step[n_str0*9+  k_array*3] = 0.;
						array_step[n_str0*9+1+k_array*3] = 0.;
						array_step[n_str0*9+2+k_array*3] = 0.;
						n_step[n_str0*6+  k_array*2] = 0;
						n_step[n_str0*6+1+k_array*2] = 0;
					}// End of initialization
					while(fgets(buf, sizeof( buf ), fp_i2) != NULL) { 
						if(sscanf(buf,"%s", command) != EOF){
							if (strcmp(command, "media") == 0) media_count += input_media(buf, n_str0, media_no);
							if (strcmp(command, "shape") == 0) shape_count += input_shape(buf, n_str0, shape_str, shape_length);
							if (strcmp(command, "origin") == 0) origin_count += input_origin(buf, n_str0, origin_str);
							if (strcmp(command, "array") == 0) 
								array_count += input_array(buf, n_str0, array_count, array_step, n_step);
							if (strcmp(command, "end") == 0) {
								if(media_count != 1 || shape_count != 1 || origin_count != 1 || array_count > 3){
									fprintf(stderr,"end error @ input_file2!: media_count = %d, shape_count = %d,", 
										media_count, shape_count);
									fprintf(stderr," origin_count = %d, array_count= %d\n", origin_count, array_count);
									exit(1);
								}
								else{
									n_str0 += 1;
									if(n_str0 > N_str){
										fprintf(stderr,"Error @ input_file2!: n_str0 > N_str, n_str0 = %d,", n_str0);
										exit(1);
									}
									goto NEXT_step;
								}
							}
						}
					}
				}
			}
		}
		NEXT_step:;
	}
	if(n_str0 != N_str){
		fprintf(stderr,"Error @ input_file2!: n_str0 != N_str, n_str0 = %d,", n_str0);
		exit(1);
	}
}
int input_media(const char buf[], const int n_str0, int media_no[])
// media_no[N_str]     : media number (int).
{
	if(sscanf(buf,"%*[^=] %*[=] %d", &media_no[n_str0]) != 1){
		fprintf(stderr,"Error in input_media that cannot read media_no @ n_str0 = %d\n", n_str0);
		return(0);
	}else{
		return(1);
	}
}
double ScaleUnit(char x[])
{
	double scale = -1.;
	if (strncmp(x, "m", 1) == 0) scale = 1e-0;// order of m, mm, micron is important!
	if (strncmp(x, "mm", 2) == 0) scale = 1e-3;// order of m, mm, micron is important!
	if (strncmp(x, "micron", 6) == 0) scale = 1e-6;// order of m, mm, micron is important!
	
	if (strncmp(x, "km", 2) == 0) scale = 1e3;
	if (strncmp(x, "cm", 2) == 0) scale = 1e-2;
	
	if (strncmp(x, "um", 2) == 0) scale = 1e-6;
	if (strncmp(x, "nm", 2) == 0) scale = 1e-9;
	if (strncmp(x, "deg", 3) == 0) scale = Pi/180.;
//	if (strncmp(x, "degree", 6) == 0) scale = Pi/180.;
	if (strncmp(x, "rad", 3) == 0) scale = 1.;
//	if (strncmp(x, "radian",6) == 0) scale = 1.;
	if (scale < 0.) {
		fprintf(stderr,"ScaleUnit error!  x = %s\n", x);
		exit(1);
	}
	return(scale);
}
double ScaleUnit(char x[]);// unit variation: km, m, cm, mm, micron, um, nm, deg, degree, rad, radian.
int input_shape(const char buf[], const int n_str0, int shape_str[], double shape_length[])
{
// shape_str[N_str]       : 'block' == 0, 'cylinder' == 1, 'prism' == 2, 'pyramid' == 3, 'spheroid' == 4, (int).
// shaple_length[N_str*6]:  shape_length[][0] = A, shape_length[][1] = B, shape_length[][2] = C (meter, double),
//                           structure rotation: shape_length[][3] = theta_x, shape_length[][4] = theta_y, shape_length[][5] = theta_z (radian, double).
	int count = 0;
	char command[BUFSIZE], shape[10], A[BUFSIZE], B[BUFSIZE], C[BUFSIZE], x[BUFSIZE], y[BUFSIZE], z[BUFSIZE];
	if(sscanf(buf,"%s %s %*[^=] %*[=] %lf %s %*[^=] %*[=] %lf %s %*[^=] %*[=] %lf %s %*[^=] %*[=] %lf %s %*[^=] %*[=] %lf %s %*[^=] %*[=] %lf %s", 
		command, shape, 
		&shape_length[n_str0*6+0],A, &shape_length[n_str0*6+1],B, &shape_length[n_str0*6+2],C, 
		&shape_length[n_str0*6+3],x, &shape_length[n_str0*6+4],y, &shape_length[n_str0*6+5],z) != 14){
		fprintf(stderr,"command, shape error! `%s'\n", buf);
		exit(1);
	}
	shape_str[n_str0] = -1;
	if (strcmp(shape, "block") == 0)  shape_str[n_str0] = 0;
	if (strcmp(shape, "cylinder") == 0)  shape_str[n_str0] = 1;
	if (strcmp(shape, "prism") == 0)  shape_str[n_str0] = 2;
	if (strcmp(shape, "pyramid") == 0)  shape_str[n_str0] = 3;
	if (strcmp(shape, "spheroid") == 0)  shape_str[n_str0] = 4;
	shape_length[n_str0*6+0] *= ScaleUnit(A);
	shape_length[n_str0*6+1] *= ScaleUnit(B);
	shape_length[n_str0*6+2] *= ScaleUnit(C);
	shape_length[n_str0*6+3] *= ScaleUnit(x);
	shape_length[n_str0*6+4] *= ScaleUnit(y);
	shape_length[n_str0*6+5] *= ScaleUnit(z);
	if(shape_length[n_str0*6+0] < 0. || shape_length[n_str0*6+1] < 0. || shape_length[n_str0*6+2] < 0.) {
		fprintf(stderr,"command, shape error!  Parameter is negative. `%s'\n", buf);
		exit(1);
	}
	count = 1;
	return(count);
}
int input_origin(const char buf[], const int n_str0, double origin_str[])
{
// origin_str[N_str*3]   :  an origin of (x, y, z) coordinates for a structure.(meter, double).
	int count = 0;
	char command[BUFSIZE], x[BUFSIZE], y[BUFSIZE], z[BUFSIZE];
	if(sscanf(buf,"%s %*[^=] %*[=] %lf %s %*[^=] %*[=] %lf %s %*[^=] %*[=] %lf %s", command, &origin_str[n_str0*3+0],x, &origin_str[n_str0*3+1],y, &origin_str[n_str0*3+2],z) != 7){
		fprintf(stderr,"command, shape error! `%s'\n", buf);
		exit(1);
	}
	origin_str[n_str0*3+0] *= ScaleUnit(x);
	origin_str[n_str0*3+1] *= ScaleUnit(y);
	origin_str[n_str0*3+2] *= ScaleUnit(z);
	count = 1;
	return(count);
}
int input_array(const char buf[], const int n_str0, const int k_array, double array_step[], int n_step[])
{
// array_step[N_str*9]   :  step lengths for (1st stage, 2nd stage, 3rd stage) and (meter, double),
//                                        x-direction (array_step[][0], array_step[][3], array_step[][6]), 
//                                        y-direction (array_step[][1], array_step[][4], array_step[][7]), 
//                                        z-direction (array_step[][2], array_step[][5], array_step[][8]). 
// n_step[N_str*6]       :  step number for (1st stage, 2nd stage, 3rd stage), (int),
//                                        initial number (n_step[][0], n_step[][2], n_step[][4]), 
//                                         final  number (n_step[][1], n_step[][3], n_step[][5]). 
	int count = 0;
	char command[BUFSIZE], x[BUFSIZE], y[BUFSIZE], z[BUFSIZE];
	if(sscanf(buf,"%s %*[^=] %*[=] %lf %s %*[^=] %*[=] %lf %s %*[^=] %*[=] %lf %s  %*[^=] %*[=] %d  %*[^=] %*[=] %d", command, &array_step[n_str0*9+k_array*3], x, &array_step[n_str0*9+1+k_array*3], y, &array_step[n_str0*9+2+k_array*3], z, &n_step[n_str0*6+k_array*2], &n_step[n_str0*6+1+k_array*2]) != 9){
		fprintf(stderr,"command, array error! `%s'\n", buf);
		exit(1);
	}
	array_step[n_str0*9+  k_array*3] *= ScaleUnit(x);
	array_step[n_str0*9+1+k_array*3] *= ScaleUnit(y);
	array_step[n_str0*9+2+k_array*3] *= ScaleUnit(z);
	count = 1;
	return(count);
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                    output_file                                  ||||||||||||
||||||||||||                            Last updated on Oct 28, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
//
//              void output_file ------------------- 
//                                 |                |
//                                 |                |-- media_f
//                                 |                |
//                                  -- est_volume --             
//                                   (recursive use)
//
int media_f(const double X[3], const int N_str, const int media_no[N_str], const int shape_str[N_str], const double shape_length[N_str*6], const double origin_str[N_str*3], const double array_step[N_str*9], const int n_step[N_str*6]);
void est_volume(int media_p[8*8], double media_r[8*8], int D_depth[2], const double XXX[9], const int N_str, const int media_no[N_str], const int shape_str[N_str], const double shape_length[N_str*6], const double origin_str[N_str*3], const double array_step[N_str*9], const int n_step[N_str*6]);
void output_file(char **argv, const char Yee[], const int division_depth, const int L[3], const double x[L[0]*2+1], const double y[L[1]*2+1], const double z[L[2]*2+1], const double dxdu[L[0]*2], const double dydv[L[1]*2], const double dzdw[L[2]*2], const int N_str, const int media_no[N_str], const int shape_str[N_str], const double shape_length[N_str*6], const double origin_str[N_str*3], const double array_step[N_str*9], const int n_step[N_str*6])
{
// This 'output_file' creates a set of 3D files for optical media.
// Yee[]       : an output file name (char).
// division_depth: division_depth means depth of recursion for est_volume (int).
// L[3]        : space size of periodic (u, v, w) coordinates (int).
// x[L[0]*2+1],    y[L[1]*2+1],    z[L[2]*2+1]   : sub-lattice position in (x, y, z) coordinates (meter, double).
// dxdu[L[0]*2], dydv[L[1]*2], dzdw[L[2]*2]: derivation of (x, y, z) by (u, v, w) (double).
// N_str                  : structure quantity (int).
// media_no[N_str]     : media number (int).
// shape_str[N_str]       : 'block' == 0, 'cylinder' == 1, 'prism' == 2, 'pyramid' == 3, 'spheroid' == 4, (int).
// shaple_length[N_str*6]:  shape_length[][0] = A, shape_length[][1] = B, shape_length[][2] = C (meter, double),
//                           structure rotation: shape_length[][3] = theta_x, shape_length[][4] = theta_y, shape_length[][5] = theta_z (radian, double).
// origin_str[N_str*3]   :  an origin of (x, y, z) coordinates for a structure.(meter, double).
// array_step[N_str*9]   :  step lengths for (1st stage, 2nd stage, 3rd stage) and (meter, double),
//                                        x-direction (array_step[][0], array_step[][3], array_step[][6]), 
//                                        y-direction (array_step[][1], array_step[][4], array_step[][7]), 
//                                        z-direction (array_step[][2], array_step[][5], array_step[][8]). 
// n_step[N_str*6]       :  step number for (1st stage, 2nd stage, 3rd stage), (int),
//                                        initial number (n_step[][0], n_step[][2], n_step[][4]), 
//                                         final  number (n_step[][1], n_step[][3], n_step[][5]). 
	int Max_media_no = media_no[0];
	for(int j0 = 1 ; j0 < N_str ; j0++) {
		if(Max_media_no < media_no[j0]) Max_media_no = media_no[j0];
	}
	FILE *fp_o;
	int *sheet_uvw;
	int L_max = L[0]+1;
	int M_max = L[1]+1;
	sheet_uvw = malloc(sizeof(int)*L_max*M_max*2);
	for(int jw0 = 0 ; jw0 < L[2] ; jw0++) { 
		char buf[BUFSIZE];
		sprintf(buf,"%s%d.dat", Yee, jw0);
		fp_o = fopen(buf,"w");
		if (fp_o == NULL){
			fprintf(stderr,"open error for %s\n", buf);
			exit(1);
		}
		time_t timer_f = time(0);
		fprintf(fp_o,"# Created on %s", ctime(&timer_f));
		fprintf(fp_o,"# Exe-file    : %s (Updated on Oct 28, 2014)\n", argv[0]);
		fprintf(fp_o,"# Lattice file: %s, Structure file: %s, Cell control file: %s\n", argv[1], argv[2], argv[3]);
		fprintf(fp_o,"# j_u < %d, j_v < %d, j_w < %d, media_no < %d\n", L[0], L[1], L[2], Max_media_no+1);
		fprintf(fp_o,"# k_u k_v k_w media_max\n");
		fprintf(fp_o,"# media_no. ratio\n");
		for(int jv0 = 0 ; jv0 <= L[1] ; jv0++) { 
			for(int ju0 = 0 ; ju0 <= L[0] ; ju0++) {
				if(jw0 == 0){
					double X[3] = {x[ju0*2+0], y[jv0*2+0], z[jw0*2+0]};
					sheet_uvw[ju0+L_max*(jv0+M_max*0)] = 
						media_f(X, N_str, media_no, shape_str, shape_length, origin_str, array_step, n_step);
				}else{
					sheet_uvw[ju0+L_max*(jv0+M_max*0)] = sheet_uvw[ju0+L_max*(jv0+M_max*1)];
				}
				double X[3] = {x[ju0*2+0], y[jv0*2+0], z[jw0*2+2]};
				sheet_uvw[ju0+L_max*(jv0+M_max*1)] = 
					media_f(X, N_str, media_no, shape_str, shape_length, origin_str, array_step, n_step);
			}
		}
		int media_p[8*8];
		double media_r[8*8];
		for(int jv0 = 0 ; jv0 < L[1] ; jv0++) { 
			for(int ju0 = 0 ; ju0 < L[0] ; ju0++) {
				for(int kw0 = 0 ; kw0 < 2 ; kw0++) { 
					for(int kv0 = 0 ; kv0 < 2 ; kv0++) { 
						for(int ku0 = 0 ; ku0 < 2 ; ku0++) { 
							media_p[0+8*(ku0+2*(kv0+2*kw0))] = sheet_uvw[(ju0+ku0) + L_max*((jv0+kv0) + M_max*kw0)];
							media_r[0+8*(ku0+2*(kv0+2*kw0))] = 1.e0;
							for(int i0 = 1 ; i0 < 8 ; i0++) {
								media_p[i0+8*(ku0+2*(kv0+2*kw0))] = -1;// 1 <= i0 <= 7
								media_r[i0+8*(ku0+2*(kv0+2*kw0))] = 0.;
							}
						}
					}
				}
				int check_i = -1;
				for(int i0 = 1 ; i0 < 8 ; i0++){
					if(media_p[0+8*i0] != media_p[0+8*0]) check_i = 1;
				}
				fprintf(fp_o,"%d %d\n", ju0, jv0);
				if(check_i < 0){
					for(int kw0 = 0 ; kw0 < 2 ; kw0++) { 
						for(int kv0 = 0 ; kv0 < 2 ; kv0++) { 
							for(int ku0 = 0 ; ku0 < 2 ; ku0++) { 
								fprintf(fp_o,"%d %d %d %d\n", ku0,kv0,kw0, 1);
								fprintf(fp_o,"%d %.20E\n", media_p[0+8*(ku0+2*(kv0+2*kw0))], media_r[0+8*(ku0+2*(kv0+2*kw0))]);
							}
						}
					}
				}else{
					double XXX[9] = {x[ju0*2+0], y[jv0*2+0], z[jw0*2+0],
										x[ju0*2+1], y[jv0*2+1], z[jw0*2+1],
										x[ju0*2+2], y[jv0*2+2], z[jw0*2+2]};
					int D_depth[2] = {0,division_depth};					
					est_volume(media_p, media_r, D_depth, XXX, N_str, 
									media_no, shape_str, shape_length, origin_str, array_step, n_step);
					for(int kw0 = 0 ; kw0 < 2 ; kw0++) { 
						for(int kv0 = 0 ; kv0 < 2 ; kv0++) { 
							for(int ku0 = 0 ; ku0 < 2 ; ku0++) {
								int i0 = 0;
								while(media_p[i0+8*(ku0+2*(kv0+2*kw0))] >= 0 && i0 < 9) i0++;
								fprintf(fp_o,"%d %d %d %d\n", ku0,kv0,kw0, i0);
								for(int j0 = 0 ; j0 < i0 ; j0++) fprintf(fp_o,"%d %.20E\n", 
									media_p[j0+8*(ku0+2*(kv0+2*kw0))], media_r[j0+8*(ku0+2*(kv0+2*kw0))]);
								double sum = 0.;
								for(int j0 = 0 ; j0 < i0 ; j0++) sum += media_r[j0+8*(ku0+2*(kv0+2*kw0))];
								if(sum != 1.) fprintf(fp_o,"volume error: sum = %.20E\n", sum);
							}
						}
					}
				}
			}
		}
		if(fclose(fp_o) != 0) {
			fprintf(stderr,"fclose error for %s\n", buf);
			exit(1);
		}
	}
	free(sheet_uvw);
}
void est_volume(int media_p[8*8], double media_r[8*8], int D_depth[2], const double XXX[9], const int N_str, const int media_no[N_str], const int shape_str[N_str], const double shape_length[N_str*6], const double origin_str[N_str*3], const double array_step[N_str*9], const int n_step[N_str*6])
{
// N_str                  : structure quantity (int).
// media_no[N_str]     : media number (int).
// shape_str[N_str]       : 'block' == 0, 'cylinder' == 1, 'prism' == 2, 'pyramid' == 3, 'spheroid' == 4, (int).
// shaple_length[N_str*6]:  shape_length[][0] = A, shape_length[][1] = B, shape_length[][2] = C (meter, double),
//                           structure rotation: shape_length[][5] = theta_x, shape_length[][6] = theta_y, shape_length[][6] = theta_z (radian, double).
// origin_str[N_str*3]   :  an origin of (x, y, z) coordinates for a structure.(meter, double).
// array_step[N_str*9]   :  step lengths for (1st stage, 2nd stage, 3rd stage) and (meter, double),
//                                        x-direction (array_step[][0], array_step[][3], array_step[][6]), 
//                                        y-direction (array_step[][1], array_step[][4], array_step[][7]), 
//                                        z-direction (array_step[][2], array_step[][5], array_step[][8]). 
// n_step[N_str*6]       :  step number for (1st stage, 2nd stage, 3rd stage), (int),
//                                        initial number (n_step[][0], n_step[][2], n_step[][4]), 
//                                         final  number (n_step[][1], n_step[][3], n_step[][5]). 
	D_depth[0]++;
	if( D_depth[0] <= D_depth[1] ){//20141028 revised
		int media_p3[3*3*3]; 
		for(int kw0 = 0 ; kw0 < 2 ; kw0++) { 
			for(int kv0 = 0 ; kv0 < 2 ; kv0++) { 
				for(int ku0 = 0 ; ku0 < 2 ; ku0++) {
					media_p3[(ku0*2)+3*((kv0*2)+3*(kw0*2))] = media_p[0+8*(ku0+2*(kv0+2*kw0))];
				}
			}
		}
		for(int kw0 = 0 ; kw0 < 3 ; kw0++) { 
			for(int kv0 = 0 ; kv0 < 3 ; kv0++) { 
				for(int ku0 = 0 ; ku0 < 3 ; ku0++) {
					if(ku0*(ku0-2) != 0 || kv0*(kv0-2) != 0 || kw0*(kw0-2) != 0){
						double X[3];
						X[0] = XXX[0+3*ku0];
						X[1] = XXX[1+3*kv0];
						X[2] = XXX[2+3*kw0];
						media_p3[ku0+3*(kv0+3*kw0)] = 
							media_f(X, N_str, media_no, shape_str, shape_length, origin_str, array_step, n_step);
					}
				}
			}
		}
		for(int kw0 = 0 ; kw0 < 2 ; kw0++) { 
			for(int kv0 = 0 ; kv0 < 2 ; kv0++) { 
				for(int ku0 = 0 ; ku0 < 2 ; ku0++) {
					int media_p2[2*2*2];
					for(int i0 = 0 ; i0 < 8 ; i0++) media_p2[i0] = -1;
					for(int LW = 0 ; LW < 2 ; LW++) { 
						for(int LV = 0 ; LV < 2 ; LV++) { 
							for(int LU = 0 ; LU < 2 ; LU++) {
								media_p2[LU+2*(LV+2*LW)] = media_p3[(ku0+LU)+3*((kv0+LV)+3*(kw0+LW))];
							}
						}
					}
					int check_i = -1;
					for(int i0 = 1 ; i0 < 8 ; i0++){
						if(media_p2[i0] != media_p2[0]) check_i = 1;
					}
					if(check_i < 0){
						media_p[0+8*(ku0+2*(kv0+2*kw0))] = media_p2[0];
						media_r[0+8*(ku0+2*(kv0+2*kw0))] = 1.;
						for(int i0 = 1 ; i0 < 8 ; i0++){
							media_p[i0+8*(ku0+2*(kv0+2*kw0))] = -1;
							media_r[i0+8*(ku0+2*(kv0+2*kw0))] = 0.;
						}
					}else{
						int media_pD[8*8];//media_p for sub region
						double media_rD[8*8];//media_r for sub region
						for(int i0 = 0 ; i0 < 8 ; i0++) {
							media_pD[0+8*i0] = media_p2[i0];
							media_rD[0+8*i0] = 1.e0;
							for(int j0 = 1 ; j0 < 8 ; j0++) {
								media_pD[j0+8*i0] = -1;
								media_rD[j0+8*i0] = 0.;
							}
						}
						{//recursive region
							double XXXD[9] = {XXX[0+3*ku0], XXX[1+3*kv0], XXX[2+3*kw0], 0, 0, 0, 
												XXX[0+3*(ku0+1)], XXX[1+3*(kv0+1)], XXX[2+3*(kw0+1)]};//20141028 revised
							for(int i0 = 0 ; i0 < 3 ; i0++) XXXD[i0+3*1] = ldexp(XXXD[i0+3*0]+XXXD[i0+3*2],-1);
							int D_depthD[2] = {D_depth[0],D_depth[1]};					
							est_volume(media_pD, media_rD, D_depthD, XXXD, N_str, media_no, 
										shape_str, shape_length, origin_str, array_step, n_step);
						}//end of recursive region
						for(int i0 = 1 ; i0 < 8 ; i0++) {//shift i0-th column elements to 0-th column for media_pD and media_rD
							for(int k0 = 0 ; k0 < 8 ; k0++) {
								if(media_pD[k0+8*i0] >= 0){
									for(int j0 = 0 ; j0 < 8 ; j0++) {
										if(media_pD[j0+8*0] == media_pD[k0+8*i0]){
											media_rD[j0+8*0] += media_rD[k0+8*i0];
											goto next_search;
										}
										if(media_pD[j0+8*0] < 0){
											media_pD[j0+8*0] = media_pD[k0+8*i0];
											media_rD[j0+8*0] = media_rD[k0+8*i0];
											goto next_search;
										}
									}
								}
								next_search:;
								media_rD[k0+8*i0] = 0.;
								media_pD[k0+8*i0] = -1;
							}
						}//end of shift i0-th column elements to 0-th column for media_pD and media_rD
						for(int j0 = 0 ; j0 < 8 ; j0++) {
							media_p[j0+8*(ku0+2*(kv0+2*kw0))] = media_pD[j0+8*0];
							media_r[j0+8*(ku0+2*(kv0+2*kw0))] = ldexp(media_rD[j0+8*0],-3);
						}
					}
				}
			}
		}
	}
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                    int media_f                                  ||||||||||||
||||||||||||                            Last updated on Sep 06, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
//
//           int media_f ----- void rotation
//                         |
//                          -- int shap_region
//
void rotation(const double theta, const double x0, const double y0, double *x1, double *y1);
int shape_region(const int type, const double A, const double B, const double C, const double x, const double y, const double z);
int media_f(const double X[3], const int N_str, const int media_no[N_str], const int shape_str[N_str], const double shape_length[N_str*6], const double origin_str[N_str*3], const double array_step[N_str*9], const int n_step[N_str*6])
{
// This 'media_f' gives a media number at a coordinate.
// X[3]: (X[0],X[1],X[2]) is a coordinate in 3D structure.
// N_str: structure number (int).
// media_no[N_str]: media number of a structure (int).
// N_str                  : structure quantity (int).
// media_no[N_str]     : media number (int).
// shape_str[N_str]       : 'block' == 0, 'cylinder' == 1, 'prism' == 2, 'pyramid' == 3, 'spheroid' == 4, (int).
// shaple_length[N_str*6]:  shape_length[][0] = A, shape_length[][1] = B, shape_length[][2] = C (meter, double),
//                           structure rotation: shape_length[][3] = theta_x, shape_length[][4] = theta_y, shape_length[][5] = theta_z (radian, double).
// origin_str[N_str*3]   :  an origin of (x, y, z) coordinates for a structure.(meter, double).
// array_step[N_str*9]   :  step lengths for (1st stage, 2nd stage, 3rd stage) and (meter, double),
//                                        x-direction (array_step[][0], array_step[][3], array_step[][6]), 
//                                        y-direction (array_step[][1], array_step[][4], array_step[][7]), 
//                                        z-direction (array_step[][2], array_step[][5], array_step[][8]). 
// n_step[N_str*6]       :  step number for (1st stage, 2nd stage, 3rd stage), (int),
//                                        initial number (n_step[][0], n_step[][2], n_step[][4]), 
//                                         final  number (n_step[][1], n_step[][3], n_step[][5]). 
	int med = 0;
	for(int J = 0 ; J < N_str ; J++){
		for(int L2 = n_step[J*6+4] ; L2 <= n_step[J*6+5]; L2++){
			for(int L1 = n_step[J*6+2] ; L1 <= n_step[J*6+3]; L1++){
				for(int L0 = n_step[J*6+0] ; L0 <= n_step[J*6+1]; L0++){
					double xx = origin_str[J*3+0] + L0*array_step[J*9+0+0*3] + L1*array_step[J*9+0+1*3] + L2*array_step[J*9+0+2*3];
					double yy = origin_str[J*3+1] + L0*array_step[J*9+1+0*3] + L1*array_step[J*9+1+1*3] + L2*array_step[J*9+1+2*3];
					double zz = origin_str[J*3+2] + L0*array_step[J*9+2+0*3] + L1*array_step[J*9+2+1*3] + L2*array_step[J*9+2+2*3];
				// begin Cardan angles rotation X -> Y -> Z (X[0]-xx,X[1]-yy,X[2]-zz) --> (x1,y1,z1)
				// following calc. is inverse rotation Z -> Y -> X
					double x0, y0;// begin yaw (z-axis) angle rotation
					rotation(-shape_length[J*6+5], X[0] - xx, X[1] - yy, &x0, &y0);// end yaw angle rotation
					double z0, x1;// begin pitch (y-axis) angle rotation
					rotation(-shape_length[J*6+4], X[2] - zz, x0, &z0, &x1);// end pitch angle rotation
					double y1, z1;// begin roll (x-axis) angle rotation
					rotation(-shape_length[J*6+3], y0, z0, &y1, &z1);// end roll angle rotation
				//  end  Cardan angles rotation ZYX (X[0]-xx,X[1]-yy,X[2]-zz) --> (x1,y1,z1)
					if(shape_region(shape_str[J], shape_length[J*6+0],shape_length[J*6+1],shape_length[J*6+2], x1,y1,z1) == 1) med = media_no[J];
				}
			}
		}
	}
	return(med);
}
void rotation(const double theta, const double x0, const double y0, double *x1, double *y1)
{
// This 'rotation' is the rotating operator of coordinates.
// theta is rotating angle (radian: double).
// (x0, y0) is an input coordinate.
// (x1, y1) is a rotated coordinate.
//
	double s = sin(theta);
	double c = cos(theta);
	*x1 = c*x0 - s*y0;
	*y1 = s*x0 + c*y0;
}
int shape_region(const int type, const double A, const double B, const double C, const double x, const double y, const double z)
{
// This 'shape_region' gives 1 (0) when (x,y,z) is in (out) a shape.
// type: 'block' == 0, 'cylinder' == 1, 'prism' == 2, 'pyramid' == 3, 'spheroid' == 4 (int).
//
	int tag = -1;
	if (type == 0) {//block
//		if((A-x)*x > 0. && (B-y)*y > 0. && (C-z)*z > 0.) {
		if((A-x)*(A+x) >= 0. && (B-y)*(B+y) >= 0. && (C-z)*(C+z) >= 0.) {
			tag = 1;
		}
		else {
			tag = 0;
		}
	}
	if (type == 1) {//cylinder
		if(x*x/(A*A) + y*y/(B*B) <= 1. && (C-z)*(C+z) >= 0.) {
			tag = 1;
		}
		else {
			tag = 0;
		}
	}
	if (type == 2) {//prism
		if(x >= 0. && y >= 0. && x/A + y/B <= 1. && (C-z)*(C+z) >= 0.) {
			tag = 1;
		}
		else {
			tag = 0;
		}
	}
	if (type == 3) {//pyramid
		if(x >= 0. && y >= 0. && z >= 0. && x/A + y/B + z/C <= 1.) {
			tag = 1;
		}
		else {
			tag = 0;
		}
	}
	if (type == 4) {//spheroid
		if(x*x/(A*A) + y*y/(B*B) + z*z/(C*C) <= 1.) {
			tag = 1;
		}
		else {
			tag = 0;
		}
	}
	
	if (tag == -1) {
		fprintf(stderr,"shape_region error!\n");
		exit(1);
	}
	return(tag);
}
//////////////////////////////  block, cylinder, spheroid  //////////////////////////////
//                                                                                     //
//                                                                                     //
//                       (-A,-B,+C) --                    -- (-A,+B,+C)                //
//                                /|                       /|                          //
//                                                                                     //
//                                                                                     //
//                                                                                     //
//                                                                                     //
//                                                                                     //
//                           /                        /                                //
//                (+A,-B,+C) --                    -- (+A,+B,+C)                       //
//                          |                        |                                 //
//                                                                                     //
//                                         |/                                          //
//                                       -----(0,0,0)                                  //
//                                        /|                                           //
//                                                                                     //
//                                 |                        |                          //
//                       (-A,-B,-C) --                    -- (-A,+B,-C)                //
//                                /                        /                           //
//                                                                                     //
//                                                                                     //
//                                                                                     //
//                                                                                     //
//                                                                                     //
//                          |/                       |/                                //
//                (+A,-B,-C) --                    -- (+A,+B,-C)                       //
//                                                                                     //
//                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////   prism   /////////////////////////////////////////
//                                                                                     //
//                                                                                     //
//                                 (0,0,+C) --           -- (0,+B,+C)                  //
//                                        /|           *   |                           //
//                                                 *                                   //
//                                             *                                       //
//                                      /  *                                           //
//                           (+A,0,+C) *                                               //
//                                     |                                               //
//                                         |/                                          //
//                                       -----(0,0,0)                                  //
//                                        /|                                           //
//                                                                                     //
//                                                                                     //
//                                                                                     //
//                                                                                     //
//                                                                                     //
//                                         |               |                           //
//                                 (0,0,-C) --           -- (0,+B,-C)                  //
//                                        /            *                               //
//                                                 *                                   //
//                                             *                                       //
//                                     |/  *                                           //
//                           (+A,0,-C) *                                               //
//                                                                                     //
//                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////   pyramid   ////////////////////////////////////////
//                                                                                     //
//                                                                                     //
//                              (0,0,+C)                                               //
//                                      *                                              //
//                                      |   *                                          //
//                                     *|        *                                     //
//                                      |             *                                //
//                                    *  ------------------* (0,+B,0)                  //
//                                     /(0,0,0)       *                                //
//                                   */         *                                      //
//                                   /    *                                            //
//                         (+A,0,0) *                                                  //
//                                                                                     //
//                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////
