/*                        This program is Main Program of MeshPlot.                       */
/*                                                     Old name: MatrixOut. Sep/30/2014   */
/*                    Main and sub programs were coded by Tatsuya Usuki.                  */
/*                                   Program language is C99.                             */
/*                                                                                        */
/*                                                         Last modified on Nov/12/2014.  */
#include <tgmath.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#define BUFSIZE 1024 /* buffer for fgets */
#define Pi 3.141592653589793238462643 /* circular constant */
//#define Pi2 6.283185307179586476925286
void input_file1(FILE *fp_i1, int L[3]);
void input_fileM(FILE *fp_i1, const int L, double x[L*2], double dxdu[L*2]);
void EPS_out(char *datafile, char X[], char Y[], const long L, const long M, const double x[], const double y[]);

int main(int argc, char **argv)
{
	time_t timer_s = time(0);
	fprintf(stderr,"# `MeshPlot' starts on %s\n", ctime(&timer_s));
	if(argc != 2) {
		fprintf(stderr,"error: number of files \n");
		exit(1);
	}
	
//---------- begin reading Yee lattice information ------------
	FILE *fp_i;
	fp_i = fopen(argv[1],"r");
	if (fp_i == NULL){
		fprintf(stderr,"open error!: open input-file1!\n");
		exit(1);
	}
	fprintf(stderr,"Mesh data file: %s\n",argv[1]);

	int L[3]; //division number: Lu = L, Lv = M, Lw = N
	
	input_file1(fp_i, L); // inputting parameters from an inputfile 1
	
	double x[L[0]*2+1], dxdu[L[0]*2];
	input_fileM(fp_i, L[0], x, dxdu);
	
	double y[L[1]*2+1], dydv[L[1]*2];
	input_fileM(fp_i, L[1], y, dydv);
	
	double z[L[2]*2+1], dzdw[L[2]*2];
	input_fileM(fp_i, L[2], z, dzdw);
	
	fclose(fp_i);
	
	double x_max = x[2*L[0]]; 
	double y_max = y[2*L[1]]; 
	double z_max = z[2*L[2]]; 
	for(int i0 = 0 ; i0 < 3 ; i0++){
		switch(i0){
			case 0:	fprintf(stderr,"l, x[2*l], x[2*l+1], dx/du[2*l], dx/du[2*l+1]\n");
			for(int i1 = 0 ; i1 < L[i0] ; i1++)	fprintf(stderr,"%d, %.7E, %.7E, %.7E, %.7E\n", 
														i1, x[2*i1], x[2*i1+1], dxdu[2*i1], dxdu[2*i1+1]);
			fprintf(stderr,"x_max = %.7E\n", x_max);
			fprintf(stderr,"\n");
			break;
			case 1:	fprintf(stderr,"m, y[2*m], y[2*m+1], dy/dv[2*m], dy/dv[2*m+1]\n");
			for(int i1 = 0 ; i1 < L[i0] ; i1++)	fprintf(stderr,"%d, %.7E, %.7E, %.7E, %.7E\n", 
														i1, y[2*i1], y[2*i1+1], dydv[2*i1], dydv[2*i1+1]);
			fprintf(stderr,"y_max = %.7E\n", y_max);
			fprintf(stderr,"\n");
			break;
			case 2:	fprintf(stderr,"n, z[2*n], z[2*n+1], dz/dw[2*n], dz/dw[2*n+1]\n");
			for(int i1 = 0 ; i1 < L[i0] ; i1++)	fprintf(stderr,"%d, %.7E, %.7E, %.7E, %.7E\n", 
														i1, z[2*i1], z[2*i1+1], dzdw[2*i1], dzdw[2*i1+1]);
			fprintf(stderr,"z_max = %.7E\n", z_max);
			fprintf(stderr,"\n");
			break;
			default: fprintf(stderr,"Error!\n");
			exit(1);
		}
	}
	////////////////////////////////////////////////////////////

	EPS_out(argv[1],"X","Y",L[0],L[1],x,y);
	EPS_out(argv[1],"Y","Z",L[1],L[2],y,z);
	EPS_out(argv[1],"Z","X",L[2],L[0],z,x);
	
	time_t timer_f = time(0);
	fprintf(stderr,"# `MeshPlot' finished on %s\n", ctime(&timer_f));
	
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                              input_file1, input_fileM                           ||||||||||||
||||||||||||                            Last updated on Oct 29, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void input_file1(FILE *fp_i, int L[3])
{
	char buf[BUFSIZE];	// buffer for fgets
	while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %d %*[^=] %*[=] %d %*[^=] %*[=] %d", &L[0], &L[1], &L[2]) == 3) goto End_of_LMN;
	}
	fprintf(stderr,"No L,M,N are read @ input_file1!\n");
	exit(1);
	End_of_LMN:;

	if(L[0] < 1 || L[1] < 1 || L[2] < 1) {
		fprintf(stderr,"L,M,N input error!: L = %d, M = %d, N = %d \n", L[0], L[1], L[2]);
		exit(1);
	}
}
void input_fileM(FILE *fp_i, const int L, double x[L*2+1], double dxdu[L*2])
{
	char buf[BUFSIZE];	// buffer for fgets
	int i0 = 0;
	int i1;
	while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
		if(sscanf(buf,"%d %*[,] %lf  %*[,] %lf %*[,] %lf %*[,] %lf", 
			&i1, &x[i0*2+0], &x[i0*2+1], &dxdu[i0*2+0], &dxdu[i0*2+1]) == 5){
			if(i0 == i1) i0++;
			if(i0 == L) goto End_of_x_data;
		}
	}
	fprintf(stderr,"Full %d-data were not read @ input_fileM!\n",L);
	exit(1);
	End_of_x_data:;

	while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
		if(sscanf(buf,"%d %*[,] %lf", &i1, &x[2*L]) == 2){
			if(i1 == L) goto End_of_x_max;
		}
	}
	fprintf(stderr,"The %d-th data were not read @ input_fileM!\n",L+1);
	exit(1);
	End_of_x_max:;
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                      EPS_out                                    ||||||||||||
||||||||||||                            Last updated on Nov 14, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void Plot_boundary_line(FILE *fp_o, const long linewidth_unit, const long Ox, const long Oy, 
						const long ix_min, const long ix_max, const long iy_min, const long iy_max);
void Plot_ticks(FILE *fp_o, const long linewidth_unit, const long Ox, const long Oy, 
						const long ix_min, const long ix_max, const long iy_min, const long iy_max, 
						const long boundary_margin, const long char_margin, const char *X, const char *Y);

void EPS_out(char *datafile, char X[], char Y[], const long L, const long M, const double x[], const double y[])
{
	char dummy[BUFSIZE], buf[BUFSIZE];
	sscanf(datafile,"%s", dummy);//	sscanf(datafile,"%[^.]s", dummy); revised on 20141114
	sprintf(buf, "%s_%s%s.eps", dummy, X, Y);
	FILE *fp_o;
	fp_o = fopen(buf,"w");

	double x_min = x[0];
	double y_min = y[0];
	double x_max = x[2*L];
	double y_max = y[2*M];

	double dl = fabs(x_max - x_min) + fabs(y_max - y_min);
	for(long i0 = 0 ; i0 < 2*L ; i0++){
		if(dl > fabs(x[i0+1]-x[i0])) dl = fabs(x[i0+1]-x[i0]);
	}
	for(long i0 = 0 ; i0 < 2*M ; i0++){
		if(dl > fabs(y[i0+1]-y[i0])) dl = fabs(y[i0+1]-y[i0]);
	}
	dl /= 10.;
	long square_size = 2;
	long boundary_margin = square_size*10;
	long char_margin = boundary_margin*4;
	
	long ix_min = (long) (x_min*((double) square_size)/dl);
	long ix_max = (long) (x_max*((double) square_size)/dl);
	long iy_min = (long) (y_min*((double) square_size)/dl);
	long iy_max = (long) (y_max*((double) square_size)/dl);

	double char_size = (double) boundary_margin;
	long linewidth_unit = 1;
	
	long Ox = -(ix_min - boundary_margin - char_margin);
	long Oy = -(iy_min - boundary_margin - char_margin);
	
	//========================================== Plot title ============================================
	fprintf(fp_o,"%%!PS-Adobe-3.0 EPSF-3.0\n%%%%BoundingBox: %ld %ld %ld %ld\n", 
					Ox + ix_min - boundary_margin - char_margin, Oy + iy_min - boundary_margin - char_margin, 
					Ox + ix_max + boundary_margin + char_margin, Oy + iy_max + boundary_margin + char_margin);
	fprintf(fp_o,"gsave\n");
	fprintf(fp_o,"/Helvetica findfont %g scalefont setfont\n", char_size);
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n(Yee lattice on %s-%s plane for `%s') show\n", 
					Ox + ix_min, Oy + iy_max + 7*boundary_margin/2, X, Y, datafile);
	fprintf(fp_o,"%ld %ld moveto\n(%smin = %g m, %smax = %g m, %smin = %g m, %smax = %g m) show\n", 
					Ox + ix_min, Oy + iy_max + 3*boundary_margin/2, X, x_min, X, x_max, Y, y_min, Y, y_max);
	//========================================== Title end  ============================================
	Plot_boundary_line(fp_o, linewidth_unit, Ox, Oy, ix_min, ix_max, iy_min, iy_max);
	Plot_ticks(fp_o, linewidth_unit, Ox, Oy, ix_min, ix_max, iy_min, iy_max, boundary_margin, char_margin, X, Y);

	for(long i0 = 0 ; i0 <= L ; i0++){
		fprintf(fp_o,"newpath\n");
		fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
		fprintf(fp_o,"0 setgray\n");
//		fprintf(fp_o,"[] 0 setdash\n");
		long ix = (long) (x[2*i0]*((double) square_size)/dl);
		fprintf(fp_o,"%ld %ld moveto\n",  Ox + ix, Oy + iy_min - boundary_margin);
		fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix, Oy + iy_max + boundary_margin);
		fprintf(fp_o,"stroke\n");
		if(i0 != L){
			fprintf(fp_o,"newpath\n");
			fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
			fprintf(fp_o,"0.5 setgray\n");
//			fprintf(fp_o,"[] 0 setdash\n");
			ix = (long) (x[2*i0+1]*((double) square_size)/dl);
			fprintf(fp_o,"%ld %ld moveto\n",  Ox + ix, Oy + iy_min - boundary_margin);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix, Oy + iy_max + boundary_margin);
			fprintf(fp_o,"stroke\n");
		}
	}
	for(long i0 = 0 ; i0 <= M ; i0++){
		fprintf(fp_o,"newpath\n");
		fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
		fprintf(fp_o,"0 setgray\n");
		long iy = (long) (y[2*i0]*((double) square_size)/dl);
		fprintf(fp_o,"%ld %ld moveto\n",  Ox + ix_min - boundary_margin, Oy + iy);
		fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix_max + boundary_margin, Oy + iy);
		fprintf(fp_o,"stroke\n");
		if(i0 != M){
			fprintf(fp_o,"newpath\n");
			fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
			fprintf(fp_o,"0.5 setgray\n");
			iy = (long) (y[2*i0+1]*((double) square_size)/dl);
			fprintf(fp_o,"%ld %ld moveto\n",  Ox + ix_min - boundary_margin, Oy + iy);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix_max + boundary_margin, Oy + iy);
			fprintf(fp_o,"stroke\n");
		}
	}
	fprintf(fp_o,"showpage\n");
	fprintf(fp_o,"grestore\n");
	
	fclose(fp_o);
}
void Plot_boundary_line(FILE *fp_o, const long linewidth_unit, const long Ox, const long Oy, 
						const long ix_min, const long ix_max, const long iy_min, const long iy_max)
{
	//====================================== Plot boundary line ========================================
	fprintf(fp_o,"0.5 setgray\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit*8);
	fprintf(fp_o,"[] 0 setdash\n");//	fprintf(fp_o,"[5 3 100 3] 0 setdash\n");
	fprintf(fp_o,"2 setlinejoin\n");
	fprintf(fp_o,"\n%ld %ld moveto\n",  Ox + ix_min, Oy + iy_min);
	fprintf(fp_o,"%ld %ld lineto\n",    Ox + ix_min, Oy + iy_max);
	fprintf(fp_o,"%ld %ld lineto\n",    Ox + ix_max, Oy + iy_max);
	fprintf(fp_o,"%ld %ld lineto\n",    Ox + ix_max, Oy + iy_min);
	fprintf(fp_o,"closepath\n");
	fprintf(fp_o,"stroke\n");
}
void Plot_ticks(FILE *fp_o, const long linewidth_unit, const long Ox, const long Oy, 
						const long ix_min, const long ix_max, const long iy_min, const long iy_max, 
						const long boundary_margin, const long char_margin, const char *X, const char *Y)
{
	//======================================== Plot Xmin tick ==========================================
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n( %smin) show\n", 
					Ox + ix_min, Oy + iy_min-7*boundary_margin/2, X);
	fprintf(fp_o,"newpath\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
	fprintf(fp_o,"%ld %ld moveto\n", Ox + ix_min, Oy + iy_min-boundary_margin-char_margin);
	fprintf(fp_o,"%ld %ld lineto\n", Ox + ix_min, Oy + iy_min-boundary_margin);
	fprintf(fp_o,"stroke\n");
	//======================================== Plot Xmid tick ==========================================
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n( %smid) show\n", 
					Ox + (ix_min+ix_max)/2, Oy + iy_min-7*boundary_margin/2, X);
	fprintf(fp_o,"newpath\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
	fprintf(fp_o,"%ld %ld moveto\n", Ox + (ix_min+ix_max)/2, Oy + iy_min-boundary_margin-char_margin);
	fprintf(fp_o,"%ld %ld lineto\n", Ox + (ix_min+ix_max)/2, Oy + iy_min-boundary_margin);
	fprintf(fp_o,"stroke\n");
	//======================================== Plot Xmax tick ==========================================
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n( %smax) show\n", 
					Ox + ix_max, Oy + iy_min-7*boundary_margin/2, X);
	fprintf(fp_o,"newpath\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
	fprintf(fp_o,"%ld %ld moveto\n", Ox + ix_max, Oy + iy_min-boundary_margin-char_margin);
	fprintf(fp_o,"%ld %ld lineto\n", Ox + ix_max, Oy + iy_min-boundary_margin);
	fprintf(fp_o,"stroke\n");
	//======================================== Plot Ymin tick ==========================================
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n90 rotate\n( %smin) show\n-90 rotate\n", 
					Ox + ix_min-5*boundary_margin/2, Oy + iy_min, Y);
	fprintf(fp_o,"newpath\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
	fprintf(fp_o,"%ld %ld moveto\n", Ox + ix_min-boundary_margin-char_margin, Oy + iy_min);
	fprintf(fp_o,"%ld %ld lineto\n", Ox + ix_min-boundary_margin,             Oy + iy_min);
	fprintf(fp_o,"stroke\n");
	//======================================== Plot Ymid tick ==========================================
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n90 rotate\n( %smid) show\n-90 rotate\n", 
					Ox + ix_min-5*boundary_margin/2, Oy + (iy_min+iy_max)/2, Y);
	fprintf(fp_o,"newpath\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
	fprintf(fp_o,"%ld %ld moveto\n", Ox + ix_min-boundary_margin-char_margin, Oy + (iy_min+iy_max)/2);
	fprintf(fp_o,"%ld %ld lineto\n", Ox + ix_min-boundary_margin,             Oy + (iy_min+iy_max)/2);
	fprintf(fp_o,"stroke\n");
	//======================================== Plot Ymax tick ==========================================
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n90 rotate\n( %smax) show\n-90 rotate\n", 
					Ox + ix_min-5*boundary_margin/2, Oy + iy_max, Y);
	fprintf(fp_o,"newpath\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
	fprintf(fp_o,"%ld %ld moveto\n", Ox + ix_min-boundary_margin-char_margin, Oy + iy_max);
	fprintf(fp_o,"%ld %ld lineto\n", Ox + ix_min-boundary_margin,             Oy + iy_max);
	fprintf(fp_o,"stroke\n");
}

