/* This program is Check Program of BottomTop. Aug/08/2012   */
/*     Main and sub programs were coded by Tatsuya Usuki.    */
/*                            Last updated on Nov/14/2014    */
#include <tgmath.h>
//#include <float.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
//#include "constant.h"
#define BUFSIZE 1024 // buffer for fgets
#define Pi 3.141592653589793238462643 // circular constant
#define Pi2 6.283185307179586476925286
#define epsilon_0 8.85418782e-12 // permittivity of vacuum [F/m]
#define mu_0 1.25663706e-6 // permeability of vacuum [H/m]
#define c_0 2.99792458e8 // light velocity in vacuum [m/s]

//
// * --> Main_calc ---- mode_calc --> *
//
void mode_calc(char Yeex[], const long L[4], const double fg[(L[0]+L[1])*2], double complex U_ini[4*L[0]*L[1]*L[0]*L[1]], double Max_err[3], double ModeD[4*L[0]*L[1]*L[0]*L[1]], double Evu[4*L[0]*L[1]*L[0]*L[1]], const double complex eigenvalue[2*L[0]*L[1]], double complex UMU[4*L[0]*L[1]*L[0]*L[1]]);

void Main_calc(const char Yee[], const double Lambda, const int LMN[3], const int solver_sw, 
				const double dxdu[(LMN[0]+LMN[1]+LMN[2])*2], const int Buv[2], const double x[(LMN[0]+LMN[1]+LMN[2])*2])
{
	char buf[BUFSIZE];
	long L[4]={LMN[0],LMN[1],Buv[0],Buv[1]};// L[0]=L, L[1]=M, L[2]=B_u, L[4]=B_v.
	double k0 = Pi2/Lambda;
	double *fg;
	fg = malloc(sizeof(double)*(LMN[0]+LMN[1])*2);
	                              // f_k (j) = fg[2*j + k] where 0 <= j < LMN[0] and k = 0 or 1,
	                              // g_k (j) = fg[2*(j+LMN[0]) + k] where 0 <= j < LMN[1] and k = 0 or 1,
	for(int J_LMN = 0 ; J_LMN < (LMN[0]+LMN[1])*2 ; J_LMN++) fg[J_LMN] = sqrt(1./(k0*dxdu[J_LMN]));
		                          // dxdu_k (j) = dxdu[2*j + k] where 0 <= j < LMN[0] and k = 0 or 1,
		                          // dydv_k (j) = dxdu[2*(j+LMN[0]) + k] where 0 <= j < LMN[1] and k = 0 or 1.
	for(int J_bt = 0 ; J_bt < 2 ; J_bt++) { 
		char Yeex[BUFSIZE];
		FILE *fp_i;
		if(J_bt == 0) sprintf(Yeex,"%sBottom.dat", Yee);
		if(J_bt == 1) sprintf(Yeex,"%sTop.dat", Yee);
		fp_i = fopen(Yeex,"r");
		long MM = 2*L[0]*L[1];
		double complex *eigenvalue;
		eigenvalue = malloc(sizeof(double complex)*MM);
		{
			long J_column = 0;
			while(fgets(buf, sizeof( buf ), fp_i) != NULL) {
				long J_check;
				double eigenvalue_r, eigenvalue_i;
				if(sscanf(buf,"%ld %lf %lf", &J_check, &eigenvalue_r, &eigenvalue_i) == 3){
					if(J_check == J_column) {
						eigenvalue[J_column] = eigenvalue_r + I*eigenvalue_i;
						J_column++;
					}
					if(J_column == MM) goto End_of_eigenvalue;
				}
			}
			fprintf(stderr,"The %ld-th eigenvalue was not read @ %s!\n",J_column, Yeex);
			exit(1);
			End_of_eigenvalue:;
		}
		double complex *U_ini;
		U_ini = malloc(sizeof(double complex)*MM*MM);
		{
			long J_column = 0;
			while(fgets(buf, sizeof( buf ), fp_i) != NULL) {
				long J_check, K_check;
				if(sscanf(buf,"%*[^(] %*[(] %ld %*[^(] %*[(] %ld", &J_check, &K_check) == 2){
					if(J_check == J_column && J_check == K_check) {
						long J_row = 0;
						while(fgets(buf, sizeof( buf ), fp_i) != NULL) {
							double r_U, i_U;
							if(sscanf(buf,"%lf %lf", &r_U, &i_U) == 2){
								U_ini[J_row + J_column*MM] = r_U + I*i_U;
								J_row++;
							}
							if(J_row == MM) goto End_of_columnvector;
						}
						fprintf(stderr,"The %ld-th element of %ld column was not read @ %s!\n", J_row, J_column, Yeex);
						exit(1);
						End_of_columnvector:;
						J_column++;
					}
					if(J_column == MM) goto End_of_eigenvector;
				}
			}
			fprintf(stderr,"The %ld-th eigenvector was not read @ %s!\n",J_column, Yeex);
			exit(1);
			End_of_eigenvector:;
		}
		fclose(fp_i);
		sprintf(Yeex,"%s_Med%d.dat", Yee, J_bt*(LMN[2]-1));
		double Max_err[3] = {0., 0., 0.};
		double *ModeD;
		ModeD = malloc(sizeof(double)*MM*MM);
		double *Evu;
		Evu = malloc(sizeof(double)*MM*MM);
		double complex *UMU;
		UMU = malloc(sizeof(double complex)*MM*MM);
		time_t timer_s = time(0);
		fprintf(stderr,"# Step= %d, mode check calc. start time= %s\n", J_bt, ctime(&timer_s));
		mode_calc(Yeex, L, fg, U_ini, Max_err, ModeD, Evu, eigenvalue, UMU);
		free(U_ini);
		time_t timer_f = time(0);
		double t_span = difftime(timer_f,timer_s);
		fprintf(stderr,"# Step= %d, exec time= %1.1f sec\n\n", J_bt, t_span);

		long wave_MM = 0;		
		for(long Mode = 0 ; Mode < MM ; Mode++) {
			if(creal(eigenvalue[Mode]) >= 0. && cimag(eigenvalue[Mode]) == 0.) wave_MM = Mode;
		}
		wave_MM++;
		for(long Mode = 0 ; Mode < wave_MM ; Mode++) {
			if(J_bt == 0) sprintf(Yeex,"%s_BottomMode%ld.dat", Yee, Mode);
			if(J_bt == 1) sprintf(Yeex,"%s_TopMode%ld.dat", Yee, Mode);

			FILE *fp_o;
			fp_o = fopen(Yeex,"w");
			fprintf(fp_o,"# File name : ./%s on %s# Exe-file  : ./ModeCheck (Updated on Nov 14, 2014)\n", Yeex, ctime(&timer_s));
			if(solver_sw == 0) fprintf(fp_o,"# Eigenproblem solver: LAPACK dgeev (default)\n");
			if(solver_sw != 0) fprintf(fp_o,"# Eigenproblem solver: lgeev_ (long double-precision)\n");
			fprintf(fp_o,"# Max_norm within propagating modes = %.5E\n", Max_err[0]);
			fprintf(fp_o,"# Max_norm between propagating and evanescent modes = %.5E\n", Max_err[1]);
			fprintf(fp_o,"# total Max_norm= %.5E\n", Max_err[2]);
			fprintf(fp_o,"# Mode = %ld/%ld, total no. with evanescent modes = %ld\n", Mode, wave_MM, MM);
			fprintf(fp_o,"# x[m] y[m] EuHv Eu Ev\n");
			for(long J_y = 0 ; J_y < L[1] ; J_y++) {
				fprintf(fp_o,"\n");
				for(long J_x = 0 ; J_x < L[0] ; J_x++) {
					fprintf(fp_o,"%.2E %.2E ", x[2*(J_x + L[0]*0) + 1], x[2*(J_y + L[0]*1) + 1]);
					double Poynting = 0.25*ModeD[J_x + L[0]*J_y + L[0]*L[1] + Mode*MM];
					Poynting += 0.25*ModeD[J_x + L[0]*J_y + Mode*MM];
					long J_xp = (J_x + 1)%L[0];//					if(J_xp == L[0]) J_xp = 0;
					long J_yp = (J_y + 1)%L[1];//					if(J_yp == L[1]) J_yp = 0;
					Poynting += 0.25*ModeD[J_xp + L[0]*J_y + L[0]*L[1] + Mode*MM];
					Poynting += 0.25*ModeD[J_x + L[0]*J_yp + Mode*MM];
					Poynting *= pow(fg[2*J_x + 1]*fg[2*(J_y +L[0]) + 1],2);
					fprintf(fp_o,"%.2E ", Poynting);
					fprintf(fp_o,"%.2E ", Evu[J_x+L[0]*J_y+L[0]*L[1]+Mode*MM]*pow(fg[2*J_x+0]*fg[2*(J_y+L[0])+1],2));
					fprintf(fp_o,"%.2E\n", Evu[J_x+L[0]*J_y          +Mode*MM]*pow(fg[2*J_x+1]*fg[2*(J_y+L[0])+0],2));
				}
			}
			fclose(fp_o);
		}
		free(ModeD);
		free(Evu);
		
		if(J_bt == 0) sprintf(Yeex,"%s_Bottom_check.dat", Yee);
		if(J_bt == 1) sprintf(Yeex,"%s_Top_check.dat", Yee);

		FILE *fp_o;
		fp_o = fopen(Yeex,"w");
		fprintf(fp_o,"# File name : ./%s on %s# Exe-file  : ./ModeCheck (Updated on Nov 14, 2014)\n", Yeex, ctime(&timer_s));
		if(solver_sw == 0) fprintf(fp_o,"# Eigenproblem solver: LAPACK dgeev (default)\n");
		if(solver_sw != 0) fprintf(fp_o,"# Eigenproblem solver: lgeev_ (long double-precision)\n");
		fprintf(fp_o,"# Max_norm within propagating modes = %.5E\n", Max_err[0]);
		fprintf(fp_o,"# Max_norm between propagating and evanescent modes = %.5E\n", Max_err[1]);
		fprintf(fp_o,"# total Max_norm= %.5E\n", Max_err[2]);
		fprintf(fp_o,"# %ld x %ld small matrix of (U^+MU)-1\n", wave_MM, wave_MM);
		fprintf(fp_o,"# Column Row |(U^+MU)-1| Arg((U^+MU)-1)\n\n");
		for(long KC = 0 ; KC < MM ; KC++) {
			if(creal(eigenvalue[KC]) < 0.) goto ColumnLast;
			for(long KR = 0 ; KR < MM ; KR++) {
				if(creal(eigenvalue[KR]) < 0.) goto RowLast;
				fprintf(fp_o,"%ld %ld %.20E %.20E\n", KC, KR, fabs(UMU[KR + MM*KC]), carg(UMU[KR + MM*KC]));
			}
			RowLast:;
			fprintf(fp_o,"\n");
		}
		ColumnLast:;
		fclose(fp_o);
		free(eigenvalue);
		free(UMU);
	}
	free(fg);
}
void read_epmu(char Yeex[], const long L[4], double complex epmu[6*L[0]*L[1]]);
void M_EH(const long L[4], const double complex epmu[6*L[0]*L[1]], const double fg[(L[0]+L[1])*2], 
			const double complex Psi_i[L[0]*L[1]*4], const long Pshift_i, 
			double complex Psi_o[L[0]*L[1]*4], const long Pshift_o);
void mode_calc(char Yeex[], const long L[4], const double fg[(L[0]+L[1])*2], double complex U_ini[4*L[0]*L[1]*L[0]*L[1]], double Max_err[3], double ModeD[4*L[0]*L[1]*L[0]*L[1]], double Evu[4*L[0]*L[1]*L[0]*L[1]], const double complex eigenvalue[2*L[0]*L[1]], double complex UMU[4*L[0]*L[1]*L[0]*L[1]])
{
// L[4]             : L[0] = L, L[1] = M, L[2] = B_u and L[3] = B_v.
// epmu[6*L[0]*L[1]]: epmu[l + L[0]*m + 0*L[0]*L[1]] = epsi_u(l,m), epmu[l + L[0]*m + 1*L[0]*L[1]] = epsi_v(l,m), epmu[l + L[0]*m + 2*L[0]*L[1]] = 1/epsi_w(l,m),
//                    epmu[l + L[0]*m + 3*L[0]*L[1]] =   mu_u(l,m), epmu[l + L[0]*m + 4*L[0]*L[1]] =   mu_v(l,m), epmu[l + L[0]*m + 5*L[0]*L[1]] =   1/mu_w(l,m).
// fg[(L[0]+L[1])*2]: fg[2*l + j] = f_j(l),  fg[2*(L[0]+m) + j] = g_j(m), where j = 0 or 1.
//
	double complex *epmu;
	epmu = malloc(sizeof(double complex)*6*L[0]*L[1]);
	read_epmu(Yeex, L, epmu);
	
	long MM = 2*L[0]*L[1];
	double complex *U_out;
	U_out = malloc(sizeof(double complex)*MM*MM);
	for(long J_column = 0 ; J_column < MM ; J_column++) {
		M_EH(L, epmu, fg, U_ini, J_column, U_out, J_column);
	}
	free(epmu);
	for(long J_column = 0 ; J_column < MM ; J_column++) {
		for(long J_row = 0 ; J_row < MM ; J_row++) {
			UMU[J_row + MM*J_column] = 0.;
			for(long J_M = 0 ; J_M < MM ; J_M++) {
				UMU[J_row + MM*J_column] += U_ini[J_M + MM*J_row]*U_out[J_M + MM*J_column];
			}
		}
	}
	for(long J_column = 0 ; J_column < MM ; J_column++) {
		UMU[J_column + MM*J_column] -= 1.;
	}
	long J_cut = 0;
	for(long J_column = 0 ; J_column < MM ; J_column++) {
		if(fabs(cimag(eigenvalue[J_column])) > 1.e-16 || creal(eigenvalue[J_column]) < 0) {
			J_cut = J_column;
			goto CUTOFF;
		}
	}
	CUTOFF:;
	for(long J_column = 0 ; J_column < MM ; J_column++) {
		for(long J_row = 0 ; J_row < MM ; J_row++) {
			double a = fabs(UMU[J_row + MM*J_column]);
			if(J_row < J_cut && J_column < J_cut) {
				if(Max_err[0] < a) Max_err[0] = a;
			}
			if((2*J_row - 2*J_cut + 1)*(2*J_column - 2*J_cut + 1) < 0) {
				if(Max_err[1] < a) Max_err[1] = a;
			}
			if(Max_err[2] < a) Max_err[2] = a;
		}
	}
	for(long J_column = 0 ; J_column < MM ; J_column++) {
		for(long J_row = 0 ; J_row < MM ; J_row++) {
			ModeD[J_row + MM*J_column] = creal(U_ini[J_row+ MM*J_column]*conj(U_out[J_row+ MM*J_column]));
			Evu[J_row + MM*J_column] = fabs(U_out[J_row+ MM*J_column]);
		}
	}
	free(U_out);
}
void read_epmu(char Yeex[], const long L[4], double complex epmu[6*L[0]*L[1]])
{
// L[4]             : L[0] = L, L[1] = M, L[2] = B_u and L[3] = B_v.
// epmu[6*L[0]*L[1]]: epmu[l + L[0]*m + 0*L[0]*L[1]] = epsi_u(l,m), epmu[l + L[0]*m + 1*L[0]*L[1]] = epsi_v(l,m), epmu[l + L[0]*m + 2*L[0]*L[1]] = 1/epsi_w(l,m),
//                    epmu[l + L[0]*m + 3*L[0]*L[1]] =   mu_u(l,m), epmu[l + L[0]*m + 4*L[0]*L[1]] =   mu_v(l,m), epmu[l + L[0]*m + 5*L[0]*L[1]] =   1/mu_w(l,m).
//
	char buf[BUFSIZE];
	FILE *fp_i;
	fp_i = fopen(Yeex,"r");
	if (fp_i == NULL){
		fprintf(stderr,"Do not open data file @ read_epmu!\n");
		exit(1);
	}
	long J_sum = 0;
	while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
		long NU, NV, SubN;
		double r_mu, i_mu, r_mv, i_mv, r_ew, i_ew;
		if(sscanf(buf,"%ld %ld %ld %lf %lf %lf %lf %lf %lf", 
			&NU, &NV, &SubN, &r_mu, &i_mu, &r_mv, &i_mv, &r_ew, &i_ew) == 9){
			if(NU == J_sum%L[0] && NV == (J_sum/L[0]) && SubN == 0){
				while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
					double r_eu, i_eu, r_ev, i_ev, r_mw, i_mw;
					if(sscanf(buf,"%ld %ld %ld %lf %lf %lf %lf %lf %lf",
						 &NU, &NV, &SubN, &r_ev, &i_ev, &r_eu, &i_eu, &r_mw, &i_mw) == 9){
						if(NU == J_sum%L[0] && NV == (J_sum/L[0]) && SubN == 1) {
							epmu[J_sum + 0*L[0]*L[1]] = r_eu + I*i_eu;
							epmu[J_sum + 1*L[0]*L[1]] = r_ev + I*i_ev;
							epmu[J_sum + 2*L[0]*L[1]] = 1./(r_ew + I*i_ew);
							epmu[J_sum + 3*L[0]*L[1]] = r_mu + I*i_mu;
							epmu[J_sum + 4*L[0]*L[1]] = r_mv + I*i_mv;
							epmu[J_sum + 5*L[0]*L[1]] = 1./(r_mw + I*i_mw);
							J_sum++;
							goto End_of_uv;
						}
					}
				}
			}
		}
		End_of_uv:;
		if(J_sum == L[0]*L[1]) goto End_of_data;
	}
	fprintf(stderr,"Read error @ read_epmu: File %s, J_sum = %ld, NU = %ld, NV = %ld\n", Yeex, J_sum, J_sum%L[0], J_sum/L[0]);
	exit(1);
	End_of_data:;

	if(fclose(fp_i) != 0) {
		fprintf(stderr,"fclose error after data file\n");
		exit(1);
	}
}
