/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||      Mesh3D creates three-dimensional Yee's lattice and sublattice. Feb/15/2011 ||||||||||||
||||||||||||                Main and sub programs were coded by Tatsuya Usuki.               ||||||||||||
||||||||||||                            Program language is C99.                             ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                     Please let me know any bug that you find.                   ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                                         http://homepage2.nifty.com/Pass_path/   ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                                                  Last updated on Dec 20, 2014   ||||||||||||
||||||||||||---------------------------------------------------------------------------------||||||||||||
||||||||||||     References:                                                                 ||||||||||||
||||||||||||     1) "WaveguideModeNum_ja2.pdf"                                               ||||||||||||
||||||||||||     2) "WageguideModeNum_v3.pdf"                                                ||||||||||||
||||||||||||     3) "Wageguide_Simulation_v1.pdf"                                            ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
//  Start of Mesh3D.h
#include <tgmath.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
//#include "constant.h"
#define BUFSIZE 1024 /* buffer for fgets */
#define Pi 3.141592653589793238462643 /* circular constant */
#define Pi2 6.283185307179586476925286

/*-------------------------------------------------------------------------------------------------------
//
//                        main ---- input_file1 
//                              |                 
//                              |-- input_fileM -- ScaleUnit
//                              |                 
//                               -- Calc_main -- Calc_F -- Generate_G -- LogMulCal_ini
//
//-----------------------------------------------------------------------------------------------------*/
void input_file1(FILE *fp_i1, long L[3], long K[3]);
void input_fileM(FILE *fp_i1, double x_max[3], double x_min[3], double dxdu[3]);
void Calc_main(char **argv, const long L[3], const long K[3], const double x_max[3], const double x_min[3], const double dxdu[3]);
//  End of Mesh3D.h

/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                              Main routine of Mesh3D.                            ||||||||||||
||||||||||||                            Last updated on Jul 08, 2012                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
int main(int argc, char **argv)
{
	if(argc != 3) {
		fprintf(stderr,"error: number of files \n");
		exit(1);
	}
	
	FILE *fp_i1;
	fp_i1 = fopen(argv[1],"r");
	if (fp_i1 == NULL){
		fprintf(stderr,"open error!: open input-file!\n");
		exit(1);
	}

	long L[3]; //division number: Lu = L[0], Lv = L[1], Lw = L[2]
	long K[3]; //K
	
	input_file1(fp_i1, L, K); // inputting parameters from an inputfile 1
	
	double x_max[3], x_min[3], dxdu[3];
	
	input_fileM(fp_i1, x_max, x_min, dxdu);
	
	fclose(fp_i1);
	
	Calc_main(argv, L, K, x_max, x_min, dxdu); // calc. of EM and outputting results to an outputfile
	
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                    input_file1                                  ||||||||||||
||||||||||||                            Last updated on Jul 29, 2012                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void input_file1(FILE *fp_i1, long L[3], long K[3])
{
	char buf[BUFSIZE];	// buffer for fgets
	for(int i0 = 0 ; i0 < 3 ; i0++){
		if(fgets(buf, sizeof buf, fp_i1) == NULL){
			fprintf(stderr,"fgets error!: division number \n");
			exit(1);
		}
		if(sscanf(buf,"%*[^=] %*[=] %ld %*[^=] %*[=] %ld", &L[i0], &K[i0]) != 2){
			fprintf(stderr,"input error!: division number \n"); //division number: Lu = L[0], Lv = L[1], see eqs.(38) in "WaveguideModeNum_ja2.pdf"
			exit(1);
		}
		if(L[i0] < 0 || K[i0] < 0) {
			fprintf(stderr,"division number!: L[%d] = %ld,  K[%d] = %ld\n", i0, L[i0], i0, K[i0]);
			exit(1);
		}
	}
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                    input_fileM                                  ||||||||||||
||||||||||||                            Last updated on Jul 29, 2012                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
double ScaleUnit(char x[]);
void input_fileM(FILE *fp_i1, double x_max[3], double x_min[3], double dxdu[3])
{
	char buf[BUFSIZE];	// buffer for fgets
	char A[BUFSIZE], B[BUFSIZE], C[BUFSIZE];
	for(int i0 = 0 ; i0 < 3 ; i0++){
		if(fgets(buf, sizeof buf, fp_i1) == NULL){
			fprintf(stderr,"fgets error!: x_max, min dx/du \n");
			exit(1);
		}
		if(sscanf(buf,"%*[^=] %*[=] %lf %s %*[^=] %*[=] %lf %s %*[^=] %*[=] %lf %s", &x_min[i0], A, &x_max[i0], B, &dxdu[i0], C) != 6){
			fprintf(stderr,"input error!: x_max, min dx/du \n");
			exit(1);
		}
		x_min[i0] *= ScaleUnit(A);
		x_max[i0] *= ScaleUnit(B);
		dxdu[i0] *= ScaleUnit(C);
		if(x_max[i0] < x_min[i0]) {
			fprintf(stderr,"x_max < x_min!: x_max[%d] = %8.2E, x_min[%d] = %8.2E\n", i0, x_max[i0], i0, x_min[i0]);
			exit(1);
		}
		if(dxdu[i0] < 0.) {
			fprintf(stderr,"dxdu!: dxdu[%d] = %8.2E \n", i0, dxdu[i0]);
			exit(1);
		}
	}
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                     ScaleUnit                                   ||||||||||||
||||||||||||                            Last updated on Jul 29, 2012                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
#include <string.h>
double ScaleUnit(char x[])
{
	double scale = -1.;
	if (strncmp(x, "m", 1) == 0) scale = 1e-0;// order of m, mm, micron is important!
	if (strncmp(x, "mm", 2) == 0) scale = 1e-3;// order of m, mm, micron is important!
	if (strncmp(x, "micron", 6) == 0) scale = 1e-6;// order of m, mm, micron is important!
	
	if (strncmp(x, "km", 2) == 0) scale = 1e3;
	if (strncmp(x, "cm", 2) == 0) scale = 1e-2;
	
	if (strncmp(x, "um", 2) == 0) scale = 1e-6;
	if (strncmp(x, "nm", 2) == 0) scale = 1e-9;
	if (strncmp(x, "deg", 3) == 0) scale = Pi/180.;
//	if (strncmp(x, "degree", 6) == 0) scale = Pi/180.;
	if (strncmp(x, "rad", 3) == 0) scale = 1.;
//	if (strncmp(x, "radian",6) == 0) scale = 1.;
	if (scale < 0.) {
		fprintf(stderr,"ScaleUnit error!  x = %s\n", x);
		exit(1);
	}
	return(scale);
}

/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                This program is Calc_main Program of Mesh3D. Feb/15/2011         ||||||||||||
||||||||||||                            Last updated on Oct 27, 2014                         ||||||||||||
||||||||||||                  Main and sub programs were coded by Tatsuya Usuki.             ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void Calc_F(const long L, const double xi[L], const long N_max, double F[L], double IntegF[L]);
void Calc_main(char **argv, const long L[3], const long K[3], const double x_max[3], const double x_min[3], const double dxdu[3])
{
	FILE *fp_o2;
	fp_o2 = fopen(argv[2],"w");
	if (fp_o2 == NULL){
		fprintf(stderr,"open error!:  output-epsilon\n");
		exit(1);
	}
	fprintf(fp_o2,"# Mesh number: L = %ld, M = %ld, N = %ld\n", L[0], L[1], L[2]);

	for(long i0 = 0 ; i0 < 3 ; i0++){
		fprintf(fp_o2,"\n\n");
		if(i0 == 0) fprintf(fp_o2,"# u0,                  x0[m],                  x1[m],                dxdu0[m],                dxdu1[m]\n");
		if(i0 == 1) fprintf(fp_o2,"# v0,                  y0[m],                  y1[m],                dydv0[m],                dydv1[m]\n");
		if(i0 == 2) fprintf(fp_o2,"# w0,                  z0[m],                  z1[m],                dzdw0[m],                dzdw1[m]\n");

//		double xi[2*L[i0]], F[2*L[i0]], IntegF[2*L[i0]];
		double *xi;
		xi = malloc(sizeof(double)*2*L[i0]);
		double *F;
		F = malloc(sizeof(double)*2*L[i0]);
		double *IntegF;
		IntegF = malloc(sizeof(double)*2*L[i0]);
		for(long J0 = 0 ; J0 < L[i0] ; J0++) { 
			double u0 = ((double) J0);
			xi[J0] = u0/((double) L[i0]);
			double u1 = 0.5 + ((double) J0);
			xi[J0 + L[i0]] = u1/((double) L[i0]);
		}
		
		Calc_F(2*L[i0], xi, K[i0], F, IntegF);
		
		for(long J0 = 0 ; J0 < L[i0] ; J0++) { 
			double diff_x = x_max[i0] - x_min[i0];
			double dxdu0 = dxdu[i0] + (diff_x/((double) L[i0]) - dxdu[i0])*F[J0];
			double dxdu1 = dxdu[i0] + (diff_x/((double) L[i0]) - dxdu[i0])*F[J0 + L[i0]];
			double x0 = ((double) L[i0])*(dxdu[i0]*xi[J0] + (diff_x/((double) L[i0]) - dxdu[i0])*IntegF[J0]);
			double x1 = ((double) L[i0])*(dxdu[i0]*xi[J0 + L[i0]] + (diff_x/((double) L[i0]) - dxdu[i0])*IntegF[J0 + L[i0]]);
			if(K[i0] == 0){
				dxdu0 = diff_x/((double) L[i0]);
				dxdu1 = dxdu0;
				x0 = diff_x*xi[J0];
				x1 = diff_x*xi[J0 + L[i0]];
			}
			x0 += x_min[i0];
			x1 += x_min[i0];
			fprintf(fp_o2,"%ld, %.20E, %.20E, %.20E, %.20E\n", J0, x0, x1, dxdu0, dxdu1);
		}
		fprintf(fp_o2,"%ld, %.20E\n", L[i0], x_max[i0]);
		free(xi);
		free(F);
		free(IntegF);
	}
	
	fprintf(fp_o2,"#\n");
	fprintf(fp_o2,"# Exe-file    : %s (Updated on Dec 20, 2014)\n", argv[0]);
	fprintf(stderr,"# Exe-file    : %s (Updated on Dec 20, 2014)\n", argv[0]);
	fprintf(fp_o2,"# Input file  : %s\n", argv[1]);
	fprintf(stderr,"# Input file  : %s\n", argv[1]);
	fprintf(fp_o2,"# Output file : %s (this file)\n", argv[2]);
	fprintf(fp_o2,"# See `Wageguide_Simulation_v1.pdf'\n");
	fprintf(fp_o2,"# K_u = %ld, K_v = %ld, K_w = %ld. ( K: F(xi, K) in Eq. (A1).)\n", K[0], K[1], K[2]);
	fprintf(fp_o2,"# Min(dx/du)[m] = %.2E, Min(dy/dv)[m] = %.2E, Min(dz/dw)[m] = %.2E.\n", dxdu[0], dxdu[1], dxdu[2]);
	fprintf(fp_o2,"# <<<<< Note that Min(dx/du) is unused when K_u = 0. >>>>>\n");
	fprintf(fp_o2,"# x_max[m] = %.2E, y_max[m] = %.2E, z_max[m] = %.2E.\n", x_max[0], x_max[1], x_max[2]);
	fprintf(fp_o2,"# x_min[m] = %.2E, y_min[m] = %.2E, z_min[m] = %.2E.\n", x_min[0], x_min[1], x_min[2]);
	time_t timer_f = time(0);
	fprintf(fp_o2,"# Created on %s\n", ctime(&timer_f));
	fprintf(stderr,"# Created on %s\n", ctime(&timer_f));
	fclose(fp_o2);
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                       Calc_F -- LogMulCal_ini -- Calc_F.  Feb/14/2011           ||||||||||||
||||||||||||                          Last updated on Jul 08, 2012                           ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void Generate_G(const long N_max, double G[N_max+1]);
double LogMulCal_ini(const long N1);
void Calc_F(const long L, const double xi[L], const long N_max, double F[L], double IntegF[L])
{
	if(N_max == 0){
		for(long ix = 0 ; ix < L ; ix++) {
			F[ix] = 1.;
			IntegF[ix] = xi[ix];
		}
	}
	else{
		double G[N_max + 1];
		Generate_G(N_max, G);
		for(long ix = 0 ; ix < L ; ix++) {
//			F[ix] = exp(G[0]+((double) N_max)*log(pow(cos(Pi*xi[ix]),2)));// Eq. (26) in "WageguideModeNum_v3.pdf"
			double ex_in = G[0]+((double) N_max*2)*log(fabs(cos(Pi*xi[ix])));//F is calculated by two step, 
			F[ix] = exp(ex_in);// because gcc 4.8.2 for ubuntu can not compile the above command(commented out), although Mingw 4.8.1 is OK.
			IntegF[ix] = 0.;
			for(long J = N_max ; J >= 1 ; J--){
				IntegF[ix] += G[J]*sin(Pi2*((double) J)*xi[ix])/(Pi2*((double) J));// Eq. (27) in "WageguideModeNum_v3.pdf"
			}
			IntegF[ix] += xi[ix];
		}
	}
}
void Generate_G(const long N_max, double G[N_max+1])
{
	G[0] = (((double) 2*N_max -1 )*log(2.) + LogMulCal_ini(N_max)); // G[0] is logarithm of G.
	
	for(long M = 1 ; M <= N_max ; M++){
		G[M] = 2.;
		for(long I1 = 1 ; I1 <= M ; I1++){
			G[M] *= ((double) N_max - M + I1)/((double) N_max + I1);
		}
	}
}
double LogMulCal_ini(const long N1)
{
	double Mul = 0.;
	
	for(long J = 1 ; J < N1 ; J++){
		Mul += log(((double) J)/((double) N1+J));
	}
	return(Mul);
}

