/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                Medium3D creates permititivity in each Yee's cell.  Oct/10/2014  ||||||||||||
||||||||||||                Main and sub programs were coded by Tatsuya Usuki.               ||||||||||||
||||||||||||                            Program language is C99.                             ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                     Please let me know any bug that you find.                   ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                                         http://homepage2.nifty.com/Pass_path/   ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                                                  Last updated on Dec 24, 2014   ||||||||||||
||||||||||||---------------------------------------------------------------------------------||||||||||||
||||||||||||     References:                                                                 ||||||||||||
||||||||||||     1) "WaveguideModeNum_ja2.pdf"                                               ||||||||||||
||||||||||||     2) "WageguideModeNum_v3.pdf"                                                ||||||||||||
||||||||||||     3) "Wageguide_Simulation_v1.pdf"                                            ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
#include <tgmath.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
//#include "constant.h"
#define BUFSIZE 1024 /* buffer for fgets */
#define Pi 3.141592653589793238462643 /* circular constant */
//#define Pi2 6.283185307179586476925286
//
//  main ---- input_file3 
//        |                 
//         -- input_file2
//        |                 
//         -- output_file
//
void input_file3(FILE *fp_i3, char Yee[], double env_para[2]);
int input_N_table(FILE *fp_i2);
void input_file2(FILE *fp_i2, const int N_table, const double env_para[2], double complex ep_omega[N_table], double complex mu_omega[N_table]);
void output_file(char **argv, const char Yee[], const int N_table, const double complex ep_omega[N_table], const double complex mu_omega[N_table]);
double complex epsi(const double Lambda, const double Temperature, char material[], const double charge_dens_cm3[2]);
//
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                             Main routine of YeeCells.                           ||||||||||||
||||||||||||                            Last updated on Oct 24, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
int main(int argc, char **argv)
{
	{
		time_t timer_ini = time(0);
		fprintf(stderr,"# Start time of Yee's lattice creation = %s\n", ctime(&timer_ini));
	}

	if(argc != 3) {
		fprintf(stderr,"error: number of files \n");
		exit(1);
	}
//---------- begin reading sub-mesh information and output file name ------------
	FILE *fp_i3;
	fp_i3 = fopen(argv[2],"r");
	if (fp_i3 == NULL){
		fprintf(stderr,"open error!: open input-file3!\n");
		exit(1);
	}
	fprintf(stderr,"\nMiscellaneous information (the 2nd input file): %s\n",argv[2]);
	char Yee[BUFSIZE];
	double env_para[2];// env_para[0]: wavelength, env_para[1]: temperature
	input_file3(fp_i3, Yee, env_para);
	fprintf(stderr,"Yee's lattice input file `%s*.dat'\n", Yee);
	fprintf(stderr,"wavelength = %g m, temperature = %g K\n", env_para[0], env_para[1]);
	if(fclose(fp_i3) != 0) {
		fprintf(stderr,"fclose error after input_file3\n");
		exit(1);
	}
//----------  end reading sub-mesh information and output file name  ------------

//---------- begin reading 3D-structure information ------------
	FILE *fp_i2;
	fp_i2 = fopen(argv[1],"r");
	if (fp_i2 == NULL){
		fprintf(stderr,"open error!: open medium-table (the 1st input file)!\n");
		exit(1);
	}
	fprintf(stderr,"\nMedium-table (the 2nd input file): %s\n",argv[1]);


	int N_table = input_N_table(fp_i2); //maximum of structure number
	rewind(fp_i2);
	
	fprintf(stderr,"N_table = %d\n", N_table);
	double complex ep_omega[N_table], mu_omega[N_table];

	input_file2(fp_i2, N_table, env_para, ep_omega, mu_omega);

	if(fclose(fp_i2) != 0) {
		fprintf(stderr,"fclose error after input_file2\n");
		exit(1);
	}
	for(int j0 = 0 ; j0 < N_table ; j0++) { 
		fprintf(stderr,"media_no = %d, ",j0);
		fprintf(stderr,"epsilon = (%.20E, %.20E), ", creal(ep_omega[j0]), cimag(ep_omega[j0]));
		fprintf(stderr,"mu = (%.20E, %.20E)\n", creal(mu_omega[j0]), cimag(mu_omega[j0]));
	}
//----------  end reading 3D-structure information  ------------
	
	output_file(argv, Yee, N_table, ep_omega, mu_omega);

	{
		time_t timer_f = time(0);
		fprintf(stderr,"\n# Finish time of Yee's lattice creation = %s", ctime(&timer_f));
	}
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                    output_file                                  ||||||||||||
||||||||||||                            Last updated on Oct 24, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void input_data_file0(const char Yee[], const int n_w, int L[4]);
void input_data_file(const char Yee[], const int n_w, const int L[4], const double complex ep_omega[], const double complex mu_omega[], double complex ep_cell1[8*L[0]*L[1]], double complex mu_cell1[8*L[0]*L[1]]);

void output_file(char **argv, const char Yee[], const int N_table, const double complex ep_omega[N_table], const double complex mu_omega[N_table])
{
// This 'output_file' creates a set of 3D files for optical media.
// Yee[]       : an output file name (char).
	int L[4];
	input_data_file0(Yee, 0, L);
	if(L[3] > N_table) {
		fprintf(stderr,"N_med > N_table!\n");
		exit(1);
	}
	fprintf(stderr,"From a data-file, L = %d, M = %d, N = %d, N_med = %d\n", L[0], L[1], L[2], L[3]);
	double complex ep_cell1[8*L[0]*L[1]], mu_cell1[8*L[0]*L[1]];
	input_data_file(Yee, 0, L, ep_omega, mu_omega, ep_cell1, mu_cell1);

	double complex ep_cell0[4*L[0]*L[1]], mu_cell0[4*L[0]*L[1]];
	for(int l_v = 0 ; l_v < L[1] ; l_v++){
		for(int l_u = 0 ; l_u < L[0] ; l_u++){
			for(int k_v = 0 ; k_v < 2 ; k_v++){
				for(int k_u = 0 ; k_u < 2 ; k_u++){
					ep_cell0[k_u+2*k_v + 4*l_u + 4*L[0]*l_v] = ep_cell1[k_u+2*k_v + 4*0 + 8*l_u + 8*L[0]*l_v];
					mu_cell0[k_u+2*k_v + 4*l_u + 4*L[0]*l_v] = mu_cell1[k_u+2*k_v + 4*0 + 8*l_u + 8*L[0]*l_v];
				}
			}
		}
	}
	for(int jw0 = 0 ; jw0 < L[2] ; jw0++) { 
		char output_name[BUFSIZE];
		sprintf(output_name,"%s_Med%d.dat", Yee, jw0);
		FILE *fp_o;
		fp_o = fopen(output_name,"w");
		if (fp_o == NULL){
			fprintf(stderr,"open error for %s\n", output_name);
			exit(1);
		}
		time_t timer_f = time(0);
		fprintf(fp_o,"# Created on %s", ctime(&timer_f));
		fprintf(fp_o,"# Exe-file    : %s (Updated on Dec 24, 2014)\n", argv[0]);
		fprintf(fp_o,"# Medium table: %s, Cell control file: %s\n", argv[1], argv[2]);
		fprintf(fp_o,"# l < %d, m < %d, n < %d, media_no < %d\n", L[0], L[1], L[2], L[3]);
		fprintf(fp_o,"# l m 0 mu_u mu_v ep_w\n");
		fprintf(fp_o,"# l m 1 ep_v ep_u mu_w\n");
		input_data_file(Yee, jw0, L, ep_omega, mu_omega, ep_cell1, mu_cell1);
		for(int l_v = 0 ; l_v < L[1] ; l_v++){
			int l_vm = (l_v + L[1] - 1)%L[1];
			for(int l_u = 0 ; l_u < L[0] ; l_u++){
				int l_um = (l_u + L[0] - 1)%L[0];
				{
					fprintf(fp_o,"%d %d 0 ", l_u, l_v);
					double mu_u, mu_v, ep_w;
					mu_u  = mu_cell1[0+2*0+4*0+8*l_u +8*L[0]*l_v ] + mu_cell1[1+2*0+4*0+8*l_u +8*L[0]*l_v ];
					mu_u += mu_cell1[0+2*1+4*0+8*l_u +8*L[0]*l_vm] + mu_cell1[1+2*1+4*0+8*l_u +8*L[0]*l_vm];
					mu_u += mu_cell0[0+2*0    +4*l_u +4*L[0]*l_v ] + mu_cell0[1+2*0    +4*l_u +4*L[0]*l_v ];
					mu_u += mu_cell0[0+2*1    +4*l_u +4*L[0]*l_vm] + mu_cell0[1+2*1    +4*l_u +4*L[0]*l_vm];
					mu_u *= 0.125;
					
					mu_v  = mu_cell1[0+2*0+4*0+8*l_u +8*L[0]*l_v ] + mu_cell1[0+2*1+4*0+8*l_u +8*L[0]*l_v ];
					mu_v += mu_cell1[1+2*0+4*0+8*l_um+8*L[0]*l_v ] + mu_cell1[1+2*1+4*0+8*l_um+8*L[0]*l_v ];
					mu_v += mu_cell0[0+2*0    +4*l_u +4*L[0]*l_v ] + mu_cell0[0+2*1    +4*l_u +4*L[0]*l_v ];
					mu_v += mu_cell0[1+2*0    +4*l_um+4*L[0]*l_v ] + mu_cell0[1+2*1    +4*l_um+4*L[0]*l_v ];
					mu_v *= 0.125;
					
					ep_w  = ep_cell1[0+2*0+4*0+8*l_u +8*L[0]*l_v ] + ep_cell1[0+2*1+4*0+8*l_u +8*L[0]*l_v ];
					ep_w += ep_cell1[1+2*0+4*0+8*l_u +8*L[0]*l_v ] + ep_cell1[1+2*1+4*0+8*l_u +8*L[0]*l_v ];
					ep_w += ep_cell0[0+2*0    +4*l_u +4*L[0]*l_v ] + ep_cell0[0+2*1    +4*l_u +4*L[0]*l_v ];
					ep_w += ep_cell0[1+2*0    +4*l_u +4*L[0]*l_v ] + ep_cell0[1+2*1    +4*l_u +4*L[0]*l_v ];
					ep_w *= 0.125;
					fprintf(fp_o,"%.20E %.20E %.20E %.20E %.20E %.20E\n",
						creal(mu_u),cimag(mu_v),creal(mu_v),cimag(mu_v),creal(ep_w),cimag(ep_w));
				}
				
				
				{
					fprintf(fp_o,"%d %d 1 ", l_u, l_v);
					double ep_v, ep_u, mu_w;
					ep_v  = ep_cell1[0+2*0+4*0+8*l_u +8*L[0]*l_v ] + ep_cell1[1+2*0+4*0+8*l_u +8*L[0]*l_v ];
					ep_v += ep_cell1[0+2*0+4*1+8*l_u +8*L[0]*l_v ] + ep_cell1[1+2*0+4*1+8*l_u +8*L[0]*l_v ];
					ep_v += ep_cell1[0+2*1+4*0+8*l_u +8*L[0]*l_vm] + ep_cell1[1+2*1+4*0+8*l_u +8*L[0]*l_vm];
					ep_v += ep_cell1[0+2*1+4*1+8*l_u +8*L[0]*l_vm] + ep_cell1[1+2*1+4*1+8*l_u +8*L[0]*l_vm];
					ep_v *= 0.125;
				
					ep_u  = ep_cell1[0+2*0+4*0+8*l_u +8*L[0]*l_v ] + ep_cell1[0+2*1+4*0+8*l_u +8*L[0]*l_v ];
					ep_u += ep_cell1[0+2*0+4*1+8*l_u +8*L[0]*l_v ] + ep_cell1[0+2*1+4*1+8*l_u +8*L[0]*l_v ];
					ep_u += ep_cell1[1+2*0+4*0+8*l_um+8*L[0]*l_v ] + ep_cell1[1+2*1+4*0+8*l_um+8*L[0]*l_v ];
					ep_u += ep_cell1[1+2*0+4*1+8*l_um+8*L[0]*l_v ] + ep_cell1[1+2*1+4*1+8*l_um+8*L[0]*l_v ];
					ep_u *= 0.125;
				
					mu_w  = mu_cell1[0+2*0+4*0+8*l_u +8*L[0]*l_v ] + mu_cell1[1+2*0+4*0+8*l_um+8*L[0]*l_v ];
					mu_w += mu_cell1[0+2*1+4*0+8*l_u +8*L[0]*l_vm] + mu_cell1[1+2*1+4*0+8*l_um+8*L[0]*l_vm];
					mu_w += mu_cell1[0+2*0+4*1+8*l_u +8*L[0]*l_v ] + mu_cell1[1+2*0+4*1+8*l_um+8*L[0]*l_v ];
					mu_w += mu_cell1[0+2*1+4*1+8*l_u +8*L[0]*l_vm] + mu_cell1[1+2*1+4*1+8*l_um+8*L[0]*l_vm];
					mu_w *= 0.125;
					fprintf(fp_o,"%.20E %.20E %.20E %.20E %.20E %.20E\n",creal(ep_v),cimag(ep_v),creal(ep_u),cimag(ep_u),creal(mu_w),cimag(mu_w));
				}
			}
		}
		if(fclose(fp_o) != 0) {
			fprintf(stderr,"fclose error after writing a data file!\n");
			exit(1);
		}
		for(int l_v = 0 ; l_v < L[1] ; l_v++){
			for(int l_u = 0 ; l_u < L[0] ; l_u++){
				for(int k_v = 0 ; k_v < 2 ; k_v++){
					for(int k_u = 0 ; k_u < 2 ; k_u++){
						ep_cell0[k_u+2*k_v + 4*l_u + 4*L[0]*l_v] = ep_cell1[k_u+2*k_v + 4*1 + 8*l_u + 8*L[0]*l_v];
						mu_cell0[k_u+2*k_v + 4*l_u + 4*L[0]*l_v] = mu_cell1[k_u+2*k_v + 4*1 + 8*l_u + 8*L[0]*l_v];
					}
				}
			}
		}
	}
}
void input_data_file(const char Yee[], const int n_w, const int L[4], const double complex ep_omega[], const double complex mu_omega[], double complex ep_cell[8*L[0]*L[1]], double complex mu_cell[8*L[0]*L[1]])
{
	FILE *fp_i;
	char input_name[BUFSIZE];
	sprintf(input_name,"%s%d.dat", Yee, n_w);//revised on Dec 18, 2014
	fp_i = fopen(input_name,"r");
	if (fp_i == NULL){
		fprintf(stderr,"open error for %s\n", input_name);
		exit(1);
	}
	char buf[BUFSIZE];
	int dummy[4];
	while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
		if(sscanf(buf,"%*[^<] %*[<] %d %*[^<] %*[<] %d %*[^<] %*[<] %d %*[^<] %*[<] %d", &dummy[0], &dummy[1], &dummy[2], &dummy[3]) == 4) goto End_of_LMN;
	}
	fprintf(stderr,"No L,M,N,N_med are read @ input_file1!\n");
	exit(1);
	End_of_LMN:;
	if(L[0] != dummy[0] || L[1] != dummy[1] || L[2] != dummy[2] || L[3] != dummy[3]){
		fprintf(stderr,"L,M,N,N_med are incorrect in %s!\n", input_name);
		exit(1);
	}
	
	for(int l_v = 0 ; l_v < L[1] ; l_v++){
		for(int l_u = 0 ; l_u < L[0] ; l_u++){
			while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
				int m_u, m_v;
				if(sscanf(buf,"%d %d", &m_u, &m_v) == 2) {
					if(m_u == l_u && m_v == l_v) goto Next_Step_0;
				}
			}
			fprintf(stderr,"l_u and l_v were not read @ input_data_file!\n");
			exit(1);
			Next_Step_0:;
			for(int k_w = 0 ; k_w < 2 ; k_w++){
				for(int k_v = 0 ; k_v < 2 ; k_v++){
					for(int k_u = 0 ; k_u < 2 ; k_u++){
						int N_med;
						while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
							int m_u, m_v, m_w;
							if(sscanf(buf,"%d %d %d %d", &m_u, &m_v, &m_w, &N_med) == 4) {
								if(m_u == k_u && m_v == k_v && m_w == k_w) goto Next_Step_1;
							}
						}
						fprintf(stderr,"k_u, k_v, k_w and N_med were not read @ input_data_file!\n");
						exit(1);
						Next_Step_1:;
						int j_med;
						double weight_med;
						double sum_weight_med = 0.;
						ep_cell[k_u+2*k_v + 4*k_w + 8*l_u + 8*L[0]*l_v] = 0.;
						mu_cell[k_u+2*k_v + 4*k_w + 8*l_u + 8*L[0]*l_v] = 0.;
						for(int med_no = 0 ; med_no < N_med ; med_no++){
							while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
								if(sscanf(buf,"%d %lf", &j_med, &weight_med) == 2) goto Next_Step_2;
							}
							fprintf(stderr,"j_med and weight_med were not read @ input_data_file!\n");
							exit(1);
							Next_Step_2:;
							sum_weight_med += weight_med;
							if(j_med >= L[3]) {
								fprintf(stderr,"j_med >= N_med! j_med = %d, N_med = %d\n", j_med, N_med);
								exit(1);
							}
							ep_cell[k_u+2*k_v + 4*k_w + 8*l_u + 8*L[0]*l_v] += weight_med*ep_omega[j_med];
							mu_cell[k_u+2*k_v + 4*k_w + 8*l_u + 8*L[0]*l_v] += weight_med*mu_omega[j_med];
						}
						if(sum_weight_med != 1.) {
							fprintf(stderr,"sum_weight_med = %g\n", sum_weight_med);
							exit(1);
						}
					}
				}
			}
		}
	}

	
	if(fclose(fp_i) != 0) {
		fprintf(stderr,"fclose error after reading a data file!\n");
		exit(1);
	}
}
void input_data_file0(const char Yee[], const int n_w, int L[4])
{
	FILE *fp_i;
	char input_name[BUFSIZE];
	sprintf(input_name,"%s%d.dat", Yee, n_w);//revised on Dec 18, 2014
	fp_i = fopen(input_name,"r");
	if (fp_i == NULL){
		fprintf(stderr,"open error for %s\n", input_name);
		exit(1);
	}
	char buf[BUFSIZE];
	while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
		if(sscanf(buf,"%*[^<] %*[<] %d %*[^<] %*[<] %d %*[^<] %*[<] %d %*[^<] %*[<] %d", &L[0], &L[1], &L[2], &L[3]) == 4) goto End_of_LMN;
	}
	fprintf(stderr,"No L,M,N,N_med are read @ input_file1!\n");
	exit(1);
	End_of_LMN:;
	if(fclose(fp_i) != 0) {
		fprintf(stderr,"fclose error after reading a data file!\n");
		exit(1);
	}
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                    input_file3                                  ||||||||||||
||||||||||||                            Last updated on Oct 17, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
double ScaleUnit(char x[]);// unit variation: km, m, cm, mm, micron, um, nm, deg, degree, rad, radian.
double TempUnit(double x, char s[]);
void input_file3(FILE *fp_i3, char Yee[], double env_para[2])
{
	char buf[BUFSIZE];	// buffer for fgets

	while(fgets(buf, sizeof( buf ), fp_i3) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %s", Yee) == 1) goto End_of_Yee;
	}
	fprintf(stderr,"No Yee-name is read @ input_file3!\n");
	exit(1);
	End_of_Yee:;
	int recursive_depth;
	while(fgets(buf, sizeof( buf ), fp_i3) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %d", &recursive_depth) == 1) goto End_of_recursive_depth;
	}
	fprintf(stderr,"No recursive_depth is read @ input_file3!\n");
	exit(1);
	End_of_recursive_depth:;

	char A_length[BUFSIZE], B_temperature[BUFSIZE];
	double dummy;
	while(fgets(buf, sizeof( buf ), fp_i3) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %lf %s %*[^=] %*[=] %lf %s", &env_para[0], A_length, &dummy, B_temperature) == 4) goto End_of_env_para;
	}
	fprintf(stderr,"No env_para is read @ input_file3!\n");
	exit(1);
	End_of_env_para:;

	env_para[0] *= ScaleUnit(A_length);
	env_para[1] = TempUnit(dummy, B_temperature);

}
double ScaleUnit(char x[])
{
	double scale = -1.;
	if (strncmp(x, "m", 1) == 0) scale = 1e-0;// order of m, mm, micron is important!
	if (strncmp(x, "mm", 2) == 0) scale = 1e-3;// order of m, mm, micron is important!
	if (strncmp(x, "micron", 6) == 0) scale = 1e-6;// order of m, mm, micron is important!
	
	if (strncmp(x, "km", 2) == 0) scale = 1e3;
	if (strncmp(x, "cm", 2) == 0) scale = 1e-2;
	
	if (strncmp(x, "um", 2) == 0) scale = 1e-6;
	if (strncmp(x, "nm", 2) == 0) scale = 1e-9;
	if (strncmp(x, "deg", 3) == 0) scale = Pi/180.;
//	if (strncmp(x, "degree", 6) == 0) scale = Pi/180.;
	if (strncmp(x, "rad", 3) == 0) scale = 1.;
//	if (strncmp(x, "radian",6) == 0) scale = 1.;
	if (scale < 0.) {
		fprintf(stderr,"ScaleUnit error!  x = %s\n", x);
		exit(1);
	}
	return(scale);
}
double TempUnit(double x, char s_char[])
{
	double offset_temp = -1.;
	double scale = -1.;
	if (strncmp(s_char, "K", 1) == 0) {
		scale = 1e-0;
		offset_temp = 0.;
	}// Kelvin
	if (strncmp(s_char, "degC", 4) == 0 || strncmp(s_char, "deg.C", 5) == 0 || strncmp(s_char, "deg_C", 5) == 0) {
		scale = 1.;
		offset_temp = 273.15;
	}// Celsius
	if (strncmp(s_char, "degF", 4) == 0 || strncmp(s_char, "deg.F", 5) == 0 || strncmp(s_char, "deg_F", 5) == 0) {
		scale = 5./9.;
		offset_temp = 459.67;
	}// Fahrenheit
	if (scale < 0. || offset_temp < -0.1) {
		fprintf(stderr,"TempUnit error!: %s\n", s_char);
		exit(1);
	}
	return(scale*(x + offset_temp));
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                   input_N_table                                 ||||||||||||
||||||||||||                            Last updated on Oct 22, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
int input_N_table(FILE *fp_i2)
{
	char buf[16], number_int[16];	// buffer for fgets.
	int n_table0 = 0;
	int n_table1 = -10;
	while(fgets(buf, sizeof( buf ), fp_i2) != NULL) {
		sscanf(buf,"%s", number_int);
		if (strcmp(number_int, "end") == 0||strcmp(number_int, "End") == 0||strcmp(number_int, "END") == 0) break;
		if(sscanf(number_int,"%d", &n_table1) == 1){
			if(n_table1 == n_table0) n_table0 += 1;
		}
	}
	return(n_table0);
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                    input_file2                                  ||||||||||||
||||||||||||                            Last updated on Oct 22, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void input_file2(FILE *fp_i2, const int N_table, const double env_para[2], double complex ep_omega[N_table], double complex mu_omega[N_table])
{
	char buf[BUFSIZE], command[16];	// buffer for fgets.
	int n_table0 = 0;
	int n_table1 = -10;
	while(fgets(buf, sizeof( buf ), fp_i2) != NULL) { 
		char number_int[16];
		sscanf(buf,"%s", number_int);
		if (strcmp(number_int, "end") == 0||strcmp(number_int, "End") == 0||strcmp(number_int, "END") == 0) break;
		if(sscanf(buf,"%d %s", &n_table1, command) == 2){
			if(n_table1 == n_table0){
				int dummy;
				mu_omega[n_table1] = 1.;
				if (strcmp(command, "name") == 0) {
					char material[BUFSIZE];
					double charge_dens_cm3[2];
					if(sscanf(buf,"%d %s %s %lf %lf", &dummy, command, material, &charge_dens_cm3[0], &charge_dens_cm3[1]) < 3){
						fprintf(stderr,"command, material error!: buf = %s", buf);
						exit(1);
					}
					ep_omega[n_table1] = epsi(env_para[0], env_para[1], material, charge_dens_cm3);
					goto EPSI;
				}
				if (strcmp(command, "index") == 0) {
					double Re_n, Im_n;
					if(sscanf(buf,"%d %s %*[^=] %*[=] %lf %*[^=] %*[=] %lf", &dummy, command, &Re_n, &Im_n) != 4){
						fprintf(stderr,"command, Re_n, Im_n error!: buf = %s", buf);
						exit(1);
					}
					ep_omega[n_table1] = (Re_n + I*Im_n)*(Re_n + I*Im_n);
					goto EPSI;
				}
				if (strcmp(command, "epsilon")== 0) {
					double Re_epsi, Im_epsi;
					if(sscanf(buf,"%d %s %*[^=] %*[=] %lf %*[^=] %*[=] %lf", &dummy, command, &Re_epsi, &Im_epsi) != 4){
						fprintf(stderr,"command, Re_epsi, Im_epsi error! `%s'\n", buf);
						exit(1);
					}
					ep_omega[n_table1] = Re_epsi + I*Im_epsi;
					goto EPSI;
				}
				if (strcmp(command, "epsilon+mu")== 0) {
					double Re_epsi, Im_epsi, Re_mu, Im_mu;
					if(sscanf(buf,"%d %s %*[^=] %*[=] %lf %*[^=] %*[=] %lf %*[^=] %*[=] %lf %*[^=] %*[=] %lf", &dummy, command, &Re_epsi, &Im_epsi, &Re_mu, &Im_mu) != 6){
						fprintf(stderr,"command, epsilon+mu error! `%s'\n", buf);
						exit(1);
					}
					ep_omega[n_table1] = Re_epsi + I*Im_epsi;
					mu_omega[n_table1] = Re_mu + I*Im_mu;
					goto EPSI;
				}
				fprintf(stderr,"cannot get epsilon! `%s'\n", buf);
				exit(1);
				EPSI:;
				n_table0 += 1;
			}
		}
		if(n_table0 == N_table) goto Step_end;
	}
	fprintf(stderr,"Never find final step!  n_table0 = %d, N_table = %d\n", n_table0, N_table);
	Step_end:;
}

