/* This program is Transfermatrix Method calc. Aug/14/2012    */
/*     Main and sub programs were coded by Tatsuya Usuki.     */
/*     See document file "Wageguide_Simulation_v4.pdf".       */
/*                            Last modified on Dec/24/2014    */
#include "STMM.h"

void Main_calc(const char Yee[], const double Lambda, const int LMN[3], const double dxdu[(LMN[0]+LMN[1]+LMN[2])*2], 
				const int Buv[2], const int solver_sw, const int method_sw)
{
//	char buf[BUFSIZE];// updated on Dec/17/2014
	long L[8]={LMN[0],LMN[1],Buv[0],Buv[1],0,0,LMN[2],0};
	// L[0]=L, L[1]=M, L[2]=B_u, L[3]=B_v, L[4]=J_b, L[5]=J_t, L[6]=N, L[7]=imag_cont.
	read_JbJt(Yee, L);
	//begin setting for inelastic scattering unitarity calc.
	if(method_sw == 1) cont_imag(Yee, L);
	if(method_sw == 1) fprintf(stderr,"#     cont_imag finished: imag_cont = %ld\n", L[7]);
	//long N_ad[L[7]], LMuvw[L[7]];
	long *N_ad, *LMuvw;
	N_ad = malloc(sizeof(long)*L[7]);
	LMuvw = malloc(sizeof(long)*L[7]);
	if(N_ad == NULL || LMuvw == NULL) exit(1);

	if(method_sw == 1) address_imag(Yee, L, N_ad, LMuvw);
	//double Imag_epmu[L[7]];
	double *Imag_epmu;
	Imag_epmu = malloc(sizeof(double)*L[7]);
	if(Imag_epmu == NULL) exit(1);
	
	if(method_sw == 1) gen_Imag_eqmu(Yee, Lambda, L, N_ad, LMuvw, Imag_epmu);//Imag_epmu = (c_0/(2*k_0^2))*Im(epsi, mu)
	//end setting for inelastic scattering unitarity calc.

	//double fg[(L[0]+L[1])*2], h[L[6]*2];// f_k (j) = fg[2*j + k] where 0 <= j < L[0] and k = 0 or 1,   g_k (j) = fg[2*(j+L[0]) + k] where 0 <= j < L[1] and k = 0 or 1,
	double *fg, *h;// f_k (j) = fg[2*j + k] where 0 <= j < L[0] and k = 0 or 1,   g_k (j) = fg[2*(j+L[0]) + k] where 0 <= j < L[1] and k = 0 or 1,
	fg = malloc(sizeof(double)*(L[0]+L[1])*2);
	h = malloc(sizeof(double)*L[6]*2);
	if(fg == NULL || h == NULL) exit(1);
	def_fgh(Lambda, L, dxdu, fg, h);
	
	//double gamma[L[4]+L[5]];// gamma_b (j) = gamma[j] where 0 <= j < J_b = L[4], gamma_t (j) = gamma[j + L[4]] where 0 <= j < L[5].
	double *gamma;// gamma_b (j) = gamma[j] where 0 <= j < J_b = L[4], gamma_t (j) = gamma[j + L[4]] where 0 <= j < L[5].
	gamma = malloc(sizeof(double)*(L[4]+L[5]));
	if(gamma == NULL) exit(1);
	def_gamma(Yee, Lambda, L, gamma);
	
	//double complex S_mat[(L[4]+L[5])*(L[4]+L[5])], Imag_mat[L[7]*(L[4]+L[5])];
	double complex *S_mat, *Imag_mat;
	S_mat = malloc(sizeof(double complex)*(L[4]+L[5])*(L[4]+L[5]));
	Imag_mat = malloc(sizeof(double complex)*L[7]*(L[4]+L[5]));
	if(S_mat == NULL || Imag_mat == NULL) exit(1);
	
	long MM = 2*L[0]*L[1];
	long row_column[2]={MM + L[4] + L[7], L[4]+L[5]};//forward calculation only, which is different from "Wageguide_Simulation_v3.pdf" and "STMM_20120927.zip."
	//double complex PsiL[row_column[0]*row_column[1]], PsiR[row_column[0]*MM];
	double complex *PsiL, *PsiR;
	PsiL = malloc(sizeof(double complex)*row_column[0]*row_column[1]);
	PsiR = malloc(sizeof(double complex)*row_column[0]*MM);
	if(PsiL == NULL || PsiR == NULL) exit(1);

	for(long n2 = 0 ; n2 < 2*L[6] ; n2++) {
		time_t timer_s = time(0);
		Transfer_step(Yee, n2, L, fg, h, row_column, PsiL, PsiR, N_ad, LMuvw);
		time_t timer_f = time(0);
		double t_span = difftime(timer_f,timer_s);
		fprintf(stderr,"# step=%ld: Transfer finished on %s# exec. time= %1.1f sec\n", n2, ctime(&timer_f), t_span);//revised on Dec 24, 2014
	}
	gen_S_mat(L, gamma, row_column, PsiL, S_mat);
	if(method_sw == 1) gen_Imag_mat(L, gamma, row_column, PsiL, Imag_mat);
	out_S_mat(Yee, L, S_mat, Imag_mat, Imag_epmu, solver_sw, method_sw);
	
	free(PsiL);
	free(PsiR);
	free(S_mat);
	free(Imag_mat);
	free(gamma);
	free(fg);
	free(h);
	free(Imag_epmu);
	free(N_ad);
	free(LMuvw);
}
void Transfer_step(const char Yee[], const long n2, const long L[8], const double fg[(L[0]+L[1])*2], const double h[L[6]*2], const long row_column[2], double complex PsiL[], double complex PsiR[],  const long N_ad[L[7]], const long LMuvw[L[7]])
{
	if(n2 == 0) Transfer0(Yee, L, fg, h, row_column, PsiL, PsiR);
	
	if(n2 > 0 && n2 < 2*L[6] -1) {
		if(n2%2 == 0) {
			Transfer2n(Yee, n2/2, L, fg, h, row_column, PsiL, PsiR);
			E_wavefun(Yee, n2/2, L, fg, h, row_column, PsiR, N_ad, LMuvw);
		}
		else {
			Transfer2n_1(Yee, (n2/2)+1, L, fg, h, row_column, PsiL, PsiR);
			H_wavefun(Yee, (n2/2)+1, L, fg, h, row_column, PsiR, N_ad, LMuvw);
		}
	}
	
	if(n2 == 2*L[6] -1) Transfer2Nm1(Yee, L, fg, h, row_column, PsiL, PsiR);
}
void gen_S_mat(const long L[7], const double gamma[L[4]+L[5]], const long row_column[2], const double complex PsiL[], double complex S_mat[(L[4]+L[5])*(L[4]+L[5])])
{
// PsiL[row_column[0]*row_column[1]]
	long MM = 2*L[0]*L[1];
	for(long KC = 0 ; KC < row_column[1] ; KC++) {
		for(long KR = 0 ; KR < L[4] ; KR++) {
			S_mat[KR + (L[4]+L[5])*KC] = (gamma[KR]/gamma[KC])*PsiL[MM + KR + row_column[0]*KC];//r-element, t'-element
		}
		for(long KR = 0 ; KR < L[5] ; KR++) {
			S_mat[KR + L[4] + (L[4]+L[5])*KC] = (gamma[KR + L[4]]/gamma[KC])*PsiL[KR + row_column[0]*KC];//t-element, r'-element
		}
	}
}
void gen_Imag_mat(const long L[8], const double gamma[L[4]+L[5]], const long row_column[2], const double complex PsiL[], double complex Imag_mat[L[7]*(L[4]+L[5])])
{
// PsiL[row_column[0]*row_column[1]]
	long MM = 2*L[0]*L[1];
	for(long KC = 0 ; KC < row_column[1] ; KC++) {
		for(long KR = 0 ; KR < L[7] ; KR++) {
			Imag_mat[KR + L[7]*KC] = (1./gamma[KC])*PsiL[MM + L[4] + KR + row_column[0]*KC];
		}
	}
}
double UnitaryCheck(const long L[7], const double complex S_mat[(L[4]+L[5])*(L[4]+L[5])], const double complex abs_mat[(L[4]+L[5])*(L[4]+L[5])], double complex s_check[(L[4]+L[5])*(L[4]+L[5])])
{
	double max_element = -1.;
	for(long KC = 0 ; KC < L[4] + L[5] ; KC++) {
		for(long KR = 0 ; KR < L[4]+L[5] ; KR++) {
			double complex s =  abs_mat[KR + (L[4]+L[5])*KC];
			for(long KK = 0 ; KK < L[4]+L[5] ; KK++) {
				s += conj(S_mat[KK + (L[4]+L[5])*KR]) * S_mat[KK + (L[4]+L[5])*KC];
			}
			if(KR == KC) {
				s -= 1.;
			}
			s_check[KR + (L[4]+L[5])*KC] = s; // This line was added on 2013.08.22.
			max_element = fmax(fabs(s), max_element);
		}
	}
	return max_element;
}
void out_S_mat(const char Yee[], const long L[8], const double complex S_mat[(L[4]+L[5])*(L[4]+L[5])], const double complex Imag_mat[L[7]*(L[4]+L[5])], const double Imag_epmu[L[7]], const int solver_sw, const int method_sw)
{
	char Yeex[BUFSIZE];
	sprintf(Yeex,"%s_final.dat", Yee);
	double complex *abs_mat;
	abs_mat = malloc(sizeof(double complex)*(L[4]+L[5])*(L[4]+L[5]));
	if(abs_mat == NULL) exit(1);
	for(long JC = 0 ; JC < L[4]+L[5] ; JC++) {
		for(long JL = 0 ; JL < L[4]+L[5] ; JL++) {
			double complex s = 0.;
			for(long J1 = 0 ; J1 < L[7] ; J1++) {
				s += conj(Imag_mat[J1+L[7]*JL])*Imag_epmu[J1]*Imag_mat[J1+L[7]*JC];
			}
			abs_mat[JL+(L[4]+L[5])*JC] = s;
		}
	}
	
	FILE *fp_o;
	fp_o = fopen(Yeex,"w");
	time_t timer_s = time(0);
	fprintf(fp_o,"# File name : ./%s on %s# Exe-file  : ./STMM (Updated on Dec 24, 2014)\n", Yeex, ctime(&timer_s));
	if(solver_sw == 0) fprintf(fp_o,"# Eigenproblem solver: LAPACK dgeev (default)\n");
	if(solver_sw != 0) fprintf(fp_o,"# Eigenproblem solver: lgeev_ (long double-precision)\n");
	if(method_sw == 1) fprintf(fp_o,"# Inelastic-scattering formulation solver\n");
	if(method_sw != 1) fprintf(fp_o,"# Scattering matrix solver (default)\n");
	double complex *s_check; 
	s_check = malloc(sizeof(double complex)*(L[4]+L[5])*(L[4]+L[5]));
	if(s_check == NULL) exit(1);
	
	fprintf(fp_o,"# Full-method(forward approach, heap area used): S-matrix unitarity error= %.5E\n", UnitaryCheck(L, S_mat, abs_mat, s_check));
	for(long KC = 0 ; KC < L[4]+L[5] ; KC++) {
		double refle = 0.;
		for(long KR = 0 ; KR < L[4] ; KR++) {
			double complex s = S_mat[KR + (L[4]+L[5])*KC];
			refle += (creal(s))*(creal(s)) + (cimag(s))*(cimag(s));
		}
		double trans = 0.;
		for(long KR = 0 ; KR < L[5] ; KR++) {
			double complex s = S_mat[KR+L[4] + (L[4]+L[5])*KC];
			trans += (creal(s))*(creal(s)) + (cimag(s))*(cimag(s));
		}
		double absorption = creal(abs_mat[KC+(L[4]+L[5])*KC]);
		fprintf(fp_o,"# ch.= %ld, Bottom output= %.5E, Top output= %.5E, Absorption= %.5E, error= %.5E\n", KC, refle, trans, absorption, fabs(trans + refle + absorption - 1.));
	}
	fprintf(fp_o,"# input-ch output-ch S_abs S_rad abs[(S^+)S+abs_mat-I] rad[(S^+)S+abs_mat-I]\n");// This line was revised on 2013.08.27
	for(long KC = 0 ; KC < L[4]+L[5] ; KC++) {
		for(long KR = 0 ; KR < L[4]+L[5] ; KR++) {
			fprintf(fp_o,"%ld %ld %.20E %.20E %.20E %.20E\n", KC, KR, fabs(S_mat[KR + (L[4]+L[5])*KC]), carg(S_mat[KR + (L[4]+L[5])*KC]), fabs(s_check[KR + (L[4]+L[5])*KC]), carg(s_check[KR + (L[4]+L[5])*KC]));
		}
		fprintf(fp_o,"\n");
	}
	fclose(fp_o);
	free(s_check);
	free(abs_mat);
}
void def_gamma(const char Yee[], const double Lambda, const long L[7], double gamma[L[4]+L[5]])
{
	char buf[BUFSIZE];
// L[0]=L, L[1]=M, L[2]=B_u, L[3]=B_v, L[4]=J_b, L[5]=J_t, L[6]=N.
	double k0 = Pi2/Lambda;
	for(long K_bt = 0 ; K_bt < 2 ; K_bt++) { 
		char Yeex[BUFSIZE];
		if(K_bt == 0) {
			sprintf(Yeex,"%sBottom.dat", Yee);
		}
		else {
			sprintf(Yeex,"%sTop.dat", Yee);
		}
		FILE *fp_i;
		fp_i = fopen(Yeex,"r");
		long J_bt = L[4 + K_bt];
		//double r_Beta[J_bt], i_Beta[J_bt];
		double *r_Beta, *i_Beta;
		r_Beta = malloc(sizeof(double)*J_bt);
		i_Beta = malloc(sizeof(double)*J_bt);
		if(r_Beta == NULL || i_Beta == NULL) exit(1);
		
/*		for(long J_column = 0 ; J_column < J_bt ; J_column++) {
			if(fgets(buf, sizeof buf, fp_i) == NULL){
				fprintf(stderr,"fgets error! at def_gamma!: E(%ld)\n", J_column);
				exit(1);
			}
			long K;
			double r_Lsqr, i_Lsqr;
			if(sscanf(buf,"%ld %lf %lf %lf %lf", &K, &r_Lsqr, &i_Lsqr, &r_Beta[J_column], &i_Beta[J_column]) != 5){
				fprintf(stderr,"input error at def_gamma!: step= %ld, line=%ld\n", K_bt, K);
				exit(1);
			}
			if(K != J_column){
				fprintf(stderr,"input error at def_gamma!: step= %ld, J_column= %ld, K= %ld\n", K_bt, J_column, K);
				exit(1);
			}
		}*/
//20141102		
		{
			long J_column = 0;
			while(fgets(buf, sizeof( buf ), fp_i) != NULL) {
			long K;
			double r_Lsqr, i_Lsqr;
				if(sscanf(buf,"%ld %lf %lf %lf %lf", &K, &r_Lsqr, &i_Lsqr, &r_Beta[J_column], &i_Beta[J_column]) == 5){
					if(K == J_column) J_column++;
					if(J_column == J_bt) goto End_of_eigenvalue;
				}
			}
			fprintf(stderr,"The %ld-th eigenvalue was not read @ %s in def_gamma!\n",J_column, Yeex);
			exit(1);
			End_of_eigenvalue:;
		}
//20141102end		
		fclose(fp_i);
		
		for(long J_column = 0 ; J_column < J_bt ; J_column++) {
//			if(r_Beta[J_column] < 0. || fabs(i_Beta[J_column]) > EPS){
			if(r_Beta[J_column] < 0. || fabs(i_Beta[J_column]) != 0.){//revised on Dec 24, 2014
				fprintf(stderr,"Beta error at def_gamma!: step= %ld, J_column= %ld\n", K_bt, J_column);
				exit(1);
			}
			gamma[J_column + K_bt*L[4]] = sqrt(c_0/tan(0.5*r_Beta[J_column]))/(2.*k0);
//			fprintf(stderr,"step= %ld, mode= %ld, gamma= %.5E\n", K_bt, J_column, gamma[J_column + K_bt*L[4]]);
		}
		free(r_Beta);
		free(i_Beta);
	}
}
void def_fgh(const double Lambda, const long L[7], const double dxdu[(L[0]+L[1]+L[6])*2], double fg[(L[0]+L[1])*2], double h[L[6]*2])
{// L[0]=L, L[1]=M, L[2]=B_u, L[3]=B_v, L[4]=J_b, L[5]=J_t, L[6]=N.
	double k0 = Pi2/Lambda;
	for(int J = 0 ; J < (L[0]+L[1])*2 ; J++) { 
		fg[J] = sqrt(1./(k0*dxdu[J]));
	}
	for(int J = 0 ; J < L[6]*2 ; J++) { 
		h[J] = sqrt(1./(k0*dxdu[J+(L[0]+L[1])*2]));
	}
}
void read_JbJt(const char Yee[], long L[7])
{
	char buf[BUFSIZE];
	for(long K_bt = 0 ; K_bt < 2 ; K_bt++) { 
		char Yeex[BUFSIZE];
		if(K_bt == 0) {
			sprintf(Yeex,"%sBottom.dat", Yee);
		}
		else {
			sprintf(Yeex,"%sTop.dat", Yee);
		}
		FILE *fp_i;
		fp_i = fopen(Yeex,"r");
		long MM = 2*L[0]*L[1];
		//double r_Beta[MM], i_Beta[MM];
		double *r_Beta, *i_Beta;
		r_Beta = malloc(sizeof(double)*MM);
		i_Beta = malloc(sizeof(double)*MM);
		if(r_Beta == NULL || i_Beta == NULL) exit(1);
		
/*		for(long J_column = 0 ; J_column < MM ; J_column++) {
			if(fgets(buf, sizeof buf, fp_i) == NULL){
				fprintf(stderr,"fgets error!: E(%ld)\n", J_column);
				exit(1);
			}
			long K;
			double r_Lsqr, i_Lsqr;
			if(sscanf(buf,"%ld %lf %lf %lf %lf", &K, &r_Lsqr, &i_Lsqr, &r_Beta[J_column], &i_Beta[J_column]) != 5){
				fprintf(stderr,"input error at read_JbJt!: step= %ld, line=%ld\n", K_bt, K);
				exit(1);
			}
			if(K != J_column){
				fprintf(stderr,"input error at read_JbJt!: step= %ld, J_column= %ld, K= %ld\n", K_bt, J_column, K);
				exit(1);
			}
		}*/
//20141102		
		{
			long J_column = 0;
			while(fgets(buf, sizeof( buf ), fp_i) != NULL) {
			long K;
			double r_Lsqr, i_Lsqr;
				if(sscanf(buf,"%ld %lf %lf %lf %lf", &K, &r_Lsqr, &i_Lsqr, &r_Beta[J_column], &i_Beta[J_column]) == 5){
					if(K == J_column) J_column++;
					if(J_column == MM) goto End_of_eigenvalue;
				}
			}
			fprintf(stderr,"The %ld-th eigenvalue was not read @ %s in read_JbJt!\n",J_column, Yeex);
			exit(1);
			End_of_eigenvalue:;
		}
//20141102end		
		fclose(fp_i);
		
		for(long J_column = 0 ; J_column < MM ; J_column++) {
//			if(i_Beta[J_column] > EPS){
			if(i_Beta[J_column] > 0.){//revised on Dec 24, 2014
				L[4 + K_bt] = J_column;
				goto DEF_J;
			}
		}

		fprintf(stderr,"End of eigenvalue at read_JbJt!: step= %ld\n", K_bt);
		exit(1);
		DEF_J:;
		free(r_Beta);
		free(i_Beta);
	}
	if(L[4] <= 0 || L[5] <= 0){
		fprintf(stderr,"J_b or J_t error!: J_b = %ld, J_t = %ld\n", L[4], L[5]);
		exit(1);
	}
	fprintf(stderr,"#     J_b = %ld, J_t = %ld\n", L[4], L[5]);
	
}
void cont_imag(const char Yee[], long L[8])
{
// L[0]=L, L[1]=M, L[2]=B_u, L[3]=B_v, L[4]=J_b, L[5]=J_t, L[6]=N, L[7]=imag_cont
// epmu[6*L[0]*L[1]]: epmu[l + L[0]*m + 0*L[0]*L[1]] = epsi_u(l,m), epmu[l + L[0]*m + 1*L[0]*L[1]] = epsi_v(l,m), epmu[l + L[0]*m + 2*L[0]*L[1]] = 1/epsi_w(l,m),
//                    epmu[l + L[0]*m + 3*L[0]*L[1]] =   mu_u(l,m), epmu[l + L[0]*m + 4*L[0]*L[1]] =   mu_v(l,m), epmu[l + L[0]*m + 5*L[0]*L[1]] =   1/mu_w(l,m).
//
	char Yeex[BUFSIZE];
	L[7] = 0;
	for(long n = 1 ; n < L[6]-1 ; n++) {
		//double complex epmu[6*L[0]*L[1]];
		double complex *epmu;
		epmu = malloc(sizeof(double complex)*6*L[0]*L[1]);
		if(epmu == NULL) exit(1);
		
		sprintf(Yeex,"%s_Med%ld.dat", Yee, n);
		read_epmu(Yeex, L, epmu);
		for(long m = 0 ; m < L[1] ; m++) {
			for(long l = 0 ; l < L[0] ; l++) {
				for(long uvw = 0 ; uvw < 6 ; uvw++) {
//					if(fabs(cimag(epmu[l + L[0]*m + uvw*L[0]*L[1]])) > EPS) L[7] += 1;
					if(fabs(cimag(epmu[l + L[0]*m + uvw*L[0]*L[1]])) > 0.) L[7] += 1;//revised on Dec 24, 2014
				}
			}
		}
		free(epmu);
	}
}
void address_imag(const char Yee[], const long L[8], long N_ad[L[7]], long LMuvw[L[7]])
{
// L[0]=L, L[1]=M, L[2]=B_u, L[3]=B_v, L[4]=J_b, L[5]=J_t, L[6]=N, L[7]=imag_cont
// epmu[6*L[0]*L[1]]: epmu[l + L[0]*m + 0*L[0]*L[1]] = epsi_u(l,m), epmu[l + L[0]*m + 1*L[0]*L[1]] = epsi_v(l,m), epmu[l + L[0]*m + 2*L[0]*L[1]] = 1/epsi_w(l,m),
//                    epmu[l + L[0]*m + 3*L[0]*L[1]] =   mu_u(l,m), epmu[l + L[0]*m + 4*L[0]*L[1]] =   mu_v(l,m), epmu[l + L[0]*m + 5*L[0]*L[1]] =   1/mu_w(l,m).
//
	char Yeex[BUFSIZE];
	long L_cont = 0;
	for(long n = 1 ; n < L[6]-1 ; n++) {
		//double complex epmu[6*L[0]*L[1]];
		double complex *epmu;
		epmu = malloc(sizeof(double complex)*6*L[0]*L[1]);
		if(epmu == NULL) exit(1);

		sprintf(Yeex,"%s_Med%ld.dat", Yee, n);
		read_epmu(Yeex, L, epmu);
		for(long m = 0 ; m < L[1] ; m++) {
			for(long l = 0 ; l < L[0] ; l++) {
				for(long uvw = 0 ; uvw < 6 ; uvw++) {
//					if(fabs(cimag(epmu[l + L[0]*m + uvw*L[0]*L[1]])) > EPS) {
					if(fabs(cimag(epmu[l + L[0]*m + uvw*L[0]*L[1]])) > 0.) {//revised on Dec 24, 2014
						N_ad[L_cont] = n;
						LMuvw[L_cont] = l + L[0]*m + uvw*L[0]*L[1];
						L_cont += 1;
					}
				}
			}
		}
		free(epmu);
	}
	if(L_cont == L[7]+1){
		fprintf(stderr,"address_imag error!: L_cont != L[7]+1\n");
		exit(1);
	}
}
void gen_Imag_eqmu(const char Yee[], const double Lambda, const long L[8], const long N_ad[L[7]], const long LMuvw[L[7]], double Imag_epmu[L[7]])
{
	double k0 = Pi2/Lambda;
	double c2k2 = 0.5*c_0/(k0*k0);
	//double epmu[6*L[0]*L[1]];
	double *epmu;
	epmu = malloc(sizeof(double)*6*L[0]*L[1]);
	if(epmu == NULL) exit(1);
	
	long n = -1;
	for(long J0 = 0 ; J0 < L[7] ; J0++) {
		if(n != N_ad[J0]) {
			n = N_ad[J0];
			char Yeex[BUFSIZE];
			sprintf(Yeex,"%s_Med%ld.dat", Yee, n);
			char buf[BUFSIZE];
			FILE *fp_i;
			fp_i = fopen(Yeex,"r");
			for(int JV = 0 ; JV < L[1] ; JV++) { 
				for(int JU = 0 ; JU < L[0] ; JU++) { 
					int NU, NV, SubN;
					if(fgets(buf, sizeof buf, fp_i) == NULL){
						fprintf(stderr,"fgets error!: %s %d, %d-0\n", Yeex, JU, JV);
						exit(1);
					}
					double r_mu, i_mu, r_mv, i_mv, r_ew, i_ew;
					if(sscanf(buf,"%d %d %d %lf %lf %lf %lf %lf %lf", &NU, &NV, &SubN, &r_mu, &i_mu, &r_mv, &i_mv, &r_ew, &i_ew) != 9){
						fprintf(stderr,"input error!: %s %d, %d-0\n", Yeex, JU, JV);
						exit(1);
					}
					if(NU != JU || NV != JV || SubN != 0){
						fprintf(stderr,"File %s, NU != JU || NV != JV || SubN != 0: %d, %d, %d \n", Yeex, NU, NV, SubN);
						exit(1);
					}
					if(fgets(buf, sizeof buf, fp_i) == NULL){
						fprintf(stderr,"fgets error!: %s, %d, %d-1\n", Yeex, JU, JV);
						exit(1);
					}
					double r_eu, i_eu, r_ev, i_ev, r_mw, i_mw;
					if(sscanf(buf,"%d %d %d %lf %lf %lf %lf %lf %lf", &NU, &NV, &SubN, &r_eu, &i_eu, &r_ev, &i_ev, &r_mw, &i_mw) != 9){
						fprintf(stderr,"input error!: %s, %d, %d-1\n", Yeex, JU, JV);
						exit(1);
					}
					if(NU != JU || NV != JV || SubN != 1){
						fprintf(stderr,"File %s, NU != JU || NV != JV || SubN != 1: %d, %d, %d \n", Yeex, NU, NV, SubN);
						exit(1);
					}
					epmu[JU + L[0]*JV + 0*L[0]*L[1]] = i_eu;
					epmu[JU + L[0]*JV + 1*L[0]*L[1]] = i_ev;
					epmu[JU + L[0]*JV + 2*L[0]*L[1]] = i_ew;
					epmu[JU + L[0]*JV + 3*L[0]*L[1]] = i_mu;
					epmu[JU + L[0]*JV + 4*L[0]*L[1]] = i_mv;
					epmu[JU + L[0]*JV + 5*L[0]*L[1]] = i_mw;
				}
			}
			fclose(fp_i);
		}
		Imag_epmu[J0] = c2k2*epmu[LMuvw[J0]];
	}
	free(epmu);
}
