/* This header file is for Stable Transfer Matrix Method (STMM) simulation.  Sep/05/2012   */
/*                See document file "Wageguide_Simulation_v4.pdf."                         */
/*                                                         Last modified on Aug/22/2013    */

#include <tgmath.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "constant.h"
//
//
//                          "STMM.h" covers the following functions.
//
//
//  Main_calc ----- read_JbJt
//              |
//              |-- cont_imag ---- read_epmu
//              |
//              |-- address_imag ---- read_epmu
//              |
//              |-- gen_Imag_eqmu
//              |
//              |-- def_fgh
//              |
//              |-- def_gamma
//              |
//              |----------------- Transfer_step  ----- Transfer0 -------- read_tU
//              |                                   |                  |
//              |-- gen_S_mat                       |                   -- multiplyMat
//              |                                   |
//              |-- gen_Imag_mat                    |
//              |                                   |
//               -- out_S_mat ---- UnitaryCheck     |-- Transfer2n_1 ----- read_epmu
//                                                  |                  |
//                                                  |                   -- M_HE
//                                                  |                  |
//                                                  |                   -- Inv_calc
//                                                  |                  |
//                                                  |                   -- RtoL
//                                                  |
//                                                  |-- H_wavefun -------- E_w
//                                                  |
//                                                  |-- Transfer2n ------- read_epmu
//                                                  |                  |
//                                                  |                   -- M_EH
//                                                  |                  |
//                                                  |                   -- Inv_calc
//                                                  |                  |
//                                                  |                   -- RtoL
//                                                  |
//                                                  |-- E_wavefun -------- H_w
//                                                  |
//                                                   -- Transfer2Nm1 ----- read_tU
//                                                                     |
//                                                                      -- read_epmu
//                                                                     |
//                                                                      -- M_EH
//                                                                     |
//                                                                      -- Inv_calc
//                                                                     |
//                                                                      -- RtoL
//
void read_JbJt(const char Yee[], long L[7]);
void cont_imag(const char Yee[], long L[8]);
void address_imag(const char Yee[], const long L[8], long N_ad[L[7]], long LMuvw[L[7]]);
void gen_Imag_eqmu(const char Yee[], const double Lambda, const long L[8], const long N_ad[L[7]], const long LMuvw[L[7]], double Imag_epmu[L[7]]);
void def_fgh(const double Lambda, const long L[7], const double dxdu[(L[0]+L[1]+L[6])*2], double fg[(L[0]+L[1])*2], double h[L[6]*2]);
void def_gamma(const char Yee[], const double Lambda, const long L[7], double gamma[L[4]+L[5]]);
void Transfer_step(const char Yee[], const long n2, const long L[8], const double fg[(L[0]+L[1])*2], const double h[L[6]*2], const long row_column[2], double complex PsiL[], double complex PsiR[],  const long N_ad[L[7]], const long LMuvw[L[7]]);
void Transfer0(const char Yee[], const long L[8], const double fg[(L[0]+L[1])*2], const double h[L[6]*2], const long row_column[2], double complex PsiL[], double complex PsiR[]);

void Transfer2n_1(const char Yee[], const long n, const long L[7], const double fg[(L[0]+L[1])*2], const double h[L[6]*2], const long row_column[2], double complex PsiL[], double complex PsiR[]);
void H_wavefun(const char Yee[], const long n, const long L[8], const double fg[(L[0]+L[1])*2], const double h[L[6]*2], const long row_column[2], double complex PsiR[], const long N_ad[L[7]], const long LMuvw[L[7]]);
void cal_Ew(const long L[8], const double complex epmu[6*L[0]*L[1]], const double fg[(L[0]+L[1])*2], const double complex H_uv[L[0]*L[1]*2], double complex E_w[L[0]*L[1]]);

void Transfer2n(const char Yee[], const long n, const long L[7], const double fg[(L[0]+L[1])*2], const double h[L[6]*2], const long row_column[2], double complex PsiL[], double complex PsiR[]);
void E_wavefun(const char Yee[], const long n, const long L[8], const double fg[(L[0]+L[1])*2], const double h[L[6]*2], const long row_column[2], double complex PsiR[], const long N_ad[L[7]], const long LMuvw[L[7]]);
void cal_Hw(const long L[8], const double complex epmu[6*L[0]*L[1]], const double fg[(L[0]+L[1])*2], const double complex E_vu[L[0]*L[1]*2], double complex H_w[L[0]*L[1]]);

void Transfer2Nm1(const char Yee[], const long L[7], const double fg[(L[0]+L[1])*2], const double h[L[6]*2], const long row_column[2], double complex PsiL[], double complex PsiR[]);
void gen_S_mat(const long L[7], const double gamma[L[4]+L[5]], const long row_column[2], const double complex PsiL[], double complex S_mat[(L[4]+L[5])*(L[4]+L[5])]);
void gen_Imag_mat(const long L[8], const double gamma[L[4]+L[5]], const long row_column[2], const double complex PsiL[], double complex Imag_mat[L[7]*(L[4]+L[5])]);
void out_S_mat(const char Yee[], const long L[8], const double complex S_mat[(L[4]+L[5])*(L[4]+L[5])], const double complex Imag_mat[L[7]*(L[4]+L[5])], const double Imag_epmu[L[7]], const int solver_sw, const int method_sw);
double UnitaryCheck(const long L[7], const double complex S_mat[(L[4]+L[5])*(L[4]+L[5])], const double complex abs_mat[(L[4]+L[5])*(L[4]+L[5])], double complex s_check[(L[4]+L[5])*(L[4]+L[5])]);


void read_tU(char Yeex[], const long MM, double complex theta[MM], double complex Ub[MM*MM]);
void multiplyMat(const long MM, const double complex Ub[MM*MM], double complex A[MM], const long shift_line);


void read_epmu(char Yeex[], const long L[4], double complex epmu[6*L[0]*L[1]]);
void Inv_calc(const long MM, const long row_num, double complex PsiR[row_num*MM], double complex Ref_R[MM*MM]);
void RtoL(const long MM, const long row_column[2], double complex PsiL[], double complex PsiR[], double complex Ref_L[MM*row_column[1]]);
void M_HE(const long L[4], const double complex epmu[6*L[0]*L[1]], const double fg[(L[0]+L[1])*2], const double complex Psi_i[L[0]*L[1]*4], const long Pshift_i, double complex Psi_o[L[0]*L[1]*4], const long Pshift_o);
void M_EH(const long L[4], const double complex epmu[6*L[0]*L[1]], const double fg[(L[0]+L[1])*2], const double complex Psi_i[L[0]*L[1]*4], const long Pshift_i, double complex Psi_o[L[0]*L[1]*4], const long Pshift_o);

