/*     This program is Transfer2Nm1 at Top.     Aug/16/2012   */
/*     Main and sub programs were coded by Tatsuya Usuki.     */
/*                            Last modified on Dec/24/2014    */
#include "STMM.h"
void Transfer2Nm1(const char Yee[], const long L[7], const double fg[(L[0]+L[1])*2], const double h[L[6]*2], const long row_column[2], double complex PsiL[], double complex PsiR[])
{
// L[0]=L, L[1]=M, L[2]=B_u, L[3]=B_v, L[4]=J_b, L[5]=J_t, L[6]=N.
	long MM = 2*L[0]*L[1];
	long n_step = 2*L[6] - 1;
	double complex hhi = -h[n_step]*h[n_step]*I;// Eq.(26) in "Wageguide_Simulation_v4.pdf"
	//double complex theta[MM], Ut[MM*MM], MUph[MM*L[5]];
	double complex *theta, *Ut, *MUph;
	theta = malloc(sizeof(double complex)*MM);
	Ut = malloc(sizeof(double complex)*MM*MM);
	MUph = malloc(sizeof(double complex)*MM*L[5]);
	if(theta == NULL || Ut == NULL || MUph == NULL) exit(1);
	
	//double complex epmu[6*L[0]*L[1]];
	double complex *epmu;
	epmu = malloc(sizeof(double complex)*6*L[0]*L[1]);
	if(epmu == NULL) exit(1);
	double complex *Bsub0, *Bsub1;
	Bsub0 = malloc(sizeof(double complex)*MM);
	Bsub1 = malloc(sizeof(double complex)*MM);
	if(Bsub0 == NULL || Bsub1 == NULL) exit(1);
	{
		char Yeex[BUFSIZE];
		sprintf(Yeex,"%sTop.dat", Yee);
		read_tU(Yeex, MM, theta, Ut);
		sprintf(Yeex,"%s_Med%ld.dat", Yee, L[6] - 1);//20141104 revised
		read_epmu(Yeex, L, epmu);
		for(long J_column = 0 ; J_column < L[5] ; J_column++) {
			//double complex Bsub0[MM], Bsub1[MM];
			for(long J_row = 0 ; J_row < MM ; J_row++) {
				Bsub0[J_row] = Ut[J_row + MM*J_column]/(hhi*(1.0 - theta[J_column]));
			}
			M_EH(L, epmu, fg, Bsub0, 0, Bsub1, 0);
			for(long J_row = 0 ; J_row < MM ; J_row++) {
				MUph[J_row + MM*J_column] = Bsub1[J_row];
			}
		}
	}
	
//	double complex PsiL[row_column[0]*row_column[1]], PsiR[row_column[0]*MM];
	
	for(long J_column = 0 ; J_column < L[5]; J_column++) {// just before Eq.(29) in "Wageguide_Simulation_v4.pdf"
		for(long J_row = 0 ; J_row < row_column[0] ; J_row++) {
			double complex aa = 0.;
			for(long J_2 = 0 ; J_2 < MM ; J_2++) {
				aa += PsiR[J_row + row_column[0]*J_2]*MUph[J_2 + MM*J_column];
			}
			PsiL[J_row + row_column[0]*(J_column+L[4])] = aa;
		}
		for(long J_row = 0 ; J_row < MM ; J_row++) {
			PsiL[J_row + row_column[0]*(J_column+L[4])] -= Ut[J_row + MM*J_column];
		}
	}

/////////////////////////////////////////////////////////////////////////////////////////////////////

	//double complex Ref_L[MM*row_column[1]];//	row_column[0]: row number of PsiL and PsiR, row_column[1]: column number of PsiL;
	double complex *Ref_L;//	row_column[0]: row number of PsiL and PsiR, row_column[1]: column number of PsiL;
	Ref_L = malloc(sizeof(double complex)*MM*row_column[1]);
	if(Ref_L == NULL) exit(1);
	for(long J_column = 0 ; J_column < row_column[1] ; J_column++) {
		for(long J_row = 0 ; J_row < MM ; J_row++) {
			Ref_L[J_row + MM*J_column] = PsiL[J_row + row_column[0]*J_column];
			PsiL[J_row + row_column[0]*J_column] = 0.;
		}
	}


	//double complex Ref_R[MM*MM];
	double complex *Ref_R;
	Ref_R = malloc(sizeof(double complex)*MM*MM);
	if(Ref_R == NULL) exit(1);
	for(long J_column = 0 ; J_column < MM ; J_column++) {
		//double complex Bsub0[MM], Bsub1[MM];
		for(long J_row = 0 ; J_row < MM ; J_row++) {
			double complex Theta_inv = 1.0/theta[J_row];// Eq.(26) in "Wageguide_Simulation_v4.pdf"
			Bsub0[J_row] = hhi*(1.0 - Theta_inv)*Ut[J_column + MM*J_row];// forward: -i*h_t^2*(1 - 1/Theta)*Ut^T
			Bsub1[J_row] = Bsub0[J_row];
		}
		multiplyMat(MM, Ut, Bsub1, 0);// forward: -i*h_t^2*Ut*(1 - 1/Theta)*Ut^T
		for(long J_row = 0 ; J_row < MM ; J_row++) {
			double complex a = PsiR[J_row + row_column[0]*J_column] - Bsub1[J_row];// forward: C_01 + i*h_t^2*(1 - 1/Theta)*Ut^T
			PsiR[J_row + row_column[0]*J_column] = Bsub0[J_row];// forward: -i*h_t^2*(1 - 1/Theta)*Ut^T for Eqs. (20) and (28)
			Ref_R[J_row + MM*J_column]  = a;
//			if(fabs(a) < EPS || 1./fabs(a) < EPS){
			if(fabs(a) == 0. || 1./fabs(a) == 0.){//revised on Dec 24, 2014
				fprintf(stderr,"Ref_R error at Inv_calc!: line-column= %ld,%ld, |Ref_R|= %.5E\n", J_row, J_column, fabs(a));
				exit(1);
			}
		}
	}
	free(Bsub0);
	free(Bsub1);
	free(theta);
	free(Ut);
	free(MUph);
	
	Inv_calc(MM, row_column[0], PsiR, Ref_R);
	RtoL(MM, row_column, PsiL, PsiR, Ref_L);
	
	free(Ref_L);
	free(Ref_R);
}
