/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||This program is Main Program of Stable Transfer Matrix Method (STMM). Aug/14/2012||||||||||||
||||||||||||                Main and sub programs were coded by Tatsuya Usuki.               ||||||||||||
||||||||||||                            Program language is C99.                             ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                     Please let me know any bug that you find.                   ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                                         http://homepage2.nifty.com/Pass_path/   ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                                                  Last updated on Dec 24, 2014   ||||||||||||
||||||||||||---------------------------------------------------------------------------------||||||||||||
||||||||||||     References:                                                                 ||||||||||||
||||||||||||     1) "Wageguide_Simulation_v4.pdf"                                            ||||||||||||
||||||||||||     2) PRB vol.52 p.8244 '95.                                                   ||||||||||||
||||||||||||     3) J.Comput.Electron. vol.9 p.232 '10.                                      ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include "constant.h"
void input_file1(FILE *fp_i1, int L[3]);
void input_fileM(FILE *fp_i1, const int LMN[3], double dxdu[(LMN[0]+LMN[1]+LMN[2])*2]);
void input_file2(FILE *fp_i2, double *Lambda, char Yee[], int Buv[2], int *solver_sw, int *method_sw);
void Main_calc(const char Yee[], const double Lambda, const int LMN[3], const double dxdu[(LMN[0]+LMN[1]+LMN[2])*2], 
				const int Buv[2], const int solver_sw, const int method_sw);
int main(int argc, char **argv)
{
	time_t timer_s = time(0);
	fprintf(stderr,"# Stable Transfer Method start time= %s\n", ctime(&timer_s));
	if(argc != 3) {
		fprintf(stderr,"error: number of files \n");
		exit(1);
	}
	
	FILE *fp_i1;
	fp_i1 = fopen(argv[1],"r");
	if (fp_i1 == NULL){
		fprintf(stderr,"open error!: open input-file1!\n");
		exit(1);
	}

	int LMN[3]; //division number: LMN[0] = L, LMN[1] = M, LMN[2] = N
	
	input_file1(fp_i1, LMN); // inputting parameters from an inputfile 1
	
	double *dxdu;
	dxdu = (double *)malloc(sizeof(double)*(LMN[0]+LMN[1]+LMN[2])*2);
	                                      // dxdu_k (j) = dxdu[2*j + k] where 0 <= j < LMN[0] and k = 0 or 1,
	                                      // dydv_k (j) = dxdu[2*(j+LMN[0]) + k] where 0 <= j < LMN[1] and k = 0 or 1,
	                                      // dzdw_k (j) = dxdu[2*(j+LMN[0]+LMN[1]) + k] where 0 <= j < LMN[2] and k = 0 or 1.
	if (dxdu == NULL) exit(1);
	input_fileM(fp_i1, LMN, dxdu);
	fclose(fp_i1);
	
	int j_shift = 0;
	for(int J = 0 ; J < 3 ; J++) { 
		if (J == 0) fprintf(stderr,"\n#   L, dxdu_0, dxdu_1\n");
		if (J == 1) fprintf(stderr,"\n#   M, dydv_0, dydv_1\n");
		if (J == 2) fprintf(stderr,"\n#   N, dzdw_0, dzdw_1\n");
		for(int K = 0 ; K < LMN[J] ; K++) { 
			fprintf(stderr,"#   %d, %.5E, %.5E\n", K, dxdu[2*(K+j_shift)+0], dxdu[2*(K+j_shift)+1]);
		}
		j_shift += LMN[J];
	}
	
	char Yee[BUFSIZE];
	double Lambda;
	FILE *fp_i2;
	fp_i2 = fopen(argv[2],"r");
	if (fp_i2 == NULL){
		fprintf(stderr,"open error!: open input-file2!\n");
		exit(1);
	}
	int Buv[2]; //boundary conditions: B_u = Buv[0], B_v = Buv[1]
	int solver_sw, method_sw;
	input_file2(fp_i2, &Lambda, Yee, Buv, &solver_sw, &method_sw);
	fclose(fp_i2);
	
	Main_calc(Yee, Lambda, LMN, dxdu, Buv, solver_sw, method_sw);
	
	time_t timer_f = time(0);
	double t_span = difftime(timer_f,timer_s);
	fprintf(stderr,"# Stable Transfer Method finished on %s# total exec. time= %1.1f sec\n", ctime(&timer_f), t_span);//revised on Dec 24, 2014
	
	free(dxdu);
	return 0;
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                              input_file1, input_fileM                           ||||||||||||
||||||||||||                            Last updated on Nov 01, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void input_file1(FILE *fp_i, int L[3])
{
	char buf[BUFSIZE];	// buffer for fgets
	while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %d %*[^=] %*[=] %d %*[^=] %*[=] %d", &L[0], &L[1], &L[2]) == 3) goto End_of_LMN;
	}
	fprintf(stderr,"No L,M,N are read @ input_file1!\n");
	exit(1);
	End_of_LMN:;

	if(L[0] < 1 || L[1] < 1 || L[2] < 1) {
		fprintf(stderr,"L,M,N input error!: L = %d, M = %d, N = %d \n", L[0], L[1], L[2]);
		exit(1);
	}
}
void input_fileM(FILE *fp_i1, const int LMN[3], double dxdu[(LMN[0]+LMN[1]+LMN[2])*2])
{
	int j_shift = 0;
	for(int Step_LMN = 0 ; Step_LMN < 3 ; Step_LMN++){
		char buf[BUFSIZE];	// buffer for fgets
		int i0 = 0;
		while(fgets(buf, sizeof( buf ), fp_i1) != NULL) {
			int i1;
			double x0,x1;
			if(sscanf(buf,"%d %*[,] %lf  %*[,] %lf %*[,] %lf %*[,] %lf", 
				&i1, &x0, &x1, &dxdu[2*(i0+j_shift)+0], &dxdu[2*(i0+j_shift)+1]) == 5){
				if(i1 == i0) i0++;
				if(i0 == LMN[Step_LMN]) goto End_of_x_data;
			}
		}
		fprintf(stderr,"The %d-th data were not read for Step_LMN = %d @ input_fileM!\n",i0, Step_LMN);
		exit(1);
		End_of_x_data:;

		j_shift += LMN[Step_LMN];
	}

}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                    input_file2                                  ||||||||||||
||||||||||||                            Last updated on Nov 02, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
double ScaleUnit(char x[]);// unit variation: km, m, cm, mm, micron, um, nm, deg, degree, rad, radian.
double TempUnit(double x, char s[]);
void input_file2(FILE *fp_i2, double *Lambda, char Yee[], int Buv[2], int *solver_sw, int *method_sw)
{
	char buf[BUFSIZE];	// buffer for fgets

	while(fgets(buf, sizeof( buf ), fp_i2) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %s", Yee) == 1) goto End_of_Yee;
	}
	fprintf(stderr,"No Yee-name is read @ input_file2!\n");
	exit(1);
	End_of_Yee:;
	{
		int recursive_depth;
		while(fgets(buf, sizeof( buf ), fp_i2) != NULL) { 
			if(sscanf(buf,"%*[^=] %*[=] %d", &recursive_depth) == 1) goto End_of_recursive_depth;
		}
		fprintf(stderr,"No recursive_depth is read @ input_file2!\n");
		exit(1);
		End_of_recursive_depth:;
	}

	{
		char A_length[BUFSIZE], B_temperature[BUFSIZE];
		double dummy_L, dummy_T;
		while(fgets(buf, sizeof( buf ), fp_i2) != NULL) { 
			if(sscanf(buf,"%*[^=] %*[=] %lf %s %*[^=] %*[=] %lf %s", 
				&dummy_L, A_length, &dummy_T, B_temperature) == 4) goto End_of_env_para;
		}
		fprintf(stderr,"No env_para is read @ input_file2!\n");
		exit(1);
		End_of_env_para:;
		dummy_L *= ScaleUnit(A_length);
		dummy_T = TempUnit(dummy_T, B_temperature);
		*Lambda = dummy_L;
	}

	while(fgets(buf, sizeof( buf ), fp_i2) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %d %*[^=] %*[=] %d", &Buv[0], &Buv[1]) == 2) goto End_of_BoundaryCondition;
	}
	fprintf(stderr,"No boundary condition is input_file2!\n");
	exit(1);
	End_of_BoundaryCondition:;
	
	while(fgets(buf, sizeof( buf ), fp_i2) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %d", solver_sw) == 1) goto End_of_SolverSW;
	}
	fprintf(stderr,"No solver SW is input_file2!\n");
	exit(1);
	End_of_SolverSW:;
	if(*solver_sw == 0) fprintf(stderr,"\n# Eigenproblem solver: LAPACK dgeev (default)\n");
	if(*solver_sw != 0) fprintf(stderr,"\n# Eigenproblem solver: lgeev_ (long double-precision)\n");

	while(fgets(buf, sizeof( buf ), fp_i2) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %d", method_sw) == 1) goto End_of_MethodSW;
	}
	fprintf(stderr,"No method SW is input_file2!\n");
	exit(1);
	End_of_MethodSW:;
	if(*method_sw == 1) fprintf(stderr,"\n# Inelastic-scattering formulation solver\n");
	if(*method_sw != 0) fprintf(stderr,"\n# Scattering matrix solver (default)\n");
}
double ScaleUnit(char x[])
{
	double scale = -1.;
	if (strncmp(x, "m", 1) == 0) scale = 1e-0;// order of m, mm, micron is important!
	if (strncmp(x, "mm", 2) == 0) scale = 1e-3;// order of m, mm, micron is important!
	if (strncmp(x, "micron", 6) == 0) scale = 1e-6;// order of m, mm, micron is important!
	
	if (strncmp(x, "km", 2) == 0) scale = 1e3;
	if (strncmp(x, "cm", 2) == 0) scale = 1e-2;
	
	if (strncmp(x, "um", 2) == 0) scale = 1e-6;
	if (strncmp(x, "nm", 2) == 0) scale = 1e-9;
	if (strncmp(x, "deg", 3) == 0) scale = Pi/180.;
//	if (strncmp(x, "degree", 6) == 0) scale = Pi/180.;
	if (strncmp(x, "rad", 3) == 0) scale = 1.;
//	if (strncmp(x, "radian",6) == 0) scale = 1.;
	if (scale < 0.) {
		fprintf(stderr,"ScaleUnit error!  x = %s\n", x);
		exit(1);
	}
	return(scale);
}
double TempUnit(double x, char s_char[])
{
	double offset_temp = -1.;
	double scale = -1.;
	if (strncmp(s_char, "K", 1) == 0) {
		scale = 1e-0;
		offset_temp = 0.;
	}// Kelvin
	if (strncmp(s_char, "degC", 4) == 0 || strncmp(s_char, "deg.C", 5) == 0 || strncmp(s_char, "deg_C", 5) == 0) {
		scale = 1.;
		offset_temp = 273.15;
	}// Celsius
	if (strncmp(s_char, "degF", 4) == 0 || strncmp(s_char, "deg.F", 5) == 0 || strncmp(s_char, "deg_F", 5) == 0) {
		scale = 5./9.;
		offset_temp = 459.67;
	}// Fahrenheit
	if (scale < 0. || offset_temp < -0.1) {
		fprintf(stderr,"TempUnit error!: %s\n", s_char);
		exit(1);
	}
	return(scale*(x + offset_temp));
}
