/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                      EPS_out                                    ||||||||||||
||||||||||||                            Last updated on Dec 25, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <tgmath.h>
#include "constant.h"

void Plot_color(FILE *fp_o, const long square_size, const double dl, const long Ox, const long Oy, 
						const long l_max, const long m_max, const double x[], const double y[],  
						const double WE_v[], const double WE_u[], const double WE_w[], 
						const double scale_range[2], const long func_para[3]);
void Plot_ticks(FILE *fp_o, const long linewidth_unit, const long Ox, const long Oy, 
						const long ix_min, const long ix_max, const long iy_min, const long iy_max, 
						const long boundary_margin, const long char_margin, const char *X, const char *Y);
void Plot_boundary_line(FILE *fp_o, const long linewidth_unit, const long Ox, const long Oy, 
						const long ix_min, const long ix_max, const long iy_min, const long iy_max);

void EPS_out(const char Yee[], const long slice[3], const long L[3], const long S_axis, 
				const double x[], const double y[], 
				const double WE_v[], const double WE_u[], const double WE_w[], 
				const double scale_range[2], const long func_para[3], const long boundary_SW)
{
	if(S_axis < 0 || S_axis > 2){
		fprintf(stderr,"S_axis = %ld in EPS_out\n", S_axis);
		exit(1);
	}
	if(slice[S_axis] < 0 || slice[S_axis] >= L[S_axis]){
		fprintf(stderr,"slice[%ld] = %ld in EPS_out\n", S_axis, slice[S_axis]);
		exit(1);
	}
	double E_range[2] = {WE_v[0], WE_v[0]};
	for(long iv = 0 ; iv < L[(S_axis+2)%3] ; iv++) {
		for(long iu = 0 ; iu < L[(S_axis+1)%3] ; iu++) {
			double z0 = WE_v[iu + L[(S_axis+1)%3]*iv];
			if(E_range[1] < z0) E_range[1] = z0;
			if(E_range[0] > z0) E_range[0] = z0;
			z0 = WE_u[iu + L[(S_axis+1)%3]*iv];
			if(E_range[1] < z0) E_range[1] = z0;
			if(E_range[0] > z0) E_range[0] = z0;
			z0 = WE_w[iu + L[(S_axis+1)%3]*iv];
			if(E_range[1] < z0) E_range[1] = z0;
			if(E_range[0] > z0) E_range[0] = z0;
		}
	}
	char eps_file[BUFSIZE],X[BUFSIZE],Y[BUFSIZE];
	if(S_axis == 0) {
		sprintf(eps_file,"%s_Med_yz_l%ld.eps", Yee, slice[0]);
		sprintf(X, "Y");
		sprintf(Y, "Z");
	}
	if(S_axis == 1) {
		sprintf(eps_file,"%s_Med_zx_m%ld.eps", Yee, slice[1]);
		sprintf(X, "Z");
		sprintf(Y, "X");
	}
	if(S_axis == 2) {
		sprintf(eps_file,"%s_Med_xy_n%ld.eps", Yee, slice[2]);
		sprintf(X, "X");
		sprintf(Y, "Y");
	}
	
	FILE *fp_o;
	fp_o = fopen(eps_file,"w");

	double x_min = x[0];
	double y_min = y[0];
	long l_max = L[(S_axis+1)%3];
	long m_max = L[(S_axis+2)%3];
	double x_max = x[2*l_max];
	double y_max = y[2*m_max];
	double dl = fabs(x_max - x_min) + fabs(y_max - y_min);
	for(long i0 = 0 ; i0 < 2*l_max ; i0++){
		if(dl > fabs(x[i0+1]-x[i0])) dl = fabs(x[i0+1]-x[i0]);
	}
	for(long i0 = 0 ; i0 < 2*m_max ; i0++){
		if(dl > fabs(y[i0+1]-y[i0])) dl = fabs(y[i0+1]-y[i0]);
	}
	dl /= 7.;// 4/7 is nearly equal to 3^-1/2
	long square_size = 2;
	long boundary_margin = square_size*10;
	long char_margin = boundary_margin*4;
	
	long ix_min = (long) (x_min*((double) square_size)/dl);
	long ix_max = (long) (x_max*((double) square_size)/dl);
	long iy_min = (long) (y_min*((double) square_size)/dl);
	long iy_max = (long) (y_max*((double) square_size)/dl);
	
	long Ox = -(ix_min - boundary_margin - char_margin);
	long Oy = -(iy_min - boundary_margin - char_margin);

	double char_size = (double) boundary_margin;//16.5;
	long linewidth_unit = 1;

	double scale_range1[2] = {scale_range[0],scale_range[1]};
	if(scale_range[0] > scale_range[1]) {
		scale_range1[0] = E_range[0];
		scale_range1[1] = E_range[1];
	};
	//========================================== Plot title ============================================
	fprintf(fp_o,"%%!PS-Adobe-3.0 EPSF-3.0\n%%%%BoundingBox: %ld %ld %ld %ld\n", 
					Ox + ix_min - boundary_margin - char_margin, Oy + iy_min - boundary_margin - char_margin, 
					Ox + ix_max + boundary_margin + char_margin, Oy + iy_max + boundary_margin + char_margin);
	time_t timer_f = time(0);
	fprintf(fp_o,"%% Created on %s", ctime(&timer_f));
	fprintf(fp_o,"%% YeeSlice updated on Dec 21, 2014)\n");
	fprintf(fp_o,"gsave\n");
	fprintf(fp_o,"/Helvetica findfont %g scalefont setfont\n", char_size);
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n(|epsilon| on %s-%s plane, Max|epsilon| = %g, Min|epsilon| = %g, ", 
					Ox + ix_min, Oy + iy_max + 7*boundary_margin/2, X, Y, E_range[1], E_range[0]);
	fprintf(fp_o,"Scale_max = %g, Scale_min = %g) show\n", scale_range1[1], scale_range1[0]);
	fprintf(fp_o,"%ld %ld moveto\n(%smin = %g m, %smax = %g m, %smin = %g m, %smax = %g m) show\n", 
					Ox + ix_min, Oy + iy_max + 3*boundary_margin/2, X, x_min, X, x_max, Y, y_min, Y, y_max);
	//========================================== Title end  ============================================
	Plot_color(fp_o, square_size, dl, Ox, Oy, l_max, m_max, x, y, WE_v, WE_u, WE_w, scale_range1, func_para);
	Plot_ticks(fp_o, linewidth_unit, Ox, Oy, ix_min, ix_max, iy_min, iy_max, boundary_margin, char_margin, X, Y);
	if(boundary_SW != 0) {
		Plot_boundary_line(fp_o, linewidth_unit, Ox, Oy, ix_min, ix_max, iy_min, iy_max);
		fprintf(stderr,"Plot boundary line\n");
	}
	fprintf(fp_o,"showpage\n");
	fprintf(fp_o,"grestore\n");
	//==========================================  Plot end  =============================================
	fclose(fp_o);
}
double function_gnuplot(double x, long N);
void EPS_scale(const char Yee[], const long func_para[3])
{
	char eps_file[BUFSIZE];
	sprintf(eps_file,"%s_Med_scale.eps", Yee);

	FILE *fp_o;
	fp_o = fopen(eps_file,"w");

	long square_size = 2;
	long boundary_margin = square_size*10;
	long char_margin = boundary_margin*3;

	long ix_min = 0*square_size;
	long ix_max = 50*square_size;
	long iy_min = 0*square_size;
	long iy_max = 500*square_size;
	
	double char_size = 16.5;
	long linewidth_unit = 1;

	//========================================== Plot title ============================================
	fprintf(fp_o,"%%!PS-Adobe-3.0 EPSF-3.0\n%%%%BoundingBox: %ld %ld %ld %ld\n", 
					ix_min - boundary_margin                , iy_min - boundary_margin - char_margin, 
					ix_max + boundary_margin + 2*char_margin, iy_max + boundary_margin + char_margin);
	fprintf(fp_o,"gsave\n");
	fprintf(fp_o,"/Helvetica findfont %.1f scalefont setfont\n", char_size);
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n(Scale of |epsilon|-plot) show\n", 
					ix_min, iy_max + 5*boundary_margin/2);
	//========================================== Plot scale ============================================
	for(long KR = 0 ; KR < iy_max/square_size ; KR++) {
		long iy0 = iy_min + KR*square_size;
		long iy1 = iy0 + square_size;
		fprintf(fp_o,"\n%ld %ld moveto\n",  ix_min, iy0);
		fprintf(fp_o,"%ld %ld lineto\n",  ix_min, iy1);
		fprintf(fp_o,"%ld %ld lineto\n",  ix_max, iy1);
		fprintf(fp_o,"%ld %ld lineto\n",  ix_max, iy0);
		fprintf(fp_o,"closepath\n");
		double z = KR*square_size+(square_size/2);
		z /= iy_max - iy_min;
		fprintf(fp_o,"%g %g %g setrgbcolor\n", 
				function_gnuplot(z, func_para[0]), function_gnuplot(z, func_para[1]), function_gnuplot(z, func_para[2]));
		fprintf(fp_o,"fill\n");
	}
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n(Scale_min) show\n", 
					ix_max+boundary_margin, iy_min);
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n(Scale_max) show\n", 
					ix_max+boundary_margin, iy_max);
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
	fprintf(fp_o,"[] 0 setdash\n");//	fprintf(fp_o,"[5 3 100 3] 0 setdash\n");
	fprintf(fp_o,"2 setlinejoin\n");
	fprintf(fp_o,"\n%ld %ld moveto\n",  ix_min, iy_min);
	fprintf(fp_o,"%ld %ld lineto\n",    ix_min, iy_max);
	fprintf(fp_o,"%ld %ld lineto\n",    ix_max, iy_max);
	fprintf(fp_o,"%ld %ld lineto\n",    ix_max, iy_min);
	fprintf(fp_o,"closepath\n");
	fprintf(fp_o,"stroke\n");
	fprintf(fp_o,"showpage\n");
	fprintf(fp_o,"grestore\n");
	//==========================================  Plot end  =============================================
	fclose(fp_o);
}
void Plot_color(FILE *fp_o, const long square_size, const double dl, const long Ox, const long Oy, 
						const long l_max, const long m_max, const double x[], const double y[],  
						const double WE_v[], const double WE_u[], const double WE_w[], 
						const double scale_range[2], const long func_para[3]){

	for(long iv = 0 ; iv < m_max ; iv++){
		long iy0 = (long) (y[iv*2+0]*((double) square_size)/dl);
		long iy1 = (long) (y[iv*2+1]*((double) square_size)/dl);
		long iy2 = (long) (y[iv*2+2]*((double) square_size)/dl);
		
		long iy01 = iy1 - 4*(iy1-iy0)/7;
		long iy12 = iy1 + 4*(iy2-iy1)/7; 
		for(long iu = 0 ; iu < l_max ; iu++){
			long ix0 = (long) (x[iu*2+0]*((double) square_size)/dl);
			long ix1 = (long) (x[iu*2+1]*((double) square_size)/dl);
			long ix2 = (long) (x[iu*2+2]*((double) square_size)/dl);

			long ix01 = ix1 - 4*(ix1-ix0)/7;
			long ix12 = ix1 + 4*(ix2-ix1)/7;

// start revision on Dec 18, 2014.
			double zv0 = (WE_v[iu + (l_max+1)*iv] - scale_range[0])/(scale_range[1] - scale_range[0]);
//			double zv2 = (WE_v[iu + l_max*((iv+1)%m_max)] - scale_range[0])/(scale_range[1] - scale_range[0]);
			double zv2 = (WE_v[iu + (l_max+1)*(iv+1)] - scale_range[0])/(scale_range[1] - scale_range[0]);
			double zu0 = (WE_u[iu + (l_max+1)*iv] - scale_range[0])/(scale_range[1] - scale_range[0]);
//			double zu2 = (WE_u[(iu+1)%l_max + (l_max+1)*iv] - scale_range[0])/(scale_range[1] - scale_range[0]);
			double zu2 = (WE_u[iu+1 + (l_max+1)*iv] - scale_range[0])/(scale_range[1] - scale_range[0]);
			double zw1 = (WE_w[iu + (l_max+1)*iv] - scale_range[0])/(scale_range[1] - scale_range[0]);
// end revision on Dec 18, 2014.
			
			fprintf(fp_o,"%g %g %g setrgbcolor\n", 	function_gnuplot(zv0, func_para[0]), 
															function_gnuplot(zv0, func_para[1]), 
															function_gnuplot(zv0, func_para[2]));
			fprintf(fp_o,"%ld %ld moveto\n",  Ox + ix0, Oy + iy0);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix01, Oy + iy01);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix12, Oy + iy01);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix2, Oy + iy0);
			fprintf(fp_o,"closepath\n");
			fprintf(fp_o,"fill\n\n");

			fprintf(fp_o,"%g %g %g setrgbcolor\n", 	function_gnuplot(zu0, func_para[0]), 
															function_gnuplot(zu0, func_para[1]), 
															function_gnuplot(zu0, func_para[2]));
			fprintf(fp_o,"%ld %ld moveto\n",  Ox + ix0, Oy + iy0);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix0, Oy + iy2);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix01, Oy + iy12);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix01, Oy + iy01);
			fprintf(fp_o,"closepath\n");
			fprintf(fp_o,"fill\n\n");
			
			fprintf(fp_o,"%g %g %g setrgbcolor\n", 	function_gnuplot(zw1, func_para[0]), 
															function_gnuplot(zw1, func_para[1]), 
															function_gnuplot(zw1, func_para[2]));
			fprintf(fp_o,"%ld %ld moveto\n",  Ox + ix01, Oy + iy01);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix01, Oy + iy12);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix12, Oy + iy12);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix12, Oy + iy01);
			fprintf(fp_o,"closepath\n");
			fprintf(fp_o,"fill\n\n");
			
			fprintf(fp_o,"%g %g %g setrgbcolor\n", 	function_gnuplot(zv2, func_para[0]), 
															function_gnuplot(zv2, func_para[1]), 
															function_gnuplot(zv2, func_para[2]));
			fprintf(fp_o,"%ld %ld moveto\n",  Ox + ix0, Oy + iy2);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix2, Oy + iy2);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix12, Oy + iy12);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix01, Oy + iy12);
			fprintf(fp_o,"closepath\n");
			fprintf(fp_o,"fill\n\n");

			fprintf(fp_o,"%g %g %g setrgbcolor\n", 	function_gnuplot(zu2, func_para[0]), 
															function_gnuplot(zu2, func_para[1]), 
															function_gnuplot(zu2, func_para[2]));
//			fprintf(fp_o,"%.7f setgray\n", function_gnuplot(zv0, 3));
			fprintf(fp_o,"%ld %ld moveto\n",  Ox + ix12, Oy + iy01);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix12, Oy + iy12);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix2, Oy + iy2);
			fprintf(fp_o,"%ld %ld lineto\n",  Ox + ix2, Oy + iy0);
			fprintf(fp_o,"closepath\n");
			fprintf(fp_o,"fill\n\n");

		}
	}
}
double function_gnuplot(double x, long N)
{
	if(x > 1.) x = 1.;
	if(x < 0.) x = 0.;
	
	if(N < 0) {
		x = 1. - x;
		N = -N;
	}
	
	if(N > 36) {
		fprintf(stderr,"N error!: N = %ld\n", N);
		exit(1);
	}
	
	double y = 0.; //revised on Dec 25, 2014
	if(N == 0) {
		y = 0.;
		goto rgbf_end;
	}
	if(N == 1) {
		y = 0.5;
		goto rgbf_end;
	}
	if(N == 2) {
		y = 1.;
		goto rgbf_end;
	}
	if(N == 3) {
		y = x;
		goto rgbf_end;
	}
	if(N == 4) {
		y = x*x;
		goto rgbf_end;
	}
	if(N == 5) {
		y = x*x*x;
		goto rgbf_end;
	}
	if(N == 6) {
		y = x*x*x*x;
		goto rgbf_end;
	}
	if(N == 7) {
		y = sqrt(x);
		goto rgbf_end;
	}
	if(N == 8) {
		y = sqrt(sqrt(x));
		goto rgbf_end;
	}
	if(N == 9) {
		y = sin(x*0.5*Pi);
		goto rgbf_end;
	}
	if(N == 10) {
		y = cos(x*0.5*Pi);
		goto rgbf_end;
	}
	if(N == 11) {
		y = fabs(x-0.5);
		goto rgbf_end;
	}
	if(N == 12) {
		y = (2.*x-1.)*(2.*x-1.);
		goto rgbf_end;
	}
	if(N == 13) {
		y = sin(x*Pi);
		goto rgbf_end;
	}
	if(N == 14) {
		y = fabs(cos(x*Pi));
		goto rgbf_end;
	}
	if(N == 15) {
		y = sin(x*2.*Pi);
		if(y < 0.) y = 0.;
		goto rgbf_end;
	}
	if(N == 16) {
		y = cos(x*2.*Pi);
		if(y < 0.) y = 0.;
		goto rgbf_end;
	}
	if(N == 17) {
		y = fabs(sin(x*2.*Pi));
		goto rgbf_end;
	}
	if(N == 18) {
		y = fabs(cos(x*2.*Pi));
		goto rgbf_end;
	}
	if(N == 19) {
		y = fabs(sin(x*4.*Pi));
		goto rgbf_end;
	}
	if(N == 20) {
		y = fabs(cos(x*4.*Pi));
		goto rgbf_end;
	}
	if(N == 21) {
		y = 3.*x;
		if(y > 1.) y = 1.;
		if(y < 0.) y = 0.;
		goto rgbf_end;
	}
	if(N == 22) {
		y = 3.*x - 1.;
		if(y > 1.) y = 1.;
		if(y < 0.) y = 0.;
		goto rgbf_end;
	}
	if(N == 23) {
		y = 3.*x - 2.;
		if(y > 1.) y = 1.;
		if(y < 0.) y = 0.;
		goto rgbf_end;
	}
	if(N == 24) {
		y = fabs(3.*x - 1.);
		if(y > 1.) y = 1.;
		goto rgbf_end;
	}
	if(N == 25) {
		y = fabs(3.*x - 2.);
		if(y > 1.) y = 1.;
		goto rgbf_end;
	}
	if(N == 26) {
		y = 0.5*(3.*x - 1.);
		if(y < 0.) y = 0.;
		goto rgbf_end;
	}
	if(N == 27) {
		y = 0.5*(3.*x - 2.);
		if(y < 0.) y = 0.;
		goto rgbf_end;
	}
	if(N == 28) {
		y = fabs(0.5*(3.*x - 1.));
		goto rgbf_end;
	}
	if(N == 29) {
		y = fabs(0.5*(3.*x - 2.));
		goto rgbf_end;
	}
	if(N == 30) {
		y = x/0.32-0.78125;
		if(y > 1.) y = 1.;
		if(y < 0.) y = 0.;
		goto rgbf_end;
	}
	if(N == 31) {
		y = 2*x-0.84;
		if(y > 1.) y = 1.;
		if(y < 0.) y = 0.;
		goto rgbf_end;
	}
	if(N == 32) {//		y = 4x;1;-2x+1.84;x/0.08-11.5;
		if(x < 0.25) {
			y = 4.*x;
		}
		else {
			if(x < 0.42) {
				y = 1.;
			}
			else {
				if(x < 0.92) {
					y = -2.*x+1.84;
				}
				else {
					y = x/0.08 - 11.5;
				}
			}
		}
		goto rgbf_end;
	}
	if(N == 33) {
		y = fabs(2.*x-0.5);
		if(y > 1.) y = 1.;
		if(y < 0.) y = 0.;
		goto rgbf_end;
	}
	if(N == 34) {
		y = 2.*x;
		if(y > 1.) y = 1.;
		if(y < 0.) y = 0.;
		goto rgbf_end;
	}
	if(N == 35) {
		y = 2.*x - 0.5;
		if(y > 1.) y = 1.;
		if(y < 0.) y = 0.;
		goto rgbf_end;
	}
	if(N == 36) {
		y = 2.*x - 1.;
		if(y < 0.) y = 0.;
		goto rgbf_end;
	}
// gnuplot> show palette rgbformulae
//           * there are 37 available rgb color mapping formulae:
//              0: 0               1: 0.5             2: 1
//              3: x               4: x^2             5: x^3
//              6: x^4             7: sqrt(x)         8: sqrt(sqrt(x))
//              9: sin(90x)       10: cos(90x)       11: |x-0.5|
//             12: (2x-1)^2       13: sin(180x)      14: |cos(180x)|
//             15: sin(360x)      16: cos(360x)      17: |sin(360x)|
//             18: |cos(360x)|    19: |sin(720x)|    20: |cos(720x)|
//             21: 3x             22: 3x-1           23: 3x-2
//             24: |3x-1|         25: |3x-2|         26: (3x-1)/2
//             27: (3x-2)/2       28: |(3x-1)/2|     29: |(3x-2)/2|
//             30: x/0.32-0.78125 31: 2*x-0.84       32: 4x;1;-2x+1.84;x/0.08-11.5
//             33: |2*x - 0.5|    34: 2*x            35: 2*x - 0.5
//             36: 2*x - 1
//           * negative numbers mean inverted=negative colour component
//           * thus the ranges in `set pm3d rgbformulae' are -36..36               
	rgbf_end:;
	if( y > 1. || y < 0. ) {//	if( y > 1. + EPS || y < - EPS ) {
		fprintf(stderr,"y error!: y = %g\n", y);
		exit(1);
	}
	return y;
}
void Plot_ticks(FILE *fp_o, const long linewidth_unit, const long Ox, const long Oy, 
						const long ix_min, const long ix_max, const long iy_min, const long iy_max, 
						const long boundary_margin, const long char_margin, const char *X, const char *Y){
	//======================================== Plot Xmin tick ==========================================
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n( %smin) show\n", 
					Ox + ix_min, Oy + iy_min-7*boundary_margin/2, X);
	fprintf(fp_o,"newpath\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
	fprintf(fp_o,"%ld %ld moveto\n", Ox + ix_min, Oy + iy_min-boundary_margin-char_margin);
	fprintf(fp_o,"%ld %ld lineto\n", Ox + ix_min, Oy + iy_min);
	fprintf(fp_o,"stroke\n");
	//======================================== Plot Xmid tick ==========================================
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n( %smid) show\n", 
					Ox + (ix_min+ix_max)/2, Oy + iy_min-7*boundary_margin/2, X);
	fprintf(fp_o,"newpath\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
	fprintf(fp_o,"%ld %ld moveto\n", Ox + (ix_min+ix_max)/2, Oy + iy_min-boundary_margin-char_margin);
	fprintf(fp_o,"%ld %ld lineto\n", Ox + (ix_min+ix_max)/2, Oy + iy_min);
	fprintf(fp_o,"stroke\n");
	//======================================== Plot Xmax tick ==========================================
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n( %smax) show\n", 
					Ox + ix_max, Oy + iy_min-7*boundary_margin/2, X);
	fprintf(fp_o,"newpath\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
	fprintf(fp_o,"%ld %ld moveto\n", Ox + ix_max, Oy + iy_min-boundary_margin-char_margin);
	fprintf(fp_o,"%ld %ld lineto\n", Ox + ix_max, Oy + iy_min);
	fprintf(fp_o,"stroke\n");
	//======================================== Plot Ymin tick ==========================================
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n90 rotate\n( %smin) show\n-90 rotate\n", 
					Ox + ix_min-5*boundary_margin/2, Oy + iy_min, Y);
	fprintf(fp_o,"newpath\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
	fprintf(fp_o,"%ld %ld moveto\n", Ox + ix_min-boundary_margin-char_margin, Oy + iy_min);
	fprintf(fp_o,"%ld %ld lineto\n", Ox + ix_min,                             Oy + iy_min);
	fprintf(fp_o,"stroke\n");
	//======================================== Plot Ymid tick ==========================================
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n90 rotate\n( %smid) show\n-90 rotate\n", 
					Ox + ix_min-5*boundary_margin/2, Oy + (iy_min+iy_max)/2, Y);
	fprintf(fp_o,"newpath\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
	fprintf(fp_o,"%ld %ld moveto\n", Ox + ix_min-boundary_margin-char_margin, Oy + (iy_min+iy_max)/2);
	fprintf(fp_o,"%ld %ld lineto\n", Ox + ix_min,                             Oy + (iy_min+iy_max)/2);
	fprintf(fp_o,"stroke\n");
	//======================================== Plot Ymax tick ==========================================
	fprintf(fp_o,"0 setgray\n");
	fprintf(fp_o,"%ld %ld moveto\n90 rotate\n( %smax) show\n-90 rotate\n", 
					Ox + ix_min-5*boundary_margin/2, Oy + iy_max, Y);//revised on Dec 18, 2014
	fprintf(fp_o,"newpath\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit);
	fprintf(fp_o,"%ld %ld moveto\n", Ox + ix_min-boundary_margin-char_margin, Oy + iy_max);
	fprintf(fp_o,"%ld %ld lineto\n", Ox + ix_min,                             Oy + iy_max);
	fprintf(fp_o,"stroke\n");
}
void Plot_boundary_line(FILE *fp_o, const long linewidth_unit, const long Ox, const long Oy, 
						const long ix_min, const long ix_max, const long iy_min, const long iy_max){
	//====================================== Plot boundary line ========================================
	fprintf(fp_o,"0.5 setgray\n");
	fprintf(fp_o,"%ld setlinewidth\n", linewidth_unit*8);
	fprintf(fp_o,"[] 0 setdash\n");//	fprintf(fp_o,"[5 3 100 3] 0 setdash\n");
	fprintf(fp_o,"2 setlinejoin\n");
	fprintf(fp_o,"\n%ld %ld moveto\n",  Ox + ix_min, Oy + iy_min);
	fprintf(fp_o,"%ld %ld lineto\n",    Ox + ix_min, Oy + iy_max);
	fprintf(fp_o,"%ld %ld lineto\n",    Ox + ix_max, Oy + iy_max);
	fprintf(fp_o,"%ld %ld lineto\n",    Ox + ix_max, Oy + iy_min);
	fprintf(fp_o,"closepath\n");
	fprintf(fp_o,"stroke\n");
}

