/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                     This program is Main Program of YeeSlice.     Jun/14/2012   ||||||||||||
||||||||||||                Main and sub programs were coded by Tatsuya Usuki.               ||||||||||||
||||||||||||                            Program language is C99.                             ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                     Please let me know any bug that you find.                   ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                                         http://homepage2.nifty.com/Pass_path/   ||||||||||||
||||||||||||                                                                                 ||||||||||||
||||||||||||                                                  Last updated on Dec 26, 2014   ||||||||||||
||||||||||||---------------------------------------------------------------------------------||||||||||||
||||||||||||     References:                                                                 ||||||||||||
||||||||||||     1) "WaveguideModeNum_ja2.pdf"                                               ||||||||||||
||||||||||||     2) "WageguideModeNum_v6.pdf"                                                ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
#include <tgmath.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
//#include "constant.h"
#define BUFSIZE 1024 /* buffer for fgets */
#define Pi 3.141592653589793238462643 /* circular constant */
//#define Pi2 6.283185307179586476925286
//
//  main ---- input_file1 
//        |                 
//        |-- input_fileM
//        |                 
//         -- input_file3
//        |                 
//         -- output_file
//
void input_file1(FILE *fp_i1, long L[3]);
void input_fileM(FILE *fp_i1, const long L, double x[2*L+1]);
void input_file3(FILE *fp_i3, char Yee[], long slice[3], double scale_range[2], long func_para[3], long *boundary_SW);
void output_file(const char Yee[], const long slice[3], const long L[3], 
					const double x[], const double y[], const double z[], 
					const double scale_range[2], const long func_para[3], const long boundary_SW);
int main(int argc, char **argv)
{
	if(argc != 3) {
		fprintf(stderr,"error: number of files \n");
		exit(1);
	}
	time_t timer_f = time(0);
	fprintf(stderr,"# %s (updated on Dec 26, 2014) started on %s\n",argv[0], ctime(&timer_f));
	
	FILE *fp_i1;
	fp_i1 = fopen(argv[1],"r");
	if (fp_i1 == NULL){
		fprintf(stderr,"open error!: open input-file1!\n");
		exit(1);
	}

	long L[3]; //division number: Lu = L, Lv = M, Lw = N
	
	input_file1(fp_i1, L); // inputting parameters from an inputfile 1
	
	double *x;
	x = malloc(sizeof(double)*(L[0]*2+1));
	double *y;
	y = malloc(sizeof(double)*(L[1]*2+1));
	double *z;
	z = malloc(sizeof(double)*(L[2]*2+1));

	{
		input_fileM(fp_i1, L[0], x);
	
		input_fileM(fp_i1, L[1], y);
	
		input_fileM(fp_i1, L[2], z);
	}
	fclose(fp_i1);

	for(long J = 0 ; J < 3 ; J++) { 
		if (J == 0) fprintf(stderr,"\nL, x0, x1\n");
		if (J == 1) fprintf(stderr,"\nM, y0, y1\n");
		if (J == 2) fprintf(stderr,"\nN, z0, z1\n");
		for(long K = 0 ; K < L[J] ; K++) { 
			if (J == 0) fprintf(stderr,"%ld, %.5E, %.5E\n", K, x[2*K+0], x[2*K+1]);
			if (J == 1) fprintf(stderr,"%ld, %.5E, %.5E\n", K, y[2*K+0], y[2*K+1]);
			if (J == 2) fprintf(stderr,"%ld, %.5E, %.5E\n", K, z[2*K+0], z[2*K+1]);
		}
	}
	
	char Yee[BUFSIZE];
	long slice[3];
	FILE *fp_i2;
	fp_i2 = fopen(argv[2],"r");
	if (fp_i2 == NULL){
		fprintf(stderr,"open error!: open input-file3!\n");
		exit(1);
	}
	double scale_range[2] = {0., 15.};
	long func_para[3] = {30,31,32};// 30,31,32 ...color printable on gray (black-blue-violet-yellow-white)
	long boundary_SW = 0;
	input_file3(fp_i2, Yee, slice, scale_range, func_para, &boundary_SW);
	fclose(fp_i2);
	
	output_file(Yee, slice, L, x, y, z, scale_range, func_para, boundary_SW);
	free(x);
	free(y);
	free(z);
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                              input_file1, input_fileM                           ||||||||||||
||||||||||||                            Last updated on Oct 27, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void input_file1(FILE *fp_i, long L[3])
{
	char buf[BUFSIZE];	// buffer for fgets
	while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %ld %*[^=] %*[=] %ld %*[^=] %*[=] %ld", &L[0], &L[1], &L[2]) == 3) goto End_of_LMN;
	}
	fprintf(stderr,"No L,M,N are read @ input_file1!\n");
	exit(1);
	End_of_LMN:;

	if(L[0] < 1 || L[1] < 1 || L[2] < 1) {
		fprintf(stderr,"L,M,N input error!: L = %ld, M = %ld, N = %ld \n", L[0], L[1], L[2]);
		exit(1);
	}
}
void input_fileM(FILE *fp_i, const long L, double x[2*L+1])
{
	char buf[BUFSIZE];	// buffer for fgets
	long i0 = 0;
	long i1;
	while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
		double dxdu0, dxdu1;
		if(sscanf(buf,"%ld %*[,] %lf  %*[,] %lf %*[,] %lf %*[,] %lf", 
			&i1, &x[i0*2+0], &x[i0*2+1], &dxdu0, &dxdu1) == 5){
			if(i0 == i1) i0++;
			if(i0 == L) goto End_of_x_data;
		}
	}
	fprintf(stderr,"Full %ld-data were not read @ input_fileM!\n",L);
	exit(1);
	End_of_x_data:;

	while(fgets(buf, sizeof( buf ), fp_i) != NULL) { 
		if(sscanf(buf,"%ld %*[,] %lf", &i1, &x[2*L]) == 2){
			if(i1 == L) goto End_of_x_max;
		}
	}
	fprintf(stderr,"The %ld-th data were not read @ input_fileM!\n",L+1);
	exit(1);
	End_of_x_max:;
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||||||||||||                                    input_file3                                  ||||||||||||
||||||||||||                            Last updated on Nov 11, 2014                         ||||||||||||
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void input_file3(FILE *fp_i3, char Yee[], long slice[3], double scale_range[2], long func_para[3], long *boundary_SW)
{
	char buf[BUFSIZE];	// buffer for fgets

	while(fgets(buf, sizeof( buf ), fp_i3) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %s", Yee) == 1) goto End_of_Yee;
	}
	fprintf(stderr,"No Yee-name is read @ input_file3!\n");
	exit(1);
	End_of_Yee:;

	while(fgets(buf, sizeof( buf ), fp_i3) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %ld %*[^=] %*[=] %ld %*[^=] %*[=] %ld", 
					&slice[0], &slice[1], &slice[2]) == 3) goto End_of_SlicePoint;
	}
	fprintf(stderr,"No division_depth is read @ input_file3!\n");
	exit(1);
	End_of_SlicePoint:;
	
	if(slice[0] < 0 || slice[1] < 0 || slice[2] < 0) {
		fprintf(stderr,"mesh input error!: mesh[0]= %ld, mesh[1]= %ld, mesh[2]= %ld \n", slice[0], slice[1], slice[2]);
		exit(1);
	}

	while(fgets(buf, sizeof( buf ), fp_i3) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %lf %*[^=] %*[=] %lf", 
					&scale_range[0], &scale_range[1]) == 2) goto End_of_scale;
	}
	fprintf(stderr,"No scale_range is read @ input_file3!\n");
	exit(1);
	End_of_scale:;

	while(fgets(buf, sizeof( buf ), fp_i3) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %ld %*[^=] %*[=] %ld %*[^=] %*[=] %ld", 
					&func_para[0], &func_para[1], &func_para[2]) == 3) goto End_of_color;
	}
	fprintf(stderr,"No color-parameter is read @ input_file3!\n");
	exit(1);
	End_of_color:;
	// 3,3,3    ...gray (black-white)
	// 7,5,15   ...traditional pm3d (black-blue-red-yellow)
	// 3,11,6   ...green-red-violet
	// 23,28,3  ...ocean (green-blue-white);try also all other permutations
	// 21,22,23 ...hot (black-red-yellow-white)
	// 30,31,32 ...color printable on gray (black-blue-violet-yellow-white)
	// 33,13,10 ...rainbow (blue-green-yellow-red)
	// 34,35,36 ...AFM hot (black-red-yellow-white)

	while(fgets(buf, sizeof( buf ), fp_i3) != NULL) { 
		if(sscanf(buf,"%*[^=] %*[=] %ld", 
					boundary_SW) == 1) goto End_of_boundary;
	}
	fprintf(stderr,"No boundary_SW is read @ input_file3!\n");
	exit(1);
	End_of_boundary:;
	fprintf(stderr,"boundary_SW = %ld @ input_file3\n", *boundary_SW);
}

