/*    This program operates M_EH and M_HE Aug/06/2012          */
/*        Main and sub programs were coded by Tatsuya Usuki.   */
/*         Programming language is C99.                        */
/*        See document file "Wageguide_Simulation_v2.pdf"      */
/*                            It was modified in Oct/29/2014   */
#include <tgmath.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

void Mat_p_Mat(const long L[4], const double complex A[L[0]*L[1]], const double complex B[L[0]*L[1]], double complex C[L[0]*L[1]], const long C_shift);
void Mat_m_Mat(const long L[4], const double complex A[L[0]*L[1]], const double complex B[L[0]*L[1]], double complex C[L[0]*L[1]], const long C_shift);
void Delta_u(const long L[4], const double f[(L[0]+L[1])*2], const double complex MatA[], const long A_shift, double complex MatB[L[0]*L[1]]);
void Delta_v(const long L[4], const double g[(L[0]+L[1])*2], const double complex MatA[], const long A_shift, double complex MatB[L[0]*L[1]]);
void Delta_uT(const long L[4], const double f[(L[0]+L[1])*2], const double complex MatA[], const long A_shift, double complex MatB[L[0]*L[1]]);
void Delta_vT(const long L[4], const double g[(L[0]+L[1])*2], const double complex MatA[], const long A_shift, double complex MatB[L[0]*L[1]]);
void epmu_cal(const long L[4], const double complex epmu[6*L[0]*L[1]], const long epmu_shift, const double complex MatA[], const long A_shift, double complex MatB[L[0]*L[1]]);

void M_EH(const long L[4], const double complex epmu[6*L[0]*L[1]], const double fg[(L[0]+L[1])*2], const double complex Psi_i[L[0]*L[1]*4], const long Pshift_i, double complex Psi_o[L[0]*L[1]*4], const long Pshift_o)
{
// This 'M_EH' gives an operation for 2LM*2LM matrix M_EH in Eq. (9).
// L[4]             : L[0] = L, L[1] = M, L[2] = B_u and L[3] = B_v.
// epmu[6*L[0]*L[1]]: epmu[l + L[0]*m + 0*L[0]*L[1]] = epsi_u(l,m), epmu[l + L[0]*m + 1*L[0]*L[1]] = epsi_v(l,m), epmu[l + L[0]*m + 2*L[0]*L[1]] = 1/epsi_w(l,m),
//                    epmu[l + L[0]*m + 3*L[0]*L[1]] =   mu_u(l,m), epmu[l + L[0]*m + 4*L[0]*L[1]] =   mu_v(l,m), epmu[l + L[0]*m + 5*L[0]*L[1]] =   1/mu_w(l,m).
// fg[(L[0]+L[1])*2]: fg[2*l + j] = f_j(l),  fg[2*(L[0]+m) + j] = g_j(m), where j = 0 or 1.
// Psi_i[], Pshift_i: 2LM*1 column vector H_uv[j] = Psi_i[j + Pshift_i*2*L*M] where 0 <= j < 2*L*M.
// Psi_o[], Pshift_o: 2LM*1 column vector E_vu[j] = Psi_o[j + Pshift_o*2*L*M] where 0 <= j < 2*L*M.
//
	double complex *MatA;
	MatA = malloc(sizeof(double complex)*L[0]*L[1]);
	double complex *Mat_x0;
	Mat_x0 = malloc(sizeof(double complex)*L[0]*L[1]);
	double complex *Mat_x1;
	Mat_x1 = malloc(sizeof(double complex)*L[0]*L[1]);
	
	Delta_v(L, fg, Psi_i, Pshift_i*2+0, MatA); //           MatA = Delta_v * Hu
	epmu_cal(L, epmu, 2, MatA, 0, Mat_x0); // Mat_x0 = (1/epsi_w) * MatA     = (1/epsi_w) * Delta_v * Hu
	
	Delta_u(L, fg, Psi_i, Pshift_i*2+1, MatA);      //      MatA = Delta_u * Hv
	epmu_cal(L, epmu, 2, MatA, 0, Mat_x1); // Mat_x1 = (1/epsi_w) * MatA     = (1/epsi_w) * Delta_u * Hv
		
	double complex *MatB;
	MatB = malloc(sizeof(double complex)*L[0]*L[1]);
	double complex *MatC;
	MatC = malloc(sizeof(double complex)*L[0]*L[1]);
		
	Delta_vT(L, fg, Mat_x0, 0, MatA); //      MatA = Delta_vT * Mat_x0   = Delta_vT * (1/epsi_w) * Delta_v * Hu
	epmu_cal(L, epmu, 3, Psi_i, Pshift_i*2+0, MatB); // Mat_B = mu_u * Hu
	Mat_m_Mat(L, MatB, MatA, MatC, 0); //      MatC = MatB - MatA           = mu_u * Hu - Delta_vT * (1/epsi_w) * Delta_v * Hu = (mu_u - Delta_vT * (1/epsi_w) * Delta_v) * Hu
	
	Delta_vT(L, fg, Mat_x1, 0, MatA); //      MatA = Delta_vT * Mat_x1   = Delta_vT * (1/epsi_w) * Delta_u * Hv
	
	Mat_p_Mat(L, MatA, MatC, Psi_o, Pshift_o*2+0); //      mEv = MatA + MatC         = Delta_vT * (1/epsi_w) * Delta_u * Hv + (1 - Delta_vT * (1/epsi_w) * Delta_v) * Hu
		
		
	Delta_uT(L, fg, Mat_x1, 0, MatA); //      MatA = Delta_uT * Mat_x1   = Delta_uT * (1/epsi_w) * Delta_u * Hv
	epmu_cal(L, epmu, 4, Psi_i, Pshift_i*2+1, MatB); // MatB = mu_v * Hv
	Mat_m_Mat(L, MatB, MatA, MatC, 0); //      MatC = MatB - MatA           = mu_v * Hv - Delta_uT * (1/epsi_w) * Delta_u * Hv = (mu_v - Delta_uT * (1/epsi_w) * Delta_u) * Hv
		
	Delta_uT(L, fg, Mat_x0, 0, MatA); //      MatA = Delta_uT * Mat_x0   = Delta_uT * (1/epsi_w) * Delta_v * Hu
		
	Mat_p_Mat(L, MatA, MatC, Psi_o, Pshift_o*2+1); //        Eu = MatA + MatC         = Delta_uT * (1/epsi_w) * Delta_v * Hu + (mu_v - Delta_uT * (1/epsi_w) * Delta_u) * Hv
	free(Mat_x0);
	free(Mat_x1);
	free(MatA);
	free(MatB);
	free(MatC);
}
void M_HE(const long L[4], const double complex epmu[6*L[0]*L[1]], const double fg[(L[0]+L[1])*2], const double complex Psi_i[L[0]*L[1]*4], const long Pshift_i, double complex Psi_o[L[0]*L[1]*4], const long Pshift_o)
{
// This 'M_HE' gives an operation for 2LM*2LM matrix M_EH in Eq. (9).
// L[4]             : L[0] = L, L[1] = M, L[2] = B_u and L[3] = B_v.
// epmu[6*L[0]*L[1]]: epmu[l + L[0]*m + 0*L[0]*L[1]] = epsi_u(l,m), epmu[l + L[0]*m + 1*L[0]*L[1]] = epsi_v(l,m), epmu[l + L[0]*m + 2*L[0]*L[1]] = 1/epsi_w(l,m),
//                    epmu[l + L[0]*m + 3*L[0]*L[1]] =   mu_u(l,m), epmu[l + L[0]*m + 4*L[0]*L[1]] =   mu_v(l,m), epmu[l + L[0]*m + 5*L[0]*L[1]] =   1/mu_w(l,m).
// fg[(L[0]+L[1])*2]: fg[2*l + j] = f_j(l),  fg[2*(L[0]+m) + j] = g_j(m), where j = 0 or 1.
// Psi_i[], Pshift_i: 2LM*1 column vector H_uv[j] = Psi_i[j + Pshift_i*2*L*M] where 0 <= j < 2*L*M.
// Psi_o[], Pshift_o: 2LM*1 column vector E_vu[j] = Psi_o[j + Pshift_o*2*L*M] where 0 <= j < 2*L*M.
//
	double complex *MatA;
	MatA = malloc(sizeof(double complex)*L[0]*L[1]);
	double complex *Mat_x0;
	Mat_x0 = malloc(sizeof(double complex)*L[0]*L[1]);
	double complex *Mat_x1;
	Mat_x1 = malloc(sizeof(double complex)*L[0]*L[1]);
	
	Delta_uT(L, fg, Psi_i, Pshift_i*2+0, MatA); //           MatA = Delta_uT * mEv
	epmu_cal(L, epmu, 5, MatA, 0, Mat_x0); // Mat_x0 = (1/mu_w) * MatA     = (1/mu_w) * Delta_uT * mEv
	
	Delta_vT(L, fg, Psi_i, Pshift_i*2+1, MatA);      //      MatA = Delta_vT * Eu
	epmu_cal(L, epmu, 5, MatA, 0, Mat_x1); // Mat_x1 = (1/mu_w) * MatA     = (1/mu_w) * Delta_vT * Eu
	
	double complex *MatB;
	MatB = malloc(sizeof(double complex)*L[0]*L[1]);
	double complex *MatC;
	MatC = malloc(sizeof(double complex)*L[0]*L[1]);
	
	Delta_u(L, fg, Mat_x0, 0, MatA); //      MatA = Delta_u * Mat_x0   = Delta_u * (1/mu_w) * Delta_uT * mEv
	epmu_cal(L, epmu, 1, Psi_i, Pshift_i*2+0, MatB); // Mat_B = epsi_v * mEv
	Mat_m_Mat(L, MatB, MatA, MatC, 0); //      MatC = MatB - MatA           = epsi_v * mEv - Delta_u * (1/mu_w) * Delta_uT * mEv = (epsi_v - Delta_u * (1/mu_w) * Delta_uT) * mEv
	
	Delta_u(L, fg, Mat_x1, 0, MatA); //      MatA = Delta_u * Mat_x1   = Delta_u * (1/mu_w) * Delta_vT * Eu
	
	Mat_m_Mat(L, MatC, MatA, Psi_o, Pshift_o*2+0); //      Hu = MatC - MatA          = (epsi_v - Delta_u * (1/mu_w) * Delta_uT) * mEv - Delta_u * (1/mu_w) * Delta_vT * Eu
		
		
	Delta_v(L, fg, Mat_x1, 0, MatA); //      MatA = Delta_v * Mat_x1   = Delta_v * (1/mu_w) * Delta_vT * Eu
	epmu_cal(L, epmu, 0, Psi_i, Pshift_i*2+1, MatB); // MatB = epsi_u * Eu
	Mat_m_Mat(L, MatB, MatA, MatC, 0); //      MatC = MatB - MatA           = epsi_u * Eu - Delta_v * (1/mu_w) * Delta_vT * Eu = (epsi_u - Delta_v * (1/mu_w) * Delta_vT) * Eu
		
	Delta_v(L, fg, Mat_x0, 0, MatA); //      MatA = Delta_v * Mat_x0   = Delta_v * (1/mu_w) * Delta_uT * mEv
		
	Mat_m_Mat(L, MatC, MatA, Psi_o, Pshift_o*2+1); //        Hv = MatC - MatA        = (epsi_u - Delta_v * (1/mu_w) * Delta_vT) * Eu - Delta_v * (1/mu_w) * Delta_uT * mEv
	free(Mat_x0);
	free(Mat_x1);
	free(MatA);
	free(MatB);
	free(MatC);
}
void Mat_p_Mat(const long L[4], const double complex A[L[0]*L[1]], const double complex B[L[0]*L[1]], double complex C[L[0]*L[1]], const long C_shift)
{
// C[], C_shift: LM*1 column vector C[j + C_shift*L*M] where 0 <= j < L*M.
	long LM_C_shift = C_shift*L[0]*L[1];
	for(long J0 = 0 ; J0 < L[0]*L[1] ; J0++) { 
		C[J0 + LM_C_shift] = A[J0] + B[J0];
	}
}
void Mat_m_Mat(const long L[4], const double complex A[L[0]*L[1]], const double complex B[L[0]*L[1]], double complex C[L[0]*L[1]], const long C_shift)
{
// C[], C_shift: LM*1 column vector C[j + C_shift*L*M] where 0 <= j < L*M.
	long LM_C_shift = C_shift*L[0]*L[1];
	for(long J0 = 0 ; J0 < L[0]*L[1] ; J0++) { 
		C[J0 + LM_C_shift] = A[J0] - B[J0];
	}
}
void Delta_u(const long L[4], const double f[(L[0]+L[1])*2], const double complex MatA[], const long A_shift, double complex MatB[L[0]*L[1]])
{
// MatA[], A_shift: LM*1 column vector MatA[j + A_shift*L*M] where 0 <= j < L*M.
	long LM_A_shift = A_shift*L[0]*L[1];
	for(long i1 = 0 ; i1 < L[1] ; i1++){
		for(long i0 = 0 ; i0 < L[0]-1 ; i0++){
			MatB[i0+i1*L[0]] = f[2*i0+1]*(f[2*(i0+1)+0]*MatA[(i0+1)+i1*L[0] + LM_A_shift] - f[2*i0+0]*MatA[i0+i1*L[0] + LM_A_shift]);
		}
	}
	for(long i1 = 0 ; i1 < L[1] ; i1++){
		MatB[(L[0]-1)+i1*L[0]] = f[2*(L[0]-1)+1]*(((double) L[2])*f[2*0+0]*MatA[0+i1*L[0] + LM_A_shift] - f[2*(L[0]-1)+0]*MatA[(L[0]-1)+i1*L[0] + LM_A_shift]);
	}
}
void Delta_v(const long L[4], const double g[(L[0]+L[1])*2], const double complex MatA[], const long A_shift, double complex MatB[L[0]*L[1]])
{
// MatA[], A_shift: LM*1 column vector MatA[j + A_shift*L*M] where 0 <= j < L*M.
	long LM_A_shift = A_shift*L[0]*L[1];
	for(long i1 = 0 ; i1 < L[1]-1 ; i1++){
		for(long i0 = 0 ; i0 < L[0] ; i0++){
			MatB[i0+i1*L[0]] = g[2*(L[0]+i1)+1]*(g[2*(L[0]+i1+1)+0]*MatA[i0+(i1+1)*L[0] + LM_A_shift] - g[2*(L[0]+i1)+0]*MatA[i0+i1*L[0] + LM_A_shift]);
		}
	}
	for(long i0 = 0 ; i0 < L[0] ; i0++){
			MatB[i0+(L[1]-1)*L[0]] = g[2*(L[0]+L[1]-1)+1]*(((double) L[3])*g[2*(L[0]+0)+0]*MatA[i0+0*L[0] + LM_A_shift] - g[2*(L[0]+L[1]-1)+0]*MatA[i0+(L[1]-1)*L[0] + LM_A_shift]);
	}
}
void Delta_uT(const long L[4], const double f[(L[0]+L[1])*2], const double complex MatA[], const long A_shift, double complex MatB[L[0]*L[1]])
{
// MatA[], A_shift: LM*1 column vector MatA[j + A_shift*L*M] where 0 <= j < L*M.
	long LM_A_shift = A_shift*L[0]*L[1];
	for(long i1 = 0 ; i1 < L[1] ; i1++){
		MatB[0+i1*L[0]] = f[2*0+0]*(((double) L[2])*f[2*(L[0]-1)+1]*MatA[L[0]-1 + i1*L[0] + LM_A_shift] - f[2*0+1]*MatA[0 + i1*L[0] + LM_A_shift]);
	}
	for(long i1 = 0 ; i1 < L[1] ; i1++){
		for(long i0 = 1 ; i0 < L[0] ; i0++){
			MatB[i0+i1*L[0]] = f[2*i0+0]*(f[2*(i0-1)+1]*MatA[i0-1 + i1*L[0] + LM_A_shift] - f[2*i0+1]*MatA[i0 + i1*L[0] + LM_A_shift]);
		}
	}
}
void Delta_vT(const long L[4], const double g[(L[0]+L[1])*2], const double complex MatA[], const long A_shift, double complex MatB[L[0]*L[1]])
{
// MatA[], A_shift: LM*1 column vector MatA[j + A_shift*L*M] where 0 <= j < L*M.
	long LM_A_shift = A_shift*L[0]*L[1];
	for(long i0 = 0 ; i0 < L[0] ; i0++){
		MatB[i0+0*L[0]] = g[2*(L[0]+0)+0]*(((double) L[3])*g[2*(L[0]+L[1]-1)+1]*MatA[i0+(L[1]-1)*L[0] + LM_A_shift] - g[2*(L[0]+0)+1]*MatA[i0+0*L[0] + LM_A_shift]);
	}
	for(long i1 = 1 ; i1 < L[1] ; i1++){
		for(long i0 = 0 ; i0 < L[0] ; i0++){
			MatB[i0+i1*L[0]] = g[2*(L[0]+i1)+0]*(g[2*(L[0]+i1-1)+1]*MatA[i0+(i1-1)*L[0] + LM_A_shift] - g[2*(L[0]+i1)+1]*MatA[i0+i1*L[0] + LM_A_shift]);
		}
	}
}
void epmu_cal(const long L[4], const double complex epmu[6*L[0]*L[1]], const long epmu_shift, const double complex MatA[], const long A_shift, double complex MatB[L[0]*L[1]])
{
// MatA[], A_shift: LM*1 column vector MatA[j + A_shift*L*M] where 0 <= j < L*M.
	long LM_A_shift = A_shift*L[0]*L[1];
// epmu[6*L[0]*L[1]]: epmu[l + L[0]*m + 0*L[0]*L[1]] = epsi_u(l,m), epmu[l + L[0]*m + 1*L[0]*L[1]] = epsi_v(l,m), epmu[l + L[0]*m + 2*L[0]*L[1]] = 1/epsi_w(l,m),
//                    epmu[l + L[0]*m + 3*L[0]*L[1]] =   mu_u(l,m), epmu[l + L[0]*m + 4*L[0]*L[1]] =   mu_v(l,m), epmu[l + L[0]*m + 5*L[0]*L[1]] =   1/mu_w(l,m).
	long LM_epmu_shift = epmu_shift*L[0]*L[1];
	for(long i0 = 0 ; i0 < L[0]*L[1] ; i0++){
		MatB[i0] = MatA[i0 + LM_A_shift]*epmu[i0 + LM_epmu_shift];
	}
	
}
